/*
 * Copyright (c) 1993 Parallax Graphics Inc.
 */

#pragma ident "@(#)tvfour_c3.c	1.6 Copyright (c) 1993 Parallax Graphics Inc"

#include <sys/conf.h>
#include <sys/param.h>
#include "tvfour.h"
#define DELAX(x) DELAY(delay_factor*(x))
int	delay_factor = 2;
#define DELAY1 1
#define DELAY5 5
#define DELAY10 10

int
c3_start_dma(tvfour_softc *softc, caddr_t addr, struct buf *bp);

void
c3_int_svc(tvfour_softc *softc);

void
xvbuf_int_svc(tvfour_softc *softc);

static void
c3_clear_timeout(tvfour_softc *softc);

static void
c3_set_timeout(tvfour_softc *softc, long ticks);

static void
c3_do_timeout(tvfour_softc *softc);

static void
c3_error(tvfour_softc *softc, char *message);

void
c3_load_dct(tvfour_softc *softc);

static int
c3_wait_for_vnac(tvfour_softc *softc);

#define C3_TIMEOUT (HZ/15)



/*
 * The following is a read from the C3, which is a compression operation.
 * DMA transfer count must be a multiple of 4.
 */

int
c3_start_dma(tvfour_softc *softc, caddr_t addr, struct buf *bp)
{
	register u_int start_address = ((u_int)addr) >> 2;
	register u_int end_address = ((u_int)(addr + bp->b_bcount-4)) >> 2;

	if (bp->b_flags & B_READ) {

		/* Set initial state */

		PCIV_WREG(softc, DMASA1.l, start_address);	
		PCIV_WREG(softc, DMASA2.l, start_address);
		PCIV_WREG(softc, DMAEA1.l, end_address);
		PCIV_WREG(softc, DMAEA2.l, end_address);

		/*
		 * Purge FIFO
		 */
		PCIV_WREG(softc, CCube.l, XVC3_START_DMA_TO_HOST_1);
			DELAX(1);
		PCIV_WREG(softc, CCube.l, XVC3_START_DMA_TO_HOST_2);
		DELAX(1);
		PCIV_WREG(softc, CCube.l, XVC3_PURGE_BIC_GO_PIO);
		DELAX(10);
		PCIV_WREG(softc, CCube.l, XVC3_RESET_TO_PIO);
		DELAX(5);

		PCIV_WREG(softc, C3Stall.b.cond, STALL_H);

		/* load the DMA buffers the same */

		PCIV_WREG(softc, DMASA1.l, start_address);	
		PCIV_WREG(softc, DMASA2.l, start_address);
		PCIV_WREG(softc, DMAEA1.l, end_address);
		PCIV_WREG(softc, DMAEA2.l, end_address);

		/* start the compressor */

		C3_WREG(softc, HV_Enable, C3_ON);
		C3_WREG(softc, HV_Enable, C3_OFF);
		C3_WREG(softc, HV_Enable, C3_ON);
		C3_WREG(softc, Start, C3_ON);

		if (c3_wait_for_vnac(softc) != DDI_SUCCESS) {
			return (DDI_FAILURE);
		}

		PCIV_WREG(softc, C3Stall.b.cond, STALL_H_O_FE);

		C3_WREG(softc, NMRQ_Enable, C3_VInactive);

		/* Start the DMA going */

		C3_WREG(softc, DRQ_Enable,
			C3_Codec_RegBusy | C3_FIFO_QFull);

		PCIV_WREG(softc, CCube.l, XVC3_START_DMA_TO_HOST_1);
		DELAX(1);
		PCIV_WREG(softc, CCube.l, XVC3_START_DMA_TO_HOST_2); 
		PCIV_WREG(softc, IntMask.l, XV_INT_CCUBE);
		PCIV_WREG(softc, BIntMask.l, XV_BINT_DMA); 
		softc->state = STATER1; 
		} 
		else 
		{	/* Write */ 
		/* * The following is a write of an image to the C3.
		 * It assumes that the specified buffer contains the
		 * complete image and that a "~0" terminates the buffer.
		 *
		 * A write to the C3 is a decompression operation.
		 * The write returns once all data has been transferred
		 * to the C3; however, the video pipeline may still be busy.
		 */

		/* Set initial state */
		PCIV_WREG(softc, CCube.l, XVC3_RESET_TO_PIO);
		PCIV_WREG(softc, C3Stall.b.cond, STALL_ALWAYS);

		/* Load the DMA buffer address */

		PCIV_WREG(softc, DMASA1.l, start_address);	
		PCIV_WREG(softc, DMASA2.l, start_address);
		PCIV_WREG(softc, DMAEA1.l, end_address+4);
		PCIV_WREG(softc, DMAEA2.l, end_address+4);

		/*
		 * DRQ until FIFO is 3/4 full, then interrupt.  DRQs are
		 * disabled and an interrupt is generated if a marker code is
		 * found. 
		 */

		C3_WREG(softc, NMRQ_Enable, C3_FIFO_3QFull);
		C3_WREG(softc, DRQ_Enable,
			C3_Codec_RegBusy | C3_FIFO_Not3QFull);

		/* 
		 * Enable interrupts in the XVideo hardware 
                 *
                 *   If you remove the State_Trace_Add(0) between 
                 *   XVC3_START_DMA_FROM_HOST_1 and XVC3_START_DMA_FROM HOST2
                 *   below you need to add some kind of delay in its place!!
		 */
		PCIV_WREG(softc, IntMask.l, XV_INT_CCUBE);
		PCIV_WREG(softc, BIntMask.l, XV_BINT_DMA);

		PCIV_WREG(softc, CCube.l, XVC3_START_DMA_FROM_HOST_1);
		PCIV_WREG(softc, CCube.l, XVC3_START_DMA_FROM_HOST_2);

		softc->state = STATEW1;
	}

	c3_set_timeout(softc, C3_TIMEOUT);

	return (DDI_SUCCESS);
}


void
xvbuf_int_svc(tvfour_softc *softc)
{
	if (softc->state == STATE_ERR)
		return;

	C3_WREG(softc, NMRQ_Enable, C3_OFF);
	C3_WREG(softc, DRQ_Enable, C3_OFF);

	switch (softc->state) {

	case STATEW1: {
		/*
		 * all the data has been read from the bus...now start the
		 * decompressor and wait for the video pipe to go inactive.
		 */
		C3_WREG(softc, HV_Enable, C3_ON);
		C3_WREG(softc, HV_Enable, C3_OFF);
		C3_WREG(softc, HV_Enable, C3_ON);
		C3_WREG(softc, Start, C3_ON);

		PCIV_WREG(softc, C3Stall.b.cond, STALL_NEVER);

		if (c3_wait_for_vnac(softc) != DDI_SUCCESS) {
			DEBUGF(1,(CE_CONT, "xv_buf_int_svc error"));
			softc->state = STATE_ERR;
			break;
		}

		C3_WREG(softc, NMRQ_Enable, C3_VInactive);
		PCIV_WREG(softc, C3Stall.b.cond, STALL_FE);

		softc->state = STATEW3;
		break;
		}

	case STATEW2: {

		unsigned int timers, timernow;

		/*
		 * all the data has been read from the bus... now wait for
		 * the video pipe to go inactive.
		 */
		c3_clear_timeout(softc);

		PCIV_WREG(softc, C3Stall.b.cond, STALL_NEVER);

		/*
		 * XXX an attempt to timeout on the Flags register XXX
		 */
		timers = PCIV_RREG(softc, TmrCnt.l) + 60;
		timernow = PCIV_RREG(softc, TmrCnt.l);

		while ((timernow < timers) &&
			     (!(C3_RREG(softc, Flags) & C3_VInactive))) {
			timernow = PCIV_RREG(softc, TmrCnt.l);
			if((timers < 60) && (timernow > 10000))
			    timernow = 0;
			DELAX(5);
		}

		C3_WREG(softc, HV_Enable, C3_ON);
		C3_WREG(softc, HV_Enable, C3_OFF);
		C3_WREG(softc, HV_Enable, C3_ON);
		C3_WREG(softc, Start, C3_OFF);

		softc->state = STATE0;
                softc->bp->b_resid=0; /* Address read back on WRITE not working */
		break;
		}

	default: {
		/* 
		 * the buffer must have been overrun...flag the error
		 */
		c3_error(softc ,"Buffer overrun on read");
		softc->state  = STATE_ERR;
		break;
		}
	}

	if ((softc->state == STATE_ERR)) {

		C3_WREG(softc, DRQ_Enable, C3_OFF);
		C3_WREG(softc, NMRQ_Enable, C3_OFF);
		PCIV_WREG(softc, CCube.l, XVC3_RESET_TO_PIO);

		c3_clear_timeout(softc);

	} else {

		PCIV_WREG(softc, IntMask.l, XV_INT_CCUBE);
		if (softc->state & STATEW)
			PCIV_WREG(softc, BIntMask.l, XV_BINT_DMA);

	}
}


void
c3_int_svc(tvfour_softc *softc)
{
	unsigned int c3_status;
	register unsigned int imask;

	if (softc->state == STATE_ERR)
		return;

        PCIV_WREG(softc, C3Stall.b.cond, STALL_ALWAYS);

	c3_status = C3_RREG(softc, Flags);
	C3_WREG(softc, NMRQ_Enable, C3_OFF);
	C3_WREG(softc, DRQ_Enable, C3_OFF);

	c3_clear_timeout(softc);
	c3_set_timeout(softc, C3_TIMEOUT);

	/*
	 * Process the interrupt depending on which state we're in.
	 */
	switch (softc->state) {

	case STATEW1: {

		if (c3_status & C3_FIFO_3QFull) {
			/*
			 * The FIFO is now 3/4 full.  Reset the DRQ and IRQ
			 * conditions: DRQ while FIFO is not 3/4 full and 
			 * interrupt when end of buffer has been reached.
			 */

			C3_WREG(softc, NMRQ_Enable, C3_FIFO_NotQFull);
			C3_WREG(softc, DRQ_Enable,
				C3_Codec_RegBusy | C3_FIFO_Not3QFull);
			/*
			 * Start the decompressor.
			 */
			C3_WREG(softc, HV_Enable, C3_ON);
			C3_WREG(softc, HV_Enable, C3_OFF);
			C3_WREG(softc, HV_Enable, C3_ON);
			C3_WREG(softc, Start, C3_ON);

			PCIV_WREG(softc, C3Stall.b.cond, STALL_NH);

			softc->state = STATEW2;
		} else {
			c3_error(softc, "Bad Interrupt");
			softc->state = STATE_ERR;
		}
		break;
		}

	case STATEW2: {
		/*
		* End of buffer has been reached. or the fifo is less than 1
		* quarter full.
		*/
		if(c3_status & C3_FIFO_NotQFull) {
			 printf("int not q full\n");
			 /*
			 * DRQ until FIFO is 3/4 full, then interrupt.
			 * DRQs are disabled and an interrupt is generated
			 * if a marker code is found.
			 */
			 C3_WREG(softc, NMRQ_Enable, C3_FIFO_3QFull);
			 C3_WREG(softc, DRQ_Enable,
				C3_Codec_RegBusy | C3_FIFO_Not3QFull);

			softc->state = STATEW1;
		} else {
			/*
			 * If we get here instead of processing this
			 * state in xvbuf_int_svc(), its an error. 
			 */
			c3_error(softc, "Bad Interrupt");
			softc->state = STATE_ERR;
		}
		break;
		}

	case STATEW3: {
		/*
		 * The video pipe has gone inactive.  We need to tell the
		 * decompressor to stop so that it doesn't expect more
		 * frames.
		 */
		C3_WREG(softc, HV_Enable, C3_ON);
		C3_WREG(softc, HV_Enable, C3_OFF);
		C3_WREG(softc, HV_Enable, C3_ON);
		C3_WREG(softc, Start, C3_OFF);

		softc->state = STATE0;
		break;
		}

	case STATER1: {
		/* 
		 * The video pipeline is now stopped.  Now tell the C3
		 * to stop and drain the FIFO.
		 */
		if (c3_status & C3_VInactive) {
			/* save this point as end of data, lib will fix up */
#ifdef DIAGS
			cmn_err(CE_WARN," STATER1: b_bcount=%d  DMARB=%d\n",
				softc->bp->b_bcount,softc->xvregs->r.DMARB.l);
#endif
			softc->bp->b_resid = softc->bp->b_bcount -
			(PCIV_RREG(softc, DMARB.l)-(u_int)softc->addr + 4);
			C3_WREG(softc, HV_Enable, C3_ON);
			C3_WREG(softc, HV_Enable, C3_OFF);
			C3_WREG(softc, HV_Enable, C3_ON);
			C3_WREG(softc, Start, C3_OFF);

			C3_WREG(softc, DRQ_Enable,
				C3_Codec_RegBusy | C3_FIFO_NotEmpty |
				C3_FIFO_Empty);
			C3_WREG(softc, NMRQ_Enable, C3_FIFO_Empty);

			softc->state = STATER2;
		} else {
			c3_error(softc, "Bad Interrupt");
			softc->state = STATE_ERR;
		}
		break;
		}
	case STATER2: {
		/*
		 * The FIFO is now empty.  Pump the Codec one more time
		 * to get the last word.  Gather stats and exit.
	 	 */
		if (c3_status & C3_FIFO_Late) {
			c3_error(softc, "Overrun Error");
			softc->state = STATE_ERR;
		} else if (c3_status & C3_FIFO_Empty) {
			/*
			 * We're done "get_last_word";
			 */
			PCIV_WREG(softc, CCube.l, XVC3_CONT_DMA_TO_HOST_1);
			DELAX(1);
			PCIV_WREG(softc, CCube.l, XVC3_CONT_DMA_TO_HOST_2);
			DELAX(10);

			PCIV_WREG(softc, CCube.l,XVC3_PURGE_C3_CONT_DMA);

			DELAX(10);
			PCIV_WREG(softc, CCube.l,XVC3_PAUSE_DMA_TO_HOST);

			DELAX(10);
			PCIV_WREG(softc, CCube.l,XVC3_PURGE_BIC_GO_PIO);
			DELAX(1);
			PCIV_WREG(softc, CCube.l,XVC3_PURGE_BIC_GO_PIO);
			DELAX(10);
			PCIV_WREG(softc, CCube.l,XVC3_RESET_TO_PIO);
			DELAX(10);
			C3_WREG(softc, DRQ_Enable,C3_OFF);

			softc->state = STATE0; 

		} else {
			c3_error(softc, "Bad Interrupt");
			softc->state = STATE_ERR;
		}
		break;
		}
	default: {
		c3_error(softc,"Bad c3int state");
		softc->state = STATE_ERR;
		break;
		}
	}

	/* 
 	 * IF c3_state = STATE0, we're done, so clean up and get out.  
	 * Otherwise, enable interrupts and restart DMA.
	 */
	if ((softc->state == STATE0) || (softc->state == STATE_ERR)) {

		C3_WREG(softc, DRQ_Enable, C3_OFF);
		C3_WREG(softc, NMRQ_Enable, C3_OFF);

		PCIV_WREG(softc, CCube.l, XVC3_RESET_TO_PIO);
		c3_clear_timeout(softc);

	} else {
		
#if 0
		imask=PCIV_RSDW(softc,IntMask.l);
		PCIV_WREG(softc, IntMask.l, (imask| XV_INT_CCUBE));
#endif
		PCIV_WREG(softc, IntMask.l, XV_INT_CCUBE);
		if (softc->state != STATEW3)
			PCIV_WREG(softc, BIntMask.l, XV_BINT_DMA);

	}
}

static void
c3_error(tvfour_softc *softc, char *message)
{
	cmn_err(CE_CONT, "%s in state 0x%x:  ", message, softc->state);
	cmn_err(CE_CONT, "C3_STATUS = 0x%x  XV_STATUS = 0x%x\n",
		C3_RREG(softc, Flags),
		PCIV_RREG(softc, IntStat.l));

	C3_WREG(softc, Flags, C3_OFF);
	C3_WREG(softc, Flags, C3_ON);
	C3_WREG(softc, Flags, C3_OFF);

}

static void
c3_do_timeout(tvfour_softc *softc)
{
	DEBUGF(1,(CE_CONT, "c3_do_timeout%d\n", ddi_get_instance(softc->devi)));
	cmn_err(CE_NOTE,"  bmask=0x%x\n",PCIV_RSDW(softc,BIntMask.l));
	cmn_err(CE_NOTE,"  imask=0x%x\n",PCIV_RSDW(softc,IntMask.l));

	/*
	 * Reset ???
	 */
	tvfour_pause_dma(softc);

	C3_WREG(softc, DRQ_Enable, C3_OFF);
	C3_WREG(softc, NMRQ_Enable, C3_OFF);

	/*
	 * Cancel the dma request if there was one outstanding.
	 */
	softc->state = STATE_ERR;
	softc->timeout = 0;
	cv_broadcast(&softc->dmadone);
}

static void
c3_set_timeout(tvfour_softc *softc, long ticks)
{
	if (softc->timeout) {
		DEBUGF(1,(CE_CONT, "timeout already set\n"));
		c3_clear_timeout(softc);
		return;
	}

	softc->timeout = ticks;
	softc->tid = timeout(c3_do_timeout, (caddr_t)softc, softc->timeout);
}

static void
c3_clear_timeout(tvfour_softc *softc)
{
	if (softc->timeout) {
		untimeout(softc->tid);
		softc->timeout = 0;
	}
}

void
c3_init(tvfour_softc *softc)
{
	PCIV_WREG(softc, C3Stall.l, 0x140);
	C3_WREG(softc, S_reset, C3_ON);
	c3_load_dct(softc);
}

static u_int c3_DCT_table[] = {
	0x00005A82,
	0x00007FFF,
	0x000030FC,
	0x00007642,
	0x00005A82,
	0x00007FFF,
	0x000030FC,
	0x00007642
};
	
void
c3_load_dct(tvfour_softc *softc)
{
	int i;

	for(i=0; i < (sizeof(c3_DCT_table)/sizeof(u_int)); i++)
		C3_WREG(softc, DCT[i], c3_DCT_table[i]);
}
static int
c3_wait_for_vnac(tvfour_softc *softc)
{
	/*
	 * Wait for the video to go active and enable interrupts.
	 * Instead of hanging the machine if something goes wrong
	 * we fail the request.
	 */

	int cnt = 0;
	while(C3_RREG(softc, Flags) & C3_VInactive) {
		DELAX(5);
		if (cnt++ > 200) {
			cmn_err(CE_WARN,
				"tvfour: VInactive never went low (0x%x)\n",
				C3_RREG(softc, Flags));
			return (DDI_FAILURE);
		}
	}

	return (DDI_SUCCESS);
}

c3_purge_fifo(tvfour_softc *softc)
{
    softc->dmactl->control0= DMAFLUSH;
    PCIV_WREG(softc,CCube.l,XVC3_START_DMA_TO_HOST_1);
    PCIV_WREG(softc,CCube.l,XVC3_START_DMA_TO_HOST_2);
    DELAY(DELAY1);
    PCIV_WREG(softc,CCube.l, XVC3_PURGE_BIC_GO_PIO);
    DELAY(DELAY10);
    PCIV_WREG(softc,CCube.l, XVC3_RESET_TO_PIO);
    DELAY(DELAY5);
    softc->dmactl->control0= DMACLEAR;
}

