/*
 * Copyright (c) 1993 Parallax Graphics Inc.
 */

#pragma ident "@(#)tvfour_svc.c	1.6 Copyright (c) 1993 Parallax Graphics Inc"

/*
 * device driver template
 */

#include <sys/types.h>
#include <sys/param.h>
#include <sys/cred.h>
#include <sys/errno.h>
#include <sys/cmn_err.h>
#include <sys/open.h>
#include <sys/file.h>
#include <sys/kmem.h>
#include <sys/t_lock.h>
#include <sys/stat.h>
#include <sys/conf.h>
#include <sys/ddi.h>
#include <sys/sunddi.h>

#include "tvfour.h"

#ifdef RTV_SUPPORT
extern void ccube_svc_dma(), ccube_svc_c3(),ccube_svc_v_chan(),ccube_svc_w_chan();
#endif
void
tvfour_service(tvfour_softc *softc, void (*[])(), u_int bits);

void
tvfour_continue_dma(tvfour_softc *softc);

void
tvfour_pause_dma(tvfour_softc *softc);

static void
tvfour_svc_dma(tvfour_softc *softc);

static void
tvfour_svc_timer(tvfour_softc *softc);

static void
tvfour_svc_disp(tvfour_softc *softc);

static void
tvfour_svc_c3(tvfour_softc *softc);

static void
tvfour_svc_ext(tvfour_softc *softc);

static void
tvfour_svc_video(tvfour_softc *softc);

static void
tvfour_svc_w_chan(tvfour_softc *softc);

static void
tvfour_svc_blit(tvfour_softc *softc);

static void
tvfour_null(tvfour_softc *softc);

static void
tvfour_svc_sbus(tvfour_softc *softc);

void (*bint_services[])() = {
	tvfour_svc_dma,
	tvfour_svc_timer,
	tvfour_null,
	tvfour_svc_sbus
};

void (*int_services[])() = {
	tvfour_svc_disp,		/* disp */
	tvfour_svc_c3,		/* c3 */
	tvfour_svc_ext,		/* ext */
	tvfour_svc_blit,		/* blit */
        tvfour_svc_video,    /* vsync */
        tvfour_svc_video,    /* wsync */
        tvfour_svc_video,    /* verr */
        tvfour_svc_video,    /* werr */
	tvfour_null,		/* vodd */
	tvfour_null,		/* wodd */
	tvfour_null,		/* vnsync */
	tvfour_null		/* wnsync */
};

void
tvfour_service(tvfour_softc *softc, void (*services[])(), u_int bits)
{
	while(bits) {
		if (bits & 0x1) {
			(*services)(softc);
		}
		bits = bits >> 1;
		services++;
	}
}

void
tvfour_continue_dma(tvfour_softc *softc)
{
	if(softc->state > 0) {
		if(softc->state & STATEW) {
			PCIV_WREG(softc, CCube.l, XVC3_CONT_DMA_FROM_HOST_1);
			PCIV_WREG(softc, CCube.l, XVC3_CONT_DMA_FROM_HOST_2);
		} else {
			PCIV_WREG(softc, CCube.l, XVC3_CONT_DMA_TO_HOST_1);
			PCIV_WREG(softc, CCube.l, XVC3_CONT_DMA_TO_HOST_2);
		}
	}
}


void
tvfour_pause_dma(tvfour_softc *softc)
{
	if (softc->state != 0) {
		if (softc->state & STATEW) {
			PCIV_WREG(softc, CCube.l, XVC3_PAUSE_DMA_FROM_HOST);
			DELAY(10);
			PCIV_WREG(softc, CCube.l, XVC3_WRITE_GO_PIO);
			PCIV_WREG(softc, CCube.l, XVC3_WRITE_GO_PIO);
		} else {
			PCIV_WREG(softc, CCube.l, XVC3_PAUSE_DMA_TO_HOST);
			DELAY(500);
			PCIV_WREG(softc, CCube.l, XVC3_PURGE_BIC_GO_PIO);
			PCIV_WREG(softc, CCube.l, XVC3_PURGE_BIC_GO_PIO);
		}
		DELAY(100);
	}
}

static void
tvfour_svc_dma(tvfour_softc *softc)
{
	if ((softc->state == STATE0) || (softc->state == STATE_ERR))
		return;

	xvbuf_int_svc(softc);

}
static void
tvfour_svc_sbus(tvfour_softc *softc)
{

}

static void
tvfour_svc_timer(tvfour_softc *softc)
{

	/*plxv_set_hourint();*/
	/*plxv_inc_hours();*/
}

static void
tvfour_svc_disp(tvfour_softc *softc)
{
	tvfour_pll_alignment( softc );
}

static void
tvfour_svc_c3(tvfour_softc *softc)
{
	c3_int_svc(softc);

}

static void
tvfour_svc_ext(tvfour_softc *softc)
{
}
disable_intr(tvfour_softc *softc)
{
	PCIV_WREG(softc, IntMask.l, 0);
}
static void
tvfour_svc_video(tvfour_softc *softc)
{
	int n;
}
static void
tvfour_svc_w_chan(tvfour_softc *softc)
{
}

static void
tvfour_svc_blit(tvfour_softc *softc)
{
}

static void
tvfour_null(tvfour_softc *softc)
{
}

/*---------------------------------------------------------------------------*
 |"tvfour_pll_alignment"
 |
 |  Check that the Power Video 700 frame buffer is properly synchronized
 |
 |    = 1 if pll in alignment, 0 if adjusting alignment
 *---------------------------------------------------------------------------*/
int tvfour_pll_alignment( softc )
	tvfour_softc*	softc;		/* Context for this device instance. */
{
	int         plx_v;      /* Video synchronization information. */
	int         vlines;     /* Vertical lines on screen. */


#ifdef  REVA
	plx_v= softc->xvregs->r.Bt.blu.reg0;
#else
	plx_v= BYTE_REV(softc->miscmode->pll_status);
#endif
	vlines = softc->nvram.main.ysize - 1;

	/*  FRAME BUFFER IS WAY OUT OF SYNCHRONIZATION WITH HOST */
	 
	if (0==(plx_v&0x80)) 
	{
		PCIV_WREG(softc,VDisp.l,vlines-64);
		softc->pll_minor_adjust=0;
		softc->pll_good_count=0;
	} 


    /*  FRAME BUFFER IS MORE THAN 32 LINES OUT OF SYNCHRONIZATION WITH HOST */
	 
	else if (0==(plx_v&0x40)) 
	{
		PCIV_WREG(softc,VDisp.l,vlines-8);
		softc->pll_good_count=0;
		softc->pll_minor_adjust=0;
	} 


    /*  FRAME BUFFER IS LESS THAN 32 LINES OUT OF SYNCHRONIZATION WITH HOST */

	else if (((plx_v)&0xf)>=2) 
	{
		if (softc->pll_minor_adjust&1)     
		{
		    PCIV_WREG(softc,VDisp.l,vlines-1);
		}
		else 
		{
		    PCIV_WREG(softc,VDisp.l,vlines);
		}
		softc->pll_minor_adjust++;

	} 


    /*  FRAME BUFFER IS IN SYNCHRONIZATION WITH HOST FRAME BUFFER */
	 
	else /* if ( softc->pll_good_count < 10 ) #### */
	{
		PCIV_WREG(softc,VDisp.l,vlines);
		softc->pll_minor_adjust=0;
		softc->pll_good_count++;
	}

	return (softc->pll_good_count >= 7) ? 1 : 0;
}
u_int tvfour_intr(caddr_t i_handler_arg)
{
	register tvfour_softc *softc = (tvfour_softc *) i_handler_arg;
	register int serviced = DDI_INTR_UNCLAIMED;
	register u_int ints, bints;
	register int cm,bits;
	register int imask;
	register int mutex_ok;
	register int trace=0;
	extern void (*bint_services[])();
	extern void (*int_services[])();

	if (softc == NULL) {
		return (DDI_INTR_UNCLAIMED);
	}
	 mutex_enter(&softc->interlock);
	/*
	 * Turn off DMA so we can talk to the board.
	 */
	 tvfour_pause_dma(softc);
	/*
	 * We must & in the interrupt mask bits because
	 * the masked (disabled) bits may be set even though
	 * they did not generate an interrupt.
	 */
	 softc->istat = softc->xvregs->r.IntStatC.l;
	 softc->bstat = softc->xvregs->r.BIntStatC.l;

	 bints = PCIV_RSDW(softc, BIntMask.l) & softc->bstat;
	 ints = PCIV_RSDW(softc, IntMask.l) & softc->istat;

	/*
	 * Service the BIC interrupts.
	 */

	 if (bints)
	     tvfour_service(softc, bint_services, bints);
	/*
	 * Service the CTC interrupts.
	 */
	 if (ints) 
	     tvfour_service(softc, int_services, ints);

	 mutex_exit(&softc->interlock);
	 if ((softc->state == STATE0) || (softc->state == STATE_ERR)) {
		cv_broadcast(&softc->dmadone);
	 }
	 return (DDI_INTR_CLAIMED);
}

