/*
 * (c) Copyright 1991 Parallax Graphics Inc, All Rights Reserved
 *
 * Parallax Graphics, Inc.
 * 2500 Condensa Street
 * Santa Clara, California 95051
 *
 * $Log:	c3init.c,v $
 * Revision 1.1  95/12/08  10:26:57  10:26:57  dist ()
 * Initial revision
 * 
 * Revision 1.3  1995/05/25  17:08:09  dist
 * Andy Goodrich - Combined HP/AIX version for HP 9.1 code freeze.
 *
 * Revision 1.2  95/04/17  12:27:54  12:27:54  dist ()
 * Andy Goodrich - added RCS header.
 *
 */
#ifndef lint
static char *rcsid = "@(#)$Id: c3init.c,v 1.3 1995/05/25 17:08:09 dist Exp dist $ Parallax Graphics Inc";
#endif

#include <stdio.h>
#ifdef AIX
#include <fcntl.h>
#else
#include <sys/fcntl.h>
#endif


#include <math.h>
#include <signal.h>
#ifdef sun4
#include <sys/asynch.h>
#endif
#include <unistd.h>
#include <sys/types.h>

#include "c3.h"
#include "hw_context.h"
#include "plxv_xpoint.h"
#include "videoformat.h"

extern C3_WREGS	*c3_wreg;
extern C3_RREGS	*c3_rreg;

#define RGB422 1 

int	no_ccube;
int loopsperusec = 8;

#ifndef DRIVER

/*************************************************/
/*         INIT the 550 to normal mode           */
/*************************************************/

C3Init(p_me)
	XV_struct* p_me;
{
		volatile unsigned int i;
		unsigned int j, k, l, m;
		float usecs, usecspertick;

		if ( !p_me->main.c3_rev > 0 )
		{
			no_ccube = 1;
			return;
		}
		else
		{
			no_ccube = 0;
		}
#ifdef NOTDEF
        c3_wreg = (C3_WREGS *) xv_c3base;
        c3_rreg = (C3_RREGS *) xv_c3base;
#endif /* NOTDEF */

        C3_WREG(p_me,S_reset, ~0);

	/* figure out the delay loop value */

		usecspertick = 1000000 / 90000;
		k = 10000;
		do
		{
			k *= 10;
			i = k;
			PV_RREG(p_me, m, TmrCnt.l);
			while(--i > 0);
			PV_RREG(p_me, l, TmrCnt.l);
			j = l - m;
		} while (j < 1000);		/* must get at least 1000 ticks */

		usecs = (float) j * usecspertick;
		loopsperusec = (int) ((float) k / usecs);

	/* Load DCT coefficients */

		C3_WREG(p_me,DCT[0],0x00005A82);    
		DELAY(DELAY10);
		C3_WREG(p_me,DCT[1],0x00007FFF);
		DELAY(DELAY10);
		C3_WREG(p_me,DCT[2],0x000030FC);
		DELAY(DELAY10);
		C3_WREG(p_me,DCT[3],0x00007642);
		DELAY(DELAY10);
		C3_WREG(p_me,DCT[4],0x00005A82);
		DELAY(DELAY10);
		C3_WREG(p_me,DCT[5],0x00007FFF);
		DELAY(DELAY10);
		C3_WREG(p_me,DCT[6],0x000030FC);
		DELAY(DELAY10);
		C3_WREG(p_me,DCT[7],0x00007642);
		DELAY(DELAY10);
		C3_WREG(p_me,DCT[9],0x00000000);

		PV_WREG(p_me, C3Stall.l, 0x140);
}

/*************************************************/
/*         INIT the 550 to real time mode        */
/*************************************************/

C3InitRtv(p_me)
XV_struct* p_me;
{
        int fd;

		if ( p_me->main.c3_rev > 0 )
		{
			no_ccube = 1;
			return;
		}
		else
		{
			no_ccube = 0;
		}

#ifdef NOTDEF
        c3_wreg = (C3_WREGS *) xv_c3base;
        c3_rreg = (C3_RREGS *) xv_c3base;
#endif /* NOTDEF */

	C3_WREG(p_me,S_reset, ~0);

	/* Load DCT coefficients */
	C3_WREG(p_me,DCT[0],0x00005A82);
	C3_WREG(p_me,DCT[1],0x00007FFF);
	C3_WREG(p_me,DCT[2],0x000030FC);
	C3_WREG(p_me,DCT[3],0x00007642);
	C3_WREG(p_me,DCT[4],0x00005A82);
	C3_WREG(p_me,DCT[5],0x00007FFF);
	C3_WREG(p_me,DCT[6],0x000030FC);
	C3_WREG(p_me,DCT[7],0x00007642);
	C3_WREG(p_me,DCT[9],0x00000000);
	PV_WREG(p_me, C3Stall.l, 0x140);

	if((fd = open("/dev/tvtwo0", (O_RDWR | OPEN_RTV_BIT) )) < 0) {
		perror("/dev/tvtwo0");  
		exit(-1);
	}

        return(fd);
}

#endif /* NOT DRIVER */

/*****************************************************************************
 * Prepare the CL550 for compression.  This is called by C3GetImage.
 *****************************************************************************/

C3InitCompress(p_me) 
    XV_struct* p_me;
{
	char s[20];
	int i1,i2;


	if ( no_ccube ) return;

	C3_WREG(p_me,S_reset, ~0);
	DELAY(DELAY10);

	/* Load configuration registers */

#ifdef RGB422
	C3_WREG(p_me,Config, 0x38);
	DELAY(DELAY10);
#else
	C3_WREG(p_me,Config, 0x28); 
	DELAY(DELAY10);
#endif
	C3_WREG(p_me,DRQ_Enable,0);
	DELAY(DELAY10);
	C3_WREG(p_me,NMRQ_Enable,0); 
	DELAY(DELAY10);

#ifdef RGB422
	C3_WREG(p_me,Init_1, 0x003c);
	DELAY(DELAY10);
	C3_WREG(p_me,Init_2, 0x003d);
	DELAY(DELAY10);
	C3_WREG(p_me,Init_3, 0x007b);
	DELAY(DELAY10);
	C3_WREG(p_me,Init_4, 0x00f1);
	DELAY(DELAY10);
	C3_WREG(p_me,Init_5, 0x000a);
	DELAY(DELAY10);
	C3_WREG(p_me,Init_6, 0x0000);
	DELAY(DELAY10);
	C3_WREG(p_me,Init_7, 0x0017);
	DELAY(DELAY10);
	C3_WREG(p_me,QuantSync, 0x0400);
	DELAY(DELAY10);
#else
	C3_WREG(p_me,Init_1, 0x0042);
	DELAY(DELAY10);
	C3_WREG(p_me,Init_2, 0x0001);
	DELAY(DELAY10);
	C3_WREG(p_me,Init_3, 0x0081);
	DELAY(DELAY10);
	C3_WREG(p_me,Init_4, 0x00f7);
	DELAY(DELAY10);
	C3_WREG(p_me,Init_5, 0x0000);
	DELAY(DELAY10);
	C3_WREG(p_me,Init_6, 0x0000);
	DELAY(DELAY10);
	C3_WREG(p_me,Init_7, 0x001d);
	DELAY(DELAY10);
	C3_WREG(p_me,QuantSync, 0x0406);
#endif
	C3_WREG(p_me,QuantYC, 0x2099);
	DELAY(DELAY10);
	C3_WREG(p_me,QuantAB, 0x0000);
	DELAY(DELAY10);
	C3_WREG(p_me,QuantSelect, 1);
	DELAY(DELAY10);
#ifdef RGB422
	C3_WREG(p_me,VideoLatency, 0x0185);
	DELAY(DELAY10);
#else
	C3_WREG(p_me,VideoLatency, 0x017f);
	DELAY(DELAY10);
#endif
	/* Load coder configuration registers */

	C3_WREG(p_me,CodecSeq, 0x00cc);
	DELAY(DELAY10);
	C3_WREG(p_me,CodecCompH, 0x0088);
	DELAY(DELAY10);
	C3_WREG(p_me,CodecCompL, 0x0044);
	DELAY(DELAY10);
	C3_WREG(p_me,CoderAttr, 0x00004);
	DELAY(DELAY10);
	C3_WREG(p_me,CodingIntH, 0x0000);
	DELAY(DELAY10);
#ifdef RGB422
	C3_WREG(p_me,CodingIntL, 0x0002);
	DELAY(DELAY10);
#else
	C3_WREG(p_me,CodingIntL, 0x001);
	DELAY(DELAY10);
#endif
	C3_WREG(p_me,DecLength, 0x0008);
	DELAY(DELAY10);
	C3_WREG(p_me,CodeOrder, 0x0001);
	DELAY(DELAY10);

	/* Load RGB-YUV conversion matrix */

	C3_WREG(p_me,R2Y,0x00000133);
	DELAY(DELAY10);
	C3_WREG(p_me,G2Y,0x00000259);
	DELAY(DELAY10);
	C3_WREG(p_me,B2Y,0x00000074);
	DELAY(DELAY10);
	C3_WREG(p_me,R2U,0x00000F54);
	DELAY(DELAY10);
	C3_WREG(p_me,G2U,0x00000EAD);
	DELAY(DELAY10);
	C3_WREG(p_me,B2U,0x000001FF);
	DELAY(DELAY10);
	C3_WREG(p_me,R2V,0x000001FF);
	DELAY(DELAY10);
	C3_WREG(p_me,G2V,0x00000E53);
	DELAY(DELAY10);
	C3_WREG(p_me,B2V,0x00000FAE);
	DELAY(DELAY10);

}

/*****************************************************************************
 * Prepare the CL550 for decompression.  This is called by C3PutImage.
 *****************************************************************************/

C3InitDecompress(p_me)
    XV_struct* p_me;
{
	if ( no_ccube ) return;

	C3_WREG(p_me,S_reset, ~0);

	/* Load configuration registers */

#ifdef RGB422
	C3_WREG(p_me,Config, 0x100 + 0x38);
	DELAY(DELAY10);
#else
	C3_WREG(p_me,Config, 0x100 + 0x28); 
	DELAY(DELAY10);
#endif

	C3_WREG(p_me,DRQ_Enable,0);
	DELAY(DELAY10);
	C3_WREG(p_me,NMRQ_Enable,0);
	DELAY(DELAY10);

	C3_WREG(p_me,Init_1, 0x003e);
	DELAY(DELAY10);
	C3_WREG(p_me,Init_2, 0x0037);
	DELAY(DELAY10);
	C3_WREG(p_me,Init_3, 0x01ff);
	DELAY(DELAY10);
	C3_WREG(p_me,Init_4, 0x0049);
	DELAY(DELAY10);
	C3_WREG(p_me,Init_5, 0x0005);
	DELAY(DELAY10);
	C3_WREG(p_me,Init_7, 0x0022);
	DELAY(DELAY10);
	C3_WREG(p_me,QuantSync, 0x043e);
	DELAY(DELAY10);
	C3_WREG(p_me,QuantYC, 0x2033);
	DELAY(DELAY10);
	C3_WREG(p_me,QuantAB, 0x0000);
	DELAY(DELAY10);
	C3_WREG(p_me,QuantSelect, 1);
	DELAY(DELAY10);
	C3_WREG(p_me,VideoLatency, 0x017f);
	DELAY(DELAY10);

	/* Load coder configuration registers */

	C3_WREG(p_me,CodecSeq, 0x00cc);
	DELAY(DELAY10);
	C3_WREG(p_me,CodecCompH, 0x0088);
	DELAY(DELAY10);
	C3_WREG(p_me,CodecCompL, 0x0044);
	DELAY(DELAY10);
	C3_WREG(p_me,CoderAttr, 0x00004);
	DELAY(DELAY10);
	C3_WREG(p_me,CodingIntH, 0x0000);
	DELAY(DELAY10);
#ifdef RGB422
	C3_WREG(p_me,CodingIntL, 0x0002);
	DELAY(DELAY10);
#else
	C3_WREG(p_me,CodingIntL, 0x001);
	DELAY(DELAY10);
#endif
	C3_WREG(p_me,DecLength, 0x0008);
	DELAY(DELAY10);
	C3_WREG(p_me,CodeOrder, 0x0001);
	DELAY(DELAY10);

	/* Load RGB-YUV conversion matrix */

	C3_WREG(p_me,R2Y,0x00000400);
	DELAY(DELAY10);
	C3_WREG(p_me,G2Y,0x00000FFE);
	DELAY(DELAY10);
	C3_WREG(p_me,B2Y,0x000005C9);
	DELAY(DELAY10);
	C3_WREG(p_me,R2U,0x00000400);
	DELAY(DELAY10);
	C3_WREG(p_me,G2U,0x00000EA2);
	DELAY(DELAY10);
	C3_WREG(p_me,B2U,0x00000D23);
	DELAY(DELAY10);
	C3_WREG(p_me,R2V,0x00000400);
	DELAY(DELAY10);
	C3_WREG(p_me,G2V,0x0000071B);
	DELAY(DELAY10);
	C3_WREG(p_me,B2V,0x00000FFD);
}
