/*
 * (c) Copyright 1991 Parallax Graphics Inc, All Rights Reserved
 *
 * Parallax Graphics, Inc.
 * 2500 Condensa Street
 * Santa Clara, California 95051
 *
 * $Log:	plxv.h,v $
 * Revision 1.1  95/07/05  17:19:53  17:19:53  dist ()
 * Initial revision
 * 
 *
 * Revision 1.11  94/02/15  12:00:00  andyg
 * support for Power Video 700 card
 *
 * Revision 1.10  92/10/01  16:03:11  rocky
 * temp version for SONYRGB
 * 
 * Revision 1.9  92/09/16  13:06:14  rocky
 * new option modes for SONY camera
 * 
 * Revision 1.8  91/11/13  15:23:36  eric
 * PLXV_*_PIXEL_ATTR needs to be 3f
 * 
 * Revision 1.7  91/11/12  14:34:14  eric
 * changed from 28 bit mode to 32 bit mode
 * 
 * Revision 1.6  91/09/30  16:37:47  mears
 * some additions
 * 
 * Revision 1.5  91/08/14  12:25:28  mears
 * blanking info and TmrInit in nvram
 * 
 * Revision 1.4  91/08/13  13:50:52  mears
 * added input_status bit definitions
 * 
 * Revision 1.3  91/08/09  16:52:37  mears
 * new nvram structure
 * 
 * Revision 1.2  91/07/26  19:24:16  eric
 * new from bill
 * 
 * Revision 1.1  91/06/12  16:18:18  eric
 * Initial revision
 * 
 *
 */

#ifdef H_ID
#ifndef lint
static char *rcsid_plxv_h = "@(#)$Id: plxv.h,v 1.10 92/10/01 16:03:11 rocky Exp $ Parallax Graphics Inc";
#endif
#endif

/* plxv.h: the only stuff user programs should have to deal with */

#ifndef __PLXV__

#define __PLXV__

#include "xvideo.h"
#ifndef KERNEL
#include "objects.h"
#endif
#include "c3.h"

/*REVISION CONTROL FOR NVRAM COMPATABILITY 
  ---- MUST CHANGE WHEN NV STRUCTURE CHANGES -----
--- note: parameters above the POWER STATS MUST NOT CHANGE ******
*/

#define HLIB_REV 0x8008

#define PLXV_ADDR_REG(x,y)          ( (int)p_me->maps->reg[(y)]  + (x))
#define PLXV_ADDR_24(x,y) (int *)  ( (int)p_me->maps->m24[(y)] + ((x)<<2))
#define PLXV_ADDR_8(x,y) (char *) ( (int)p_me->maps->m8[(y)]  + (x))
#define PLXV_ADDR_1(x,y) (int *) ( (int)p_me->maps->m1[(y)]  + (((x)>>5)<<2))
#define PLXV_OFFS_1(x) ((x) & 0x1f )

#define NV_POSITION(p_me, board,ptr) \
         ((int)(&(p_me->board.ptr)) - (int)(&(p_me->board)))
/*rop names */
#define PLXV_ROP_CLEAR          0
#define PLXV_ROP_AND            1
#define PLXV_ROP_ANDREVERSE     4
#define PLXV_ROP_COPY           5
#define PLXV_ROP_ANDINVERTED    2
#define PLXV_ROP_NOOP           3
#define PLXV_ROP_XOR            6
#define PLXV_ROP_OR             7
#define PLXV_ROP_NOR            8
#define PLXV_ROP_EQUIV          9
#define PLXV_ROP_INVERT         0xc
#define PLXV_ROP_ORREVERSE      0xb
#define PLXV_ROP_COPYINVERTED   0xa
#define PLXV_ROP_ORINVERTED     0xd
#define PLXV_ROP_NAND           0xe
#define PLXV_ROP_SET            0xf

/* TAG DEFINITIONS */
#define PLXV_TAG_PSEUDO_COLOR   0x00
#define PLXV_TAG_TRUE_COLOR     0x10

/* obsolete
#define PLXV_TAG_VIDEO_1        0x20
#define PLXV_TAG_VIDEO_2        0x30
*/

#define PLXV_TAG_VIDEO_0        0x20
#define PLXV_TAG_VIDEO_LEN      0x20

/* there are PLXV_TAG_VIDEO_LEN video tag values, beginning at
 * PLXV_TAG_VIDEO_0, ending at PLXV_TAG_VIDEO_0+PLXV_TAG_VIDEO_LEN-1
 */

#define PLXV_TAG_DISPLAY_ATTR   0x30
#define PLXV_TAG_CURSOR_MAP     0x40
#define PLXV_TAG_CURSOR_ENAB    0x80
#define PLXV_TAG_CURSOR_ALL     0xc0
#define PLXV_TAG_PIXEL_ATTR     0x3f
#define PLXV_TAG_ALL            0xff

/* WMASK VALUES */
#define PLXV_WMASK_24           0x00ffffff
#define PLXV_WMASK_ALL          0xffffffff
#define PLXV_WMASK_PIXEL_ALL    0x3fffffff
#define PLXV_WMASK_DISPLAY_ATTR 0x30000000
#define PLXV_WMASK_CURSOR_MAP   0x40000000
#define PLXV_WMASK_CURSOR_ENAB  0x80000000
#define PLXV_WMASK_CURSOR_ALL   0xc0000000
#define PLXV_WMASK_PIXEL_ATTR   0x3f000000
#define PLXV_WMASK_TAG_ALL      0xff000000

/* define the board codes for the XVideo boards */
#define SBUS_1         4    /* 66 Hz */
#define TCHAN_1        5
#define SBUS_2         6    /* 76 Hz */
#define SBUS_3         7    /* 60 Hz */
#define SBUS_4         8    /* SQUASH */
#define SBUS_5         9    /* Purvis */
#define OPT_1          10   /* VIO */
#define OPT_2          11   /* RGB */
#define OPT_3          12   /* SONYRGB */
#define OPT_4          13   /* SONYRGB2 */
#define EISA_BUS_1_135 14   /* POWER VIDEO 700: 1280x1024 72Hz. */
#define EISA_BUS_1_108 15   /* POWER VIDEO 700: 1280x1024 60Hz. */
#define EISA_BUS_1_84  16   /* POWER VIDEO 700: 1024x768  75Hz. */
#define EISA_BUS_1_64  17   /* POWER VIDEO 700: 1024x768  60Hz. */
#define SBUS_6         18   /* SQUASH CUP. */

/* defines for the revs of the component chip (sets) being used */
#define GAREV_1 1       /* BIC: rev 1  CTC: rev 1 PPC: rev 1 */
#define GAREV_2 2       /* BIC: rev 2  CTC: rev 2 PPC: rev 1 */
#define DMSDREV_1 1     /* proto chip w/faulty vsync */
#define DMSDREV_2 2     /* supposed to be better     */
#define DMSEREV_1 1 /* 7199 encoder chip. */
#define C3REV_1 1

#define GA_REV GAREV_2

/* defines for the "flags" field of the io_port sub-structure */
#define IO_CHAN_TYPE  0x0000003f
#define IO_DMSD       0x00000001
#define IO_RGB_IN     0x00000002
#define IO_GEN_IN     0x00000003
#define IO_SONYRGB_IN 0x00000004
#define IO_OUTPUT     0x00000020
#define IO_COMP_OUT   0x00000021
#define IO_RGB_OUT    0x00000022
#define IO_GEN_OUT    0x00000023

/* defines for the "misc_flags" field of main structure */
#define PV700_DMSD2    2                /* Second dmsd is present on board. */
#define PV700_DMSE     4        /* 7199 encoder present on board. */
#define PV700_PROTO    8        /* Power video 700 prototype board. */
#define PV700_CCUBE 0x10        /* Ccube present. */


/* redback status bit defines for plxv_input_status */
#define CHROMA_LOCK 1
#define VERT_60HZ   32
#define SYNC_LOCK   64

/* defines for the various clt's being used on XVideo */
#define BT463 1
#define BT462 2
#define BT473 3
#define BT495 5

typedef struct {
        unsigned char red;
        unsigned char green;
        unsigned char blue;
        } XV_color;

typedef struct {
        short active_pixels;    /* per line */
        short active_lines;     /* per frame */
        short blank_lines;      /* per frame */
        short reset_line;       /* in a field */
        short start_line;       /* in a field */
        short unused;
        } XV_video_format;

typedef struct {
        unsigned short disp;
        unsigned char  fp;
        unsigned char  sy;
        unsigned char  bp;
        unsigned char  hsync_ser;               /* disp param for serration  */
        unsigned short hsync_nser;              /* disp param for serration  */
		unsigned char  sync_pol;				/* 1 is positive polarity */
		unsigned char unused[3];				/* pad */
        } XV_disp_format;
        
typedef struct {
        unsigned char  bright;
        unsigned char  contrast;
        unsigned char  saturation;
        unsigned char  hue;
        } XV_knob_rec;

typedef struct {
        unsigned char   rev;            /* rev of the dmsd chipset      */
        unsigned char   unit;           /* which "unit" number          */
        unsigned char val[26];          /* locations 0:22 are 7191      */
        } XV_dmsd_config;               /* locations 23:25 unused       */

typedef struct {
        unsigned char   rev;            /* rev of the 7199 encoder chipset  */
        unsigned char   unit;           /* which "unit" number              */
        unsigned char val[26];          /* location 0:14 are 7199           */
        } XV_dmse_config;               /* locations 15:25 unused           */

typedef struct {
        unsigned char   command[4];
        unsigned short  fgate_start;
        unsigned short  fgate_stop;
        unsigned char   ngate_start;
        unsigned char   ngate_stop;
        unsigned char   hsync_start;
        unsigned char   clamp_start;
        unsigned char   clamp_stop;
        unsigned char   unused[3];
        } XV_bt261_config;
        
typedef struct {
        unsigned short  hcount;
        unsigned short  hactive;
        unsigned char   zero_stop;
        unsigned char   unused[3];
        } XV_bt261_init;

typedef struct {
        unsigned char   val[6];
        unsigned char   unused[2];
        } XV_bt253_config;

/* data structure to define the format of the RGB_IN interface */
/* this includes the Xpoint config_bits need to set up the hardware */
typedef struct {
        unsigned long   xpoint_bits;            /* Xpoint config value       */
        XV_bt261_config timing;
        XV_bt261_init   format[2];
        XV_bt253_config cal[2];
        } XV_rgbin_config;



/* data structure to define the format of a general interface */
/* this includes the Xpoint config_bits need to set up the hardware */
typedef struct {
        unsigned long   xpoint_bits;            /* Xpoint config value       */
        unsigned char   val[28];
        } XV_gen_config;

/* structure to define general interface for A and B ports */
/* port can be either DMSD, or general purpose             */
typedef struct {
        unsigned long   flags;                  /* see IO_XXXX constants     */
        XV_knob_rec     factory,u1,u2;          /* 3 sets of users adjusts */
        XV_video_format format[2];
        union  {
                    XV_gen_config       gen;
                    XV_dmsd_config      dmsd;
                    XV_dmse_config      dmse;
                    XV_rgbin_config     rgbin;
                } config;
        } XV_io_port;

/* the full data structure for the CONTROLLER NVRAM                          */

typedef struct {
        unsigned char   board_code;             /* the PARALLAX HARDWARE NUM */
        unsigned char   ga_rev;                 /* a code for which chip set */
        unsigned char   c3_rev;                 /* a code for CL550 revision */
        unsigned char   clt_type;               /* what type of output stage */
        unsigned long   serial_number;
        unsigned short  hlib_rev;
        unsigned char   unused0[2];
        unsigned long   power_hours;            /* hours of operation        */
        unsigned short  power_cycles;           /* number of power ups       */
        unsigned short  xsize;                  /* Pixels in a line          */
        unsigned short  ysize;                  /* lines on display          */
        unsigned short  Attr;                   /* another startup value     */
        unsigned short  DispKludge;             /* a startup value           */
        unsigned short  DispFlgs;               /* Stuff value at power up   */
        unsigned long   StartUp;                /* Stuff value at power up   */
        unsigned short  TmrInit;                /* Stuff value at power up   */
        unsigned short  RasMode;                /* Stuff value at power up   */
        unsigned char   OLDI2C;                 /* Stuff value at power up   */
        unsigned char   skip;
        unsigned char   delay;
        unsigned char   unused_1[1];
        XV_disp_format  h,v;                    /* general display timing    */
        XV_io_port      d;                      /* DMSD video adjustments    */
        unsigned long   icd_clock;
        unsigned short  linebytes;
        unsigned char   misc_flags;
        unsigned char   misc_flags2;
		unsigned char   zoom;					/* Current zoom factor. */
        unsigned char   unused_2[1];
		unsigned short  pixel_clock;
        } XV_controller_nvram;
/* the full data structure for the OPTION NVRAM                              */

typedef struct {
        unsigned char   board_code;             /* the PARALLAX HARDWARE NUM */
        unsigned char   board_rev;              /* board revision code       */
        unsigned char   clt_type;               /* what type of output stage */
        unsigned char   unused_1[1];
        unsigned long   serial_number;
        unsigned short  hlib_rev;
        unsigned char   unused0[2];
        unsigned long   power_hours;            /* hours of operation        */
        unsigned short  power_cycles;           /* number of power ups       */
        unsigned char   unused_2[2];
        XV_io_port      a,b;
        unsigned char   misc_flags;
        unsigned char   misc_flags2;
        unsigned char   unused[8];
        } XV_option_nvram;


/* data structure for zoom parameters. */

typedef struct {
        XV_disp_format  h,v;                    /* general display timing    */
        unsigned short  DispFlgs;               
        unsigned short  DispKludge;            
        unsigned char   delay;
        unsigned char   skip;
		unsigned short	factor;			        /* Zoom factor. */
		} XV_zoom_nvram;

typedef struct {
        int                     *m24[1024];
        char                    *m8[1024];
        int                     *m1[1024];
        int                     reg[1024];
        } XV_addr_maps;


typedef struct {
        int                     addr_wrap;
		void*					atv;		/* Platform-specific methods tv. */
        int*                    c3base;
        C3_WREGS*               c3_wreg;
        C3_RREGS*               c3_rreg;
        int*                    eprombase;
        int                     fd_ccube;
        int                     fd_image;
        XV_video_format**       format_v_ptr;
        XV_video_format**       format_w_ptr;
        XV_addr_maps*           maps;
        XV_controller_nvram     main;
        int*                    m1base;
        char*                   m8base;
        int*                    m24base;
        XV_option_nvram         opt;
#ifndef KERNEL
        O_Input*                p_inputs[5];
        O_I2C*                  p_i2c;
		O_Mmap*					p_mmap;
#endif
        XV_video_format*        plxv_io_format[5];
		int						plxv_opt_cntl_reg;
#ifndef KERNEL
        O_Output*               p_outputs[5];
		O_Rsrc*					p_rsrc;
#endif
        int*                    regbase;
        XV_mem_map*             xvm;
        XV_read_regs*           xv_rreg;
        XV_shdw_regs*           xv_shdw;
        XV_write_regs*          xv_wreg;
		XV_zoom_nvram			zoom[1];		/* Zoom display parameters. */
        } XV_struct;
        
XV_struct* O_HWL_O_HWL();
extern XV_struct *plxv_init_context();
#ifdef NOTDEF
extern XV_struct *xv_cp,*plxv_init_context();
extern XV_addr_maps *xv_ap;
extern int *xv_m24base,*xv_m1base,*xv_eprombase,*xv_c3base,*xv_regbase;
extern char *xv_m8base;
#endif /* NOTDEF */

#endif /* __PLXV__ */

