/*
 *   Copyright (c) 1994 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 */
/*
** File:
**
**  xvddx_PGIpv.c: Hardware specific DD layer routines for the Parallax
**                  PowerVideo adaptor.
**
** Author:
**
**   Andy Goodrich (Parallax)
**
** Revisions:
**
**   08.03.94 Goodrich
**     - addition of PGIpvControl 
**
**   04.02.94 Goodrich
**     - addition of PGIpvGetLongPortAttribute & PGIpvSetLongPortAttribute
**
**   20.01.94 Goodrich
**     - initial coding.
**
** $Log:	xvddx_PGIpv.c,v $
 * Revision 1.11  96/06/18  09:47:12  09:47:12  dist (Software Distribution Account)
 * added shared memory
 * 
 * Revision 1.10  1996/05/16  18:54:34  dist
 * fixed off by one error on enable/disable
 *
 * Revision 1.9  1996/04/03  22:20:43  dist
 * fixed various things
 *
 * Revision 1.8  1995/12/22  19:20:41  dist
 * Andy Goodrich: Fixed ifdef that incorrectly did not set screen
 *                save wrapper routine for AIX platform.
 *
 * Revision 1.7  1995/12/11  15:12:14  dist
 * Andy Goodrich: Checkin / merging of hedge changes.
 *
 * Revision 1.6  95/12/11  15:02:20  15:02:20  dist ()
 * Andy Goodrich: Checkin / merging of hedge changes.
 * 
 * Revision 1.4  95/07/21  15:03:06  15:03:06  dist ()
 * Andy Goodrich: Added code to zoom processing to maintain the proper
 *                input source encodings rather than defaulting back
 *                to NTSC.
 * 
 * Revision 1.3  95/06/01  11:08:40  11:08:40  dist ()
 * Andy Goodrich: Fix bug in processing of ccube decompression tables.
 * 
 * Revision 1.2  95/05/25  17:02:56  17:02:56  dist ()
 * Andy Goodrich - Combined HP/AIX version for HP 9.1 code freeze.
 * 
 * Revision 1.1  95/05/12  13:44:10  13:44:10  dist ()
 * Initial revision
 * 
**
*/
#include <sys/types.h>  /* Pick up u_int and friends. */
#include <stdio.h>              /* For diagnostics and error messages. */
 
/*  XV_DIAG bit definitions:
 *
 *    0x01 - normal one-shot diagnostics
 *    0x20 - wrapper tracing
 *    0x40 - request tracing
 *    0x80 - server initialization tracing
 *    #define XV_DIAG  1 
 *    #define XV_DIAG 0x20
 */
#include "xvdebug.h" 

#define DEV_NAME(pPort) \
	((HwPGIpv*)((XvddPortPtr)pPort->devPriv.ptr)->phw)->p_context->p_mmap->name

#ifdef _AIX
#include <fcntl.h>          /* Pick up O_RDWR and friends. */
#else
#include <sys/fcntl.h>      /* Pick up O_RDWR and friends. */
#endif
#include "X.h"
#include "Xv.h"
#include "Xproto.h"
#include "scrnintstr.h"
#include "validate.h"
#include "windowstr.h"
#include "pixmapstr.h"
#include "gcstruct.h"
#include "dixstruct.h"
#include "xvdix.h"      	 /* Pick up device-independent layer. */
#include "xvddx.h"      	 /* Pick up device-dependent layer: XvddConfigPtr.*/
#include "xvhw_PGIpv.h"      /* Pick up hardware layer: private object defs. */

#if defined(DECWINDOWS) 
#include "cfb.h"
#ifdef DECWINDOWS
#include "tfb.h"                    /* 24 PLANE DDX HEADER */
#else
#define tfbPrivGCPtr cfbPrivGCPtr
#define tfbGCPrivateIndex cfbGCPrivateIndex
#endif /* tfb.h include */
#endif

#include "Xvddx_PGI.h" 		 /* Pick up XV_xxx encoding declarations. */

#if defined(__hpux)
#include "encodings_PGIpv700.h"
#endif
#if defined(_AIX) || defined(sun)
#include "encodings_PGIpvpci.h"
#endif

#define SHARED_MEMORY

static struct _Attributes {

	/* GENERIC PORT CONTROLS: */

	Atom XvplxAttributes;
	Atom XvplxBrightness;
	Atom XvplxChannel;
	Atom XvplxContrast;
	Atom XvplxControls;
	Atom XvplxEncoding;
	Atom XvplxHue;
	Atom XvplxSaturation;
	Atom XvplxStatus;
 
	/* DEVICE-DEPENDENT PORT CONTROLS: */
   
	Atom XvplxCrop;
	Atom XvplxCcube;
	Atom XvplxQuerySignals;
	Atom XvplxNextPort;
	Atom XvplxNvramControl;
	Atom XvplxNvramMain;
	Atom XvplxNvramZoom;
	Atom XvplxRasterOps;
	Atom XvplxTableComp;
	Atom XvplxTableDecomp;
	Atom XvplxTest;
	Atom XvplxWid;
	Atom XvplxZoom;

	Atom XvropsFlipHorizontal;
	Atom XvropsFlipVertical;
	Atom XvropsInvertFields;
} attr_atoms;

static int l_attr_atoms = sizeof(attr_atoms);

static struct _Controls {
	Atom XvplxFirstPutCImage;
	Atom XvplxGetCImage;
	Atom XvplxPutCImage;
#ifdef SHARED_MEMORY
	Atom XvplxFirstShmPutCImage;
	Atom XvplxShmGetCImage;
	Atom XvplxShmPutCImage;
#endif
} ctrl_atoms;

static int l_ctrl_atoms = sizeof(ctrl_atoms);

/* 
 *  FORWARD REFERENCES:
 */
#if !defined(_AIX)
static void		PGIpvClipNotify();
#endif
static int		PGIpvControl();
extern int		PGIpvGetImage();
static int		PGIpvGetLongPortAttribute();
static int		PGIpvGetPortAttribute();
static int		PGIpvPrepareSurface();
extern int		PGIpvPutImage();
static int      PGIpvSaveScreen();
static int		PGIpvSetLongPortAttribute();
static int		PGIpvSetPortAttribute();
static void 	PGIpvSetZoom();
static void		PGIpvWindowExposures();
void            rops_to_new_encoding();

/*----------------------------------------------------------------------------*
 |"PGIpvWindowExposures"
 |
 |  Adaptor transfer vector wrapper routine to handle window exposures.
 |  If the window being exposed is the one in which video is playing restart
 |  video after setting its new geometry. 
 *----------------------------------------------------------------------------*/
static void
PGIpvWindowExposures(pxvAdapt, pWin, pReg, pOtherReg)
	XvAdaptorPtr	pxvAdapt;	/* Adaptor currently checking ports of. */
	WindowPtr 		pWin;
	RegionPtr 		pReg;
	RegionPtr 		pOtherReg;

{
    int             fill_x;     /* Starting x of enable memory to fill. */
    int             fill_y;     /* Starting y of enable memory to fill. */
	int				found_one;	/* = 1 if found port attached to this screen. */
    int             iBox;       /* # of box processing in region list. */
	int			    iPort;		/* Entry in pxvAdapt->pPorts  processing. */
    int             nBoxes;     /* # of boxes in region list. */
    BoxPtr          pBox;       /* Box now processing in region list. */
	RegionPtr       pClip;      /* pDraw's current clipping region. */
	XvddPortPtr     pddPort;    /* -> private port rec(pPort->devPriv.ptr). */
	XvPortPtr       pxvPort;	/* Port currently checking for video restart. */

	DEBUGF(0x20, (fd, "PGIpvWindowExposures: entry\n"));


	/*  LOOP OVER THIS ADAPTORS PORTS LOOKING FOR ONE WITH THIS WINDOW 
     *  ATTACHED.
     */
	for ( iPort = 0, found_one = 0; iPort < pxvAdapt->nPorts; iPort++ )
	{
		pxvPort = &pxvAdapt->pPorts[iPort];

		if ( pxvPort->pDraw == (DrawablePtr)pWin ) found_one = 1;
		if ( pxvPort->pDraw && (((WindowPtr)pxvPort->pDraw)->realized) )
		{
			if (pxvPort->pDraw->serialNumber != pxvPort->serialNumber)
			{
	DEBUGF(0x20, (fd, "PGIpvWindowExposures: resume iPort=%d\n",iPort));
				(* pxvPort->pAdaptor->ddResumeVideo)(pxvPort);
			}
		}
	}


	/*  IF PORT WAS NOT FOUND THIS IS A NON-ATTACHED WINDOW.
     *
	 *  Pick up the composite clip list for this window and zero the 
     *  power video memory associated with its visible regions.
	 */
	if ( !found_one )
	{
#ifndef DECWINDOWS 
    	pClip = &pWin->clipList;
#else  /* DECWINDOWS */
        pClip = (pxvPort->pDraw->depth == 8) ?
            ((cfbPrivGCPtr) pxvPort->pGC->devPrivates[cfbGCPrivateIndex].ptr)->
            pCompositeClip : ((tfbPrivGCPtr)
                pxvPort->pGC->devPrivates[tfbGCPrivateIndex].ptr)->pCompositeClip;
	DEBUGF(0x20, (fd, "PGIpvWindowExposures: depth=%d\n",pxvPort->pDraw->depth));
#endif /* DECWINDOWS */

		pddPort = (XvddPortPtr)pxvAdapt->pPorts[0].devPriv.ptr;
		pBox = REGION_RECTS(pClip);
		nBoxes = (int)REGION_NUM_RECTS(pClip);
		for (iBox = 0; iBox < nBoxes; iBox++, pBox++)
		{
			fill_x = pBox->x1;
			fill_y = pBox->y1;
			Xvhw_Fill_Video_Enable(pddPort->phw, fill_x, fill_y,
				(pBox->x2 - fill_x), (pBox->y2 - fill_y), PGI_Fill_Disable);
		}
	}

	DEBUGF(0x20, (fd, "PGIpvWindowExposures: exit\n"));
}



/*----------------------------------------------------------------------------*
 |"PGIpvAdaptorInit"
 |
 |  Routine to allocate and initialize the Xv adaptor record associated with
 |  a Parallax PowerVideo  card.
 |
 |    -> initialized adaptor record on success
 |    =  0 on failure
 *----------------------------------------------------------------------------*/
XvAdaptorPtr PGIpvAdaptorInit(pScreen, device)
    ScreenPtr pScreen;			/* -> screen this adaptor is connected to. */
	char*	  device;			/* =  device name of this adaptor. */
{
    int            base_id;     /* Base port id for Adaptor. */
	char*		   dname;		/* Name of device after mapping it. */
    int            id;          /* Client id for calling AddResource. */
    int            iEncode;     /* # of encodings processed so far. */
    int            iPort;       /* # of port records processed so far. */
    int            nPorts;      /* # of ports supported by this adaptor. */
	int			   options;		/* Options code from hw library. */
    XvAdaptorPtr   pAdaptor;    /* -> adapator record initializing. */
    XvddPortPtr    pddPort;     /* -> private port rec(pPort->devPriv.ptr). */
    XvEncodingPtr  pEncode;     /* -> encoding record accessing. */
	HwPGIpv*       phw;     	/* Hardware object for this port. */
	Plx_Map_Rec*   p_mapping;	/* Hardware mapping for this adaptor. */
    XvPortPtr      pPort;      	/* -> xv port record accessing (pPorts[i]). */
    XvPortPtr      pPorts;     	/* -> storage allocated for xv port records. */
    Xvhw_Signal    signal;      /* Signal timing information. */

    DEBUGF(0x80, (fd, "%s: PGIpvAdaptorInit: stack @ 0x%08x\n", 
		device, &base_id));


    /*  MAP IN THE DEVICE WE WERE PASSED. IF WE CAN'T RETURN FAILURE. */
     
    if ( !(p_mapping = (Plx_Map_Rec*)xv_pv_map(device)) )
    {
		DEBUGF(0x80, (fd, "%s: PGIpvAdaptorInit: exit - could not map device\n",
			device));
        return 0;
    }
	dname = p_mapping->p_context->p_mmap->name;
 

	/*  DEVICE MAPPED, SO ALLOCATE AN ADAPTOR RECORD AND FILL IN DD STUFF. */

	DEBUGF(0x80,(fd, "%10d: ", i_I2C_get_timer(p_mapping->p_context->p_i2c)));
	DEBUGF(0x80, (fd, 
		"%s: PGIpvAdaptorInit: before generic initialization\n", dname));
#ifdef __hpux
	nPorts = 8;
#else
	nPorts = 128;
#endif
	pAdaptor = (XvAdaptorPtr)XvddAdaptorInit(pScreen, nPorts, 1);
	if ( !pAdaptor )
	{
		ERRORF((fd, "Unable to allocate adaptor record\n"));
		xfree((char*)p_mapping);
		return 0;
	}
	DEBUGF(0x80,(fd, "%10d: ", i_I2C_get_timer(p_mapping->p_context->p_i2c)));
	DEBUGF(0x80, (fd, "%s: PGIpvAdaptorInit: after generic initialization\n",
		dname));
	 
    pAdaptor->ddClipNotify           = PGIpvClipNotify;
    pAdaptor->ddControl              = PGIpvControl;
    pAdaptor->ddGetImage             = PGIpvGetImage;
    pAdaptor->ddGetLongPortAttribute = PGIpvGetLongPortAttribute;
    pAdaptor->ddGetPortAttribute     = PGIpvGetPortAttribute;
    pAdaptor->ddPrepareSurface       = PGIpvPrepareSurface;
    pAdaptor->ddPutImage             = PGIpvPutImage;
    pAdaptor->ddSaveScreen           = PGIpvSaveScreen;
    pAdaptor->ddSetLongPortAttribute = PGIpvSetLongPortAttribute;
    pAdaptor->ddSetPortAttribute     = PGIpvSetPortAttribute;
    pAdaptor->ddWindowExposures      = PGIpvWindowExposures;
    pAdaptor->name = "PGItv2";


	/*  INTIALIZE PORT RECORDS ASSOCIATED WITH THIS ADAPTOR. */

	pPort = pAdaptor->pPorts;
	for ( iPort = 0; iPort < nPorts; iPort++, pPort++ )
	{
		DEBUGF(0x80,(fd, "%s: PGIpvAdaptorInit: initializing port %d\n", 
			dname, iPort));


		/*  ALLOCATE HARWARE LEVEL STRUCTURES FOR THIS PORT. */
 
		if ( !(phw = (HwPGIpv*)xv_pv_open(iPort, p_mapping)) )
		{
			xfree(p_mapping);
			return 0;
		}
		pddPort = (XvddPortPtr)pPort->devPriv.ptr;
		pddPort->phw = (pointer)phw;
		DEBUGF(0x80,(fd, "%10d: ", 
			i_I2C_get_timer(p_mapping->p_context->p_i2c)));
		DEBUGF(0x80, 
			(fd, "%s: PGIpvAdaptorInit: after call to xv_pv_open\n",
			dname));
 
 
		/*  PERFORM SPECIAL PROCESSING BASED ON PORT NUMBER. */
 
		DEBUGF(0x80,(fd, "%10d: ", 
			i_I2C_get_timer(p_mapping->p_context->p_i2c)));
		DEBUGF(0x80, (fd,"%s: PGIpvAdaptorInit: before special processing\n",
			dname));
		switch ( iPort )
		{
		  case 0:
			XvImageInit();
			PGIpvImageInit(pAdaptor); 
			Xvhw_Fill_Video_Enable(phw, 0, 0, 
				Xvhw_Max_Width(phw), Xvhw_Max_Height(phw), PGI_Fill_Remove);
			break;
		}


		/*  SET SIGNAL ENCODING INFORMATION. */
 
		DEBUGF(0x80,(fd, "%10d: ", 
			i_I2C_get_timer(p_mapping->p_context->p_i2c)));
		DEBUGF(0x80, (fd, "%s: PGIpvAdaptorInit: before setting encodings\n",
			dname));
		Xvhw_Control(phw, "OPTION_CODE", &options);
		switch ( options )
		{
		  default:
			pPort->nEncodings = nPGIpvBaseEncodings;
			pPort->pEncodings = PGIpvBaseEncodings;
			pPort->type = XvInputMask;
			break;
		  case OPT_1:
			pPort->nEncodings = nPGIpvOpt1Encodings;
			pPort->pEncodings = PGIpvOpt1Encodings;
			pPort->type = XvInputMask|XvOutputMask;
			break;
#ifdef __hpux
		  case OPT_2:
			pPort->nEncodings = nPGIpvOpt2Encodings;
			pPort->pEncodings = PGIpvOpt2Encodings;
			pPort->type = XvInputMask;
			break;
#endif /* __hpux */
		}
		pPort->pEncoding = pPort->pEncodings;
		pEncode = pPort->pEncodings;
		for (iEncode = 0; iEncode < pPort->nEncodings; iEncode++)
		{
			pEncode->pScreen = pScreen;
			pEncode++;
		}
 
		DEBUGF(0x80,(fd, "%10d: ", 
			i_I2C_get_timer(p_mapping->p_context->p_i2c)));
		DEBUGF(0x80, (fd, "%s: PGIpvAdaptorInit: after encoding screens\n",
			dname));

	}




    /*  ALLOCATE THIS ADAPTOR'S ATOMS FOR PORT ATTRIBUTES --- MORE LATER */
     
    attr_atoms.XvplxAttributes      = MakeAtom("XV_ATTRIBUTES", 
                                    strlen("XV_ATTRIBUTES"), 
                                    xTrue);
    attr_atoms.XvplxBrightness      = MakeAtom("XV_BRIGHTNESS", 
                                    strlen("XV_BRIGHTNESS"), 
                                    xTrue);
    attr_atoms.XvplxChannel         = MakeAtom("XV_PLX_CHANNEL", 
                                    strlen("XV_PLX_CHANNEL"), 
                                    xTrue);
    attr_atoms.XvplxContrast        = MakeAtom("XV_CONTRAST", 
                                    strlen("XV_CONTRAST"), 
                                    xTrue);
    attr_atoms.XvplxControls        = MakeAtom("XV_CONTROLS", 
                                    strlen("XV_CONTROLS"), 
                                    xTrue);
    attr_atoms.XvplxCcube           = MakeAtom("XV_PLX_CCUBE",
                                    strlen("XV_PLX_CCUBE"),
                                    xTrue);
    attr_atoms.XvplxCrop            = MakeAtom("XV_PLX_CROP",
                                    strlen("XV_PLX_CROP"),
                                    xTrue);
    attr_atoms.XvplxEncoding        = MakeAtom("XV_ENCODING", 
                                    strlen("XV_ENCODING"), 
                                    xTrue);
    ctrl_atoms.XvplxFirstPutCImage  = MakeAtom("XV_PLX_FIRST_PUTCIMAGE",
                                    strlen("XV_PLX_FIRST_PUTCIMAGE"),
                                    xTrue);
    ctrl_atoms.XvplxGetCImage =        MakeAtom("XV_PLX_GETCIMAGE",
                                    strlen("XV_PLX_GETCIMAGE"),
                                    xTrue);
    attr_atoms.XvplxHue             = MakeAtom("XV_HUE", 
                                    strlen("XV_HUE"), 
                                    xTrue);
    attr_atoms.XvplxNextPort       = MakeAtom("XV_PLX_NEXT_PORT", 
                                    strlen("XV_PLX_NEXT_PORT"), 
                                    xTrue);
    attr_atoms.XvplxNvramControl   = MakeAtom("XV_PLX_NVRAM_CONTROL", 
                                    strlen("XV_PLX_NVRAM_CONTROL"), 
                                    xTrue);
    attr_atoms.XvplxNvramMain      = MakeAtom("XV_PLX_NVRAM_MAIN", 
                                    strlen("XV_PLX_NVRAM_MAIN"), 
                                    xTrue);
    attr_atoms.XvplxNvramZoom      = MakeAtom("XV_PLX_NVRAM_ZOOM", 
                                    strlen("XV_PLX_NVRAM_ZOOM"), 
                                    xTrue);
    ctrl_atoms.XvplxPutCImage      = MakeAtom("XV_PLX_PUTCIMAGE",
                                    strlen("XV_PLX_PUTCIMAGE"),
                                    xTrue);
    attr_atoms.XvplxRasterOps       = MakeAtom("XV_PLX_RASTEROPS",
                                    strlen("XV_PLX_RASTEROPS"),
                                    xTrue);
    attr_atoms.XvplxTableComp       = MakeAtom("XV_PLX_TABLE_COMPRESSION",
                                    strlen("XV_PLX_TABLE_COMPRESSION"),
                                    xTrue);
    attr_atoms.XvplxTableDecomp     = MakeAtom("XV_PLX_TABLE_DECOMPRESSION",
                                    strlen("XV_PLX_TABLE_DECOMPRESSION"),
                                    xTrue);
    attr_atoms.XvplxQuerySignals    = MakeAtom("XV_PLX_QUERY_SIGNALS",
                                    strlen("XV_PLX_QUERY_SIGNALS"),
                                    xTrue);
    attr_atoms.XvplxSaturation      = MakeAtom("XV_SATURATION", 
                                    strlen("XV_SATURATION"), 
                                    xTrue);
    attr_atoms.XvplxStatus          = MakeAtom("XV_STATUS", 
                                    strlen("XV_STATUS"), 
                                    xTrue);
    attr_atoms.XvplxTest            = MakeAtom("XV_PLX_TEST",
                                    strlen("XV_PLX_TEST"),
                                    xTrue);
    attr_atoms.XvplxWid             = MakeAtom("XV_PLX_WID",
                                    strlen("XV_PLX_WID"),
                                    xTrue);
    attr_atoms.XvplxZoom            = MakeAtom("XV_PLX_ZOOM",
                                    strlen("XV_PLX_ZOOM"),
                                    xTrue);
    attr_atoms.XvropsFlipHorizontal = MakeAtom("XV_ROPS_FLIP_HORIZONTAL",
                                    strlen("XV_ROPS_FLIP_HORIZONTAL"),
                                    xTrue);
    attr_atoms.XvropsFlipVertical   = MakeAtom("XV_ROPS_FLIP_VERTICAL",
                                    strlen("XV_ROPS_FLIP_VERTICAL"),
                                    xTrue);
    attr_atoms.XvropsInvertFields   = MakeAtom("XV_ROPS_INVERT_FIELDS",
                                    strlen("XV_ROPS_INVERT_FIELDS"),
                                    xTrue);
#ifdef SHARED_MEMORY
    ctrl_atoms.XvplxFirstShmPutCImage  = MakeAtom("XV_PLX_FIRST_SHMPUTCIMAGE",
                                    strlen("XV_PLX_FIRST_SHMPUTCIMAGE"),
                                    xTrue);
    ctrl_atoms.XvplxShmGetCImage =  MakeAtom("XV_PLX_SHMGETCIMAGE",
                                    strlen("XV_PLX_SHMGETCIMAGE"),
                                    xTrue);
    ctrl_atoms.XvplxShmPutCImage  = MakeAtom("XV_PLX_SHMPUTCIMAGE",
                                    strlen("XV_PLX_SHMPUTCIMAGE"),
                                    xTrue);
#endif

	DEBUGF(0x80,(fd, "%10d: ", 
		i_I2C_get_timer(p_mapping->p_context->p_i2c)));
	xfree(p_mapping);
    DEBUGF(0x80, (fd, "%s: PGIpvAdaptorInit: exit stack @ 0x%08x\n", 
		dname, &base_id));
    return pAdaptor;
}


/*----------------------------------------------------------------------------* 
 |"PGIpvClipNotify"
 |
 |  Screen transfer vector wrapper routine to indicate a clipping change.
 *----------------------------------------------------------------------------*/
static void
PGIpvClipNotify(pxvAdapt, pWin, dx, dy)
	XvAdaptorPtr	pxvAdapt;	/* -> adaptor accessing. */
	WindowPtr 		pWin;
	int 			dx;
	int				dy;
{

	int				fill_h;		/* Height of enable memory to fill. */
    int             fill_x;     /* Starting x of enable memory to fill. */
    int             fill_y;     /* Starting y of enable memory to fill. */
	int				fill_w;		/* Width of enable memory to fill. */
    int             found_one;  /* = 1 if found port attached to this screen. */
    int             iBox;       /* # of box processing in region list. */
    int             iPort;      /* Entry in pxvAdapt->pPorts  processing. */
    int             nBoxes;     /* # of boxes in region list. */
    BoxPtr          pBox;       /* Box now processing in region list. */
    RegionPtr       pClip;      /* pDraw's current clipping region. */
    XvddPortPtr     pddPort;    /* -> private port rec(pPort->devPriv.ptr). */
	XvPortPtr       pxvPort;	/* Port currently checking for video suspend. */
 

	DEBUGF(0x20, (fd, "PGIpvClipNotify: entry\n"));
	/* #### m_Symtab_bear_trap();  */
	pddPort = (XvddPortPtr)pxvAdapt->pPorts[0].devPriv.ptr;

	/*  LOOP THROUGH PORTS ASSOCIATED WITH THIS ADAPTOR SUSPENDING VIDEO
	 *  IF THE WINDOW BEING CLIPPED CONTAINS VIDEO.
	 */
	for ( iPort = 0, found_one = 0; iPort < pxvAdapt->nPorts; iPort++ )
	{
		pxvPort = &pxvAdapt->pPorts[iPort];
		if (pxvPort->pDraw == (DrawablePtr)pWin)
		{
			if(pxvPort->direction != XvDrawableToSignal)
				found_one = 1;
			(* pxvAdapt->ddSuspendVideo)(pxvPort);
		}
	}


	/*  IF NOT PORT WAS FOUND THIS IS A NON-ATTACHED WINDOW.
     *
	 *  Pick up the composite clip list for this window and zero the 
     *  pv memory associated with its visible regions.
	 */
	if ( !found_one )
	{
#ifndef DECWINDOWS
    	pClip = &pWin->clipList;
#else  /* DECWINDOWS */
        pClip = (pxvPort->pDraw->depth == 8) ?
            ((cfbPrivGCPtr) pxvPort->pGC->devPrivates[cfbGCPrivateIndex].ptr)->
            pCompositeClip : ((tfbPrivGCPtr)
                pxvPort->pGC->devPrivates[tfbGCPrivateIndex].ptr)->pCompositeClip;
	DEBUGF(0x20, (fd, "PGIpvClipNotify: depth=%d\n",pxvPort->pDraw->depth));
#endif /* DECWINDOWS */

		pddPort = (XvddPortPtr)pxvAdapt->pPorts[0].devPriv.ptr;
		pBox = REGION_RECTS(pClip);
		nBoxes = (int)REGION_NUM_RECTS(pClip);
		for (iBox = 0; iBox < nBoxes; iBox++, pBox++)
		{
			fill_x = pBox->x1;
			fill_y = pBox->y1;
			fill_w = (pBox->x2 - fill_x);
			fill_h = (pBox->y2 - fill_y);
			Xvhw_Fill_Video_Enable(pddPort->phw, 
				fill_x, fill_y, fill_w, fill_h, PGI_Fill_Disable);
		}
	}

	DEBUGF(0x20, (fd, "PGIpvClipNotify: exit\n"));
}


/*---------------------------------------------------------------------------*
 |"PGIpvControl"
 |
 |  Transfer vector routine to perform an arbitrary device dependent
 |  operation. In addition to the normal request header there may be
 |  ancilliary data to qualify the request. This is supplied by the 
 |  pQualify pointer. This routine is responsible for generating the 
 |  X Reply for this request on success.
 |
 |    = 0 on success.
 |    = error on failure.
 *---------------------------------------------------------------------------*/
static int PGIpvControl(client, pPort, operation, length, pQualify)
    ClientPtr client;		/* -> client request data. */
    XvPortPtr pPort;		/* -> port request is on. */
    Atom      operation;	/* =  operation to be performed. */
	int		  length;		/* =  length of qualifying data. */
    char      *pQualify;	/* -> additional data for qualifying request. */
{
	XvddPortPtr pddPort;	/* Device-dependent storage for pPort. */
	int			rc;			/* Return code from PlxXXXCImage. */

	DEBUGF(0x40, (fd, 
		"PGIpvControl entry: port id: %x operation: %x length: %d\n",
		pPort->id, operation, length)
	);

	pddPort = (XvddPortPtr)pPort->devPriv.ptr;

#ifndef SHARED_MEMORY
	if (operation == ctrl_atoms.XvplxGetCImage)
	{
	    rc = PlxGetCImage(client, pPort);
	}
	else if (operation == ctrl_atoms.XvplxFirstPutCImage)
	{
	    rc = PlxPutCImage(client, pPort, 1);
	}
	else if (operation == ctrl_atoms.XvplxPutCImage)
	{
	    rc = PlxPutCImage(client, pPort, 0);
	}
#else
	if (operation == ctrl_atoms.XvplxGetCImage)
	{
	    rc = PlxGetCImage(client, pPort, xFalse);
	}
	else if (operation == ctrl_atoms.XvplxFirstPutCImage)
	{
	    rc = PlxPutCImage(client, pPort, 1, xFalse);
	}
	else if (operation == ctrl_atoms.XvplxPutCImage)
	{
	    rc = PlxPutCImage(client, pPort, 0, xFalse);
	}
	else if (operation == ctrl_atoms.XvplxShmGetCImage)
	{
	    rc = PlxGetCImage(client, pPort, xTrue);
	}
	else if (operation == ctrl_atoms.XvplxFirstShmPutCImage)
	{
	    rc = PlxPutCImage(client, pPort, 1, xTrue);
	}
	else if (operation == ctrl_atoms.XvplxShmPutCImage)
	{
	    rc = PlxPutCImage(client, pPort, 0, xTrue);
	}
#endif
	else 
	{
		DEBUGF(0x40, (fd, 
			"PGIpvControl BadMatch: port id: %x unknown operation: %x\n",
			pPort->id, operation)
		);
		return BadMatch;
	}

	DEBUGF(0x40, (fd, 
		"PGIpvControl %s: port id: %x operation: %x\n",
		rc == Success ? "exit" : "error", pPort->id, operation)
	);
	return rc;
}


/*---------------------------------------------------------------------------*
 |"PGIpvGetLongPortAttribute"
 |
 |  Transfer vector routine to get a long attribute's value.
 |
 |    = 0 on success.
 |    = error on failure.
 *---------------------------------------------------------------------------*/
static int PGIpvGetLongPortAttribute(client, pPort, attribute)
    ClientPtr client;		/* -> client request data. */
    XvPortPtr pPort;		/* -> port request is on. */
    Atom      attribute;	/* =  attribute requested. */
{
	int			n_nvram;	/* Size of nvram area. */
	int			options;	/* Options code from hardware library. */
	XvddPortPtr pddPort;	/* Device-dependent storage for pPort. */
	int*		p_nvram;	/* Pointer to nvram area. */

	DEBUGF(0x40, (fd, "PGIpvGetLongPortAttribute entry: port id: %x attr: %x\n",
		pPort->id, attribute));

	pddPort = (XvddPortPtr)pPort->devPriv.ptr;

	if (attribute == attr_atoms.XvplxEncoding)
	{
	    SendXvGetLongPortAttributeReply(client, 4, &pPort->pEncoding->id);
	}
	else if (attribute == attr_atoms.XvplxAttributes)
	{
	    SendXvGetLongPortAttributeReply(client, l_attr_atoms, &attr_atoms);
	}
	else if (attribute == attr_atoms.XvplxBrightness)
	{
	    SendXvGetLongPortAttributeReply(client, 4, &pddPort->brightness);
	}
	else if (attribute == attr_atoms.XvplxControls)
	{
	    SendXvGetLongPortAttributeReply(client, l_ctrl_atoms, &ctrl_atoms);
	}
	else if (attribute == attr_atoms.XvplxCrop)
	{
        /* #### */
	}
	else if (attribute == attr_atoms.XvplxContrast)
	{
	    SendXvGetLongPortAttributeReply(client, 4, &pddPort->contrast);
	}
	else if (attribute == attr_atoms.XvplxHue)
	{
	    SendXvGetLongPortAttributeReply(client, 4, &pddPort->hue);
	}
  	else if (attribute == attr_atoms.XvplxNvramMain)
  	{
        Xvhw_Control(pddPort->phw, "NVRAM_GET_MAIN_SIZE", &n_nvram);
        Xvhw_Control(pddPort->phw, "NVRAM_GET_MAIN", &p_nvram);
	    SendXvGetLongPortAttributeReply(client, n_nvram, p_nvram);
	}
  	else if (attribute == attr_atoms.XvplxNvramZoom)
  	{
        Xvhw_Control(pddPort->phw, "NVRAM_GET_ZOOM_SIZE", &n_nvram);
        Xvhw_Control(pddPort->phw, "NVRAM_GET_ZOOM", &p_nvram);
	    SendXvGetLongPortAttributeReply(client, n_nvram, p_nvram);
	}
	else if (attribute == attr_atoms.XvplxSaturation)
	{
	    SendXvGetLongPortAttributeReply(client, 4, &pddPort->saturation);
	}
	else if (attribute == attr_atoms.XvplxQuerySignals)
	{
		Xvhw_Control(pddPort->phw, "OPTION_CODE", &options);
		switch ( options )
		{
		  default:
			SendXvGetLongPortAttributeReply(client, 
				lPGIpvBaseSignals, PGIpvBaseSignals);
			break;
		  case OPT_0_ZOOM2:
			SendXvGetLongPortAttributeReply(client, 
				lPGIpvZoom2BaseSignals, PGIpvZoom2BaseSignals);
			break;
		  case OPT_1:
			SendXvGetLongPortAttributeReply(client, 
				lPGIpvOpt1Signals, PGIpvOpt1Signals);
			break;
		  case OPT_1_ZOOM2:
			SendXvGetLongPortAttributeReply(client, 
				lPGIpvZoom2Opt1Signals, PGIpvZoom2Opt1Signals);
			break;
#ifdef __hpux
		  case OPT_2:
			SendXvGetLongPortAttributeReply(client, 
				lPGIpvOpt2Signals, PGIpvOpt2Signals);
			break;
		  case OPT_2_ZOOM2:
			SendXvGetLongPortAttributeReply(client, 
				lPGIpvZoom2Opt2Signals, PGIpvZoom2Opt2Signals);
			break;
#endif /* __hpux */
		}
	}
	else 
	{
		DEBUGF(0x40, (fd, 
			"PGIpvGetLongPortAttribute BadMatch: port id: %x attr: %x\n",
			pPort->id, attribute));
        return BadMatch;
	}

	DEBUGF(0x40, (fd, "PGIpvGetLongPortAttribute exit: port id: %x attr: %x\n",
		pPort->id, attribute));
	return Success;
}

/*----------------------------------------------------------------------------*
 |"PGIpvGetPortAttribute"
 |
 |  Transfer vector routine to get an attribute's value.
 *----------------------------------------------------------------------------*/
static int PGIpvGetPortAttribute(client, pPort, attribute, p_value)
    ClientPtr client;		/* -> client request data. */
    XvPortPtr pPort;		/* -> port request is on. */
    Atom      attribute;	/* =  attribute requested. */
    INT32     *p_value;	    /* -> where to store attribute value. */
{
	XvddPortPtr pddPort;	/* Device-dependent storage for pPort. */
	extern int	RasterOps;	/* RasterOps compatbility flag. */
	Chan_pv*	p_channels;	/* Channels structure for this adaptor. */

	DEBUGF(0x40, (fd, "PGIpvGetPortAttribute entry: port id: %x attr: %x\n",
		pPort->id, attribute));

	pddPort = (XvddPortPtr)pPort->devPriv.ptr;

	if (attribute == attr_atoms.XvplxEncoding)
	{
		*p_value = pPort->pEncoding->id;
	}
	else if (attribute == attr_atoms.XvplxNextPort)
	{
		p_channels = ((HwPGIpv*)(pddPort->phw))->p_channels;
		*p_value = p_channels->next_port++;
		p_channels->next_port = p_channels->next_port % pPort->pAdaptor->nPorts;
	}
	else if (attribute == attr_atoms.XvplxBrightness)
	{
		*p_value = pddPort->brightness;
	}
	else if (attribute == attr_atoms.XvplxChannel)
	{
		Xvhw_Control(pddPort->phw, "GET_CHANNEL", p_value);
	}
	else if (attribute == attr_atoms.XvplxCcube)
	{
		Xvhw_Control(pddPort->phw, "CCUBE", p_value);
	}
	else if (attribute == attr_atoms.XvplxCrop)
	{
      /* #### */
	}
	else if (attribute == attr_atoms.XvplxContrast)
	{
		*p_value = pddPort->contrast;
	}
	else if (attribute == attr_atoms.XvplxHue)
	{
		*p_value = pddPort->hue;
	}
	else if (attribute == attr_atoms.XvplxRasterOps)
	{
		*p_value = RasterOps;
	}
	else if (attribute == attr_atoms.XvplxSaturation)
	{
		*p_value = pddPort->saturation;
	}
	else if (attribute == attr_atoms.XvplxStatus)
	{
		*p_value = Xvhw_Get_Status(pddPort->phw);
		if (pPort->pDraw != NULL) *p_value |= XVS_Attached;
	}
	else if (attribute == attr_atoms.XvplxWid)
	{
		Xvhw_Control(pddPort->phw, "GET_WID", p_value);
	}
	else if (attribute == attr_atoms.XvplxZoom)
	{
		Xvhw_Control(pddPort->phw, "GET_ZOOM", p_value);
	}
	else if (attribute == attr_atoms.XvropsFlipHorizontal)
	{
		*p_value = 0;
	}
	else if (attribute == attr_atoms.XvropsFlipVertical)
	{
		*p_value = 0;
	}
	else if (attribute == attr_atoms.XvropsInvertFields)
	{
		*p_value = 0;
	}
	else 
	{
#ifndef _AIX
	   DEBUGF(0x40, (fd,
		"%s: PGIpvGetPortAttribute: port id: %x attribute %08x unknown\n",
			DEV_NAME(pPort), pPort->id, attribute));
#else /* !_AIX #### */
	DEBUGF(0x40, (fd, 
		"PGIpvGetPortAttribute exit: port id: %x attr: %08x unknown\n",
		pPort->id, attribute)
	);
#endif /* !_AIX #### */
        return BadMatch;
	}

#ifndef _AIX
    DEBUGF(0x40, (fd,
		"%s: PGIpvGetPortAttribute: port id: %x attribute %08x = 0x%08x (%d)\n",
		DEV_NAME(pPort), pPort->id, attribute, *p_value, *p_value)
	);
#else /* !_AIX #### */
	DEBUGF(0x40, (fd, 
		"PGIpvGetPortAttribute exit: port id: %x attr: %x = 0x%08x (%d)\n",
		pPort->id, attribute, *p_value, *p_value)
	);
#endif /* _AIX #### */
	return Success;
}


/*----------------------------------------------------------------------------* 
 |"PGIpvPrepareSurface"
 |
 |  Internal routine to prepare the frame buffer to receive video for an
 |  adaptor which is separate from the frame buffer (overlay/mixing of video).
 |
 |  This routine is called from XvddAttachPort.
 |
 |
 |    = 1 on success
 |    = 0 if error detected
 *----------------------------------------------------------------------------*/
static int PGIpvPrepareSurface(pPort, operation)
    XvPortPtr   pPort;          /* -> port to prepare surface for. */   
    int         operation;      /* =  operation requested (e.g., XvPSenable). */
{
	Xvhw_Drw_Area      drw_area;    /* Destination video area. */
	int                fill_h;		/* Height of enable memory to fill. */
	int                fill_w;		/* Width of enable memory to fill. */
	int                fill_x;		/* Starting x of enable memory to fill. */
	int                fill_y;		/* Starting y of enable memory to fill. */
	int                iBox;		/* # of box processing in region list. */
	int                nBoxes;		/* # of boxes in region list. */
	BoxPtr             pBox;		/* Box now processing in region list. */
	RegionPtr          pClip;		/* pDraw's current clipping region. */
	DrawablePtr		   pDraw;		/* Drawable associated with this port. */
	XvddPortPtr        pddPort;		/* Device-dependent port information. */
	RegionPtr          pDisable;    /* Pointer to regDisable or 0. */          
	HwPGIpv*       	   phw;     	/* Hardware object for this port. */
	RegionPtr   	   pSignal;     /* Pointer to 'regSignal'. */
	RegionPtr   	   pVideo;      /* Region on screen where video is video. */
	RegionRec          regDisable;  /* Area to have video disabled. */
	static RegionRec   regSignal =  /* Current signal area on screen. */
		{{0, 0, 0, 0}, 0};
	RegionRec		   regVideo;	/* Current area of video visibility. */
	Xvhw_Sig_Area      sig_area;    /* Source signal area. */
	short              xy[2];       /* New mix position for Xvhw_Control. */

	pddPort = (XvddPortPtr)pPort->devPriv.ptr;
	phw = (HwPGIpv*)pddPort->phw;
	pDraw = pPort->pDraw;
    Xvhw_Get_Drawable_Area(phw, &drw_area);


    /*  IF THIS IS A DISABLE REQUEST JUST CLEAR THE AREA CURRENTLY ENABLED. */

    if ( (operation==XvPSdisable) || (drw_area.direction==XvDrawableToSignal) )
    {
		pDisable = &pddPort->visible_area;
		if ( pDisable->extents.x2 != 0 )
		{
			pBox = REGION_RECTS(pDisable);
			nBoxes = (int)REGION_NUM_RECTS(pDisable);
			for (iBox = 0; iBox < nBoxes; iBox++, pBox++)
			{
		  
				fill_x = pBox->x1;
				fill_y = pBox->y1;
				fill_w = (pBox->x2 - fill_x);
				fill_h = (pBox->y2 - fill_y);
				Xvhw_Fill_Video_Enable(phw, fill_x, fill_y, fill_w, fill_h, 
					PGI_Fill_Remove);
			  
			}
			(* pDraw->pScreen->RegionUninit)(pDisable);
			pDisable->extents.x2 = 0;
		}
        return 1;
    }


    /*  PICK UP THE COMPOSITE CLIP LIST FOR THE REGION TO RECEIVE VIDEO.
     *   
     *  We do this based on the depth of the visual (i.e., its cfb or tc).
     */
#ifndef DECWINDOWS
	pClip = &((WindowPtr)pDraw)->clipList;
#else  /* DECWINDOWS */
	pClip = (pDraw->depth == 8) ? 
		((cfbPrivGCPtr)
		    pPort->pGC->devPrivates[cfbGCPrivateIndex].ptr)->pCompositeClip :
		((tfbPrivGCPtr)
			pPort->pGC->devPrivates[tfbGCPrivateIndex].ptr)->pCompositeClip;
#endif /* DECWINDOWS */


	/*  CALCULATE AREA OF SCREEN WHICH WILL RECIEVE THE VIDEO SIGNAL. */
	 
    Xvhw_Get_Signal_Area(phw, &sig_area);
	pSignal = &regSignal;
	pVideo = &regVideo;
    pSignal->extents.x1 = drw_area.left;
    pSignal->extents.y1 = drw_area.top;
    pSignal->extents.x2 = pSignal->extents.x1 +
        ((drw_area.width < sig_area.width) ? drw_area.width : sig_area.width);
    pSignal->extents.y2 = pSignal->extents.y1 +
        ((drw_area.height < sig_area.height) ?
        drw_area.height : sig_area.height);
	(* pDraw->pScreen->RegionInit)(pVideo, NullBox, 1);
	(* pDraw->pScreen->Intersect)(pVideo, pSignal, pClip);
	 

	/*  IF THERE IS CURRENTLY VIDEO BEING DISPLAYED FREE ITS STORAGE.  */

	if ( pddPort->visible_area.extents.x2 != 0 )
	{
		(* pDraw->pScreen->RegionUninit)(&pddPort->visible_area);
		pddPort->visible_area.extents.x2 = 0;
	}


	/*  ENABLE THE REGION WHICH WILL HAVE VISIBLE VIDEO. */

	pddPort->visible_area = *pVideo; 
	nBoxes = (int)REGION_NUM_RECTS(pVideo);
	pBox = REGION_RECTS(pVideo);         
	for ( iBox = 0; iBox < nBoxes; iBox++, pBox++ )
	{
  
		fill_x = pBox->x1;
		fill_y = pBox->y1;
		fill_w = (pBox->x2 - fill_x);
		fill_h = (pBox->y2 - fill_y);
		Xvhw_Fill_Video_Enable(phw, 
			fill_x, fill_y, fill_w, fill_h,
			(phw->p_draw == pDraw) ? PGI_Fill_Exposure : PGI_Fill_Enable);

	}
	phw->p_draw = pDraw;

	return Success;
}


/*----------------------------------------------------------------------------*
 |"PGIpvSaveScreen"
 |
 |  Transfer vector routine to enable or disable screen saver mode.
 *----------------------------------------------------------------------------*/
#ifdef _AIX
static Bool PGIpvSaveScreen(pAdaptor, mode, phase)
    XvAdaptorPtr pAdaptor;	/* Context for adaptor now being processed. */
    int 	     mode;		/* =  visibility of screen: 0 - off, 1 - on. */
    int          phase;
{
	if ( (mode == 4) && (phase < 2) ) mode = 1;
#else
static int PGIpvSaveScreen(pAdaptor, mode)
    XvAdaptorPtr pAdaptor;	/* Context for adaptor now being processed. */
    int 	     mode;		/* =  visibility of screen: 0 - off, 1 - on. */
{
#endif
	DEBUGF(0x20, (fd, "PGIpvSaveScreen: entry mode = %d\n", mode));
	Xvhw_Control( ((XvddPortPtr)(pAdaptor->pPorts[0].devPriv.ptr))->phw, 
		"VISIBILITY", mode);
	DEBUGF(0x20, (fd, "PGIpvSaveScreen: exit\n", mode));
}


/*----------------------------------------------------------------------------*
 |"PGIpvSetLongPortAttribute"
 |
 |  Transfer vector routine to set an attribute for a port.
 *----------------------------------------------------------------------------*/
static int PGIpvSetLongPortAttribute(client, pPort, attribute, length, value)
	ClientPtr client;
	XvPortPtr pPort;
	Atom      attribute;
	INT32     length;
	INT32     *value;
{
	int       	ccube_present;
	XvddPortPtr pddPort;

	DEBUGF(0x40, (fd, 
		"PGIpvSetPortAttribute entry: port id: %x attr: %x length: %d\n",
		pPort->id, attribute, length)
	);

	pddPort = (XvddPortPtr)pPort->devPriv.ptr;

  	if (attribute == attr_atoms.XvplxNvramMain)
  	{
        Xvhw_Control(pddPort->phw, "NVRAM_SET_MAIN", value);
	}
  	else if (attribute == attr_atoms.XvplxNvramZoom)
  	{
        Xvhw_Control(pddPort->phw, "NVRAM_SET_ZOOM", value);
	}
  	else if (attribute == attr_atoms.XvplxTableComp)
  	{
		Xvhw_Control(pddPort->phw, "CCUBE", &ccube_present);
#ifdef GEORGE_FIXED
		if ( !ccube_present ) return BadMatch;
#else
		if ( !ccube_present ) return Success;
#endif
	    Xvplx_Set_Compression_Table(pddPort->phw, length, value);
  	}
  	else if (attribute == attr_atoms.XvplxTableDecomp)
  	{
		Xvhw_Control(pddPort->phw, "CCUBE", &ccube_present);
		if ( !ccube_present ) 
		{
			DEBUGF(0x40, (fd, 
				"PGIpvSetPortAttribute no ccube exit: port id: %x attr: %x \n",
				pPort->id, attribute)
			);
#ifdef GEORGE_FIXED
			return BadMatch;
#else
			return Success;
#endif
		}
	    Xvplx_Set_Decompression_Table(pddPort->phw, length, value);
  	}

	XvdiSendPortNotify(pPort, attribute, ((int)value & 3) ? 0xdeadbeef: *value);

	DEBUGF(0x40, (fd, 
		"PGIpvSetPortAttribute exit: port id: %x attr: %x \n",
		pPort->id, attribute)
	);
	return Success;
}

/*----------------------------------------------------------------------------*
 |"PGIpvSetPortAttribute"
 |
 |  Transfer vector routine to set an attribute for a port.
 *----------------------------------------------------------------------------*/
static int PGIpvSetPortAttribute(client, pPort, attribute, value)
	ClientPtr client;
	XvPortPtr pPort;
	Atom      attribute;
	INT32     value;
{
	Xvhw_Drw_Area dst_area;   /* PIP destination area. */
	int           iEncoding;
	Xvhw_Levels   levels;     /* PIP color levels. */
	unsigned char lut[256];
	int           on_off;
	XvddPortPtr   pddPort;
	XvEncodingPtr pEncode;
	Xvhw_Signal   signal;     /* Signal type and timing. */
	extern int    RasterOps;  /* RasterOps compatibilty flag. */

	DEBUGF(0x40, (fd, 
		"PGIpvSetPortAttribute entry: port id: %x attr: %x value: 0x%08x(%d)\n",
		pPort->id, attribute, value, value)
	);

	pddPort = (XvddPortPtr)pPort->devPriv.ptr;


	/* FIND THE ATTRIBUTE AND SET IT; XXX --- MORE LATER */

	if (attribute == attr_atoms.XvplxEncoding)
	{
		pEncode = pPort->pEncodings;
		for (iEncoding = 0; iEncoding < pPort->nEncodings; iEncoding++)
		{
			if (pEncode->id == (XvEncodingID)value)
			{

				/* SAVE NEW ENCODING ID */

				pPort->pEncoding = pEncode;

				Xvhw_Get_Signal_Type(pddPort->phw, &signal);  
				if ( RasterOps ) 
				{
					rops_to_new_encoding( &value );
				}
				signal.timing = value & XV_SOURCE_TIMING; 
				signal.type = value & XV_SOURCE_CONNECTION; 
				if (pddPort->gray) signal.type |= XV_GRAY_SCALE;   
				Xvhw_Suspend(pddPort->phw, 1);
				Xvhw_Set_Signal_Type(pddPort->phw, &signal);
				Xvhw_Resume(pddPort->phw, 0);
				break;
			}
			pEncode++;
		}
		if (iEncoding>=pPort->nEncodings) 
		{
			DEBUGF(0x40, (fd, 
				"PGIpvSetPortAttribute bad encoding: port id: %x attr: %x\n",
				pPort->id, attribute)
			);
			return 0;
		}
	}
	else if (attribute == attr_atoms.XvplxBrightness)
	{
		if (value < -1000)      { pddPort->brightness = -1000; }
		else if (value > 1000)  { pddPort->brightness = 1000;  }
		else                    { pddPort->brightness = value; }
		Xvhw_Get_Levels(pddPort->phw, &levels);
		levels.brightness = pddPort->brightness;
		Xvhw_Suspend(pddPort->phw, 1);
		Xvhw_Set_Levels(pddPort->phw, &levels);
		Xvhw_Resume(pddPort->phw, 0);
	}
	else if (attribute == attr_atoms.XvplxChannel)
	{
		Xvhw_Control(pddPort->phw, "SET_CHANNEL", value);
	}
	else if (attribute == attr_atoms.XvplxContrast)
	{
		if (value < -1000)      { pddPort->contrast = -1000; }
		else if (value > 1000)  { pddPort->contrast = 1000;  }
		else                    { pddPort->contrast = value; }
		Xvhw_Get_Levels(pddPort->phw, &levels);
		levels.contrast = pddPort->contrast;
		Xvhw_Suspend(pddPort->phw, 1);
		Xvhw_Set_Levels(pddPort->phw, &levels);
		Xvhw_Resume(pddPort->phw, 0);
	}

	else if (attribute == attr_atoms.XvplxCrop)
	{
		Xvhw_Suspend(pddPort->phw, 1);
        Xvhw_Control(pddPort->phw, "CROP", value);
		Xvhw_Resume(pddPort->phw, 0);
	}

	else if (attribute == attr_atoms.XvplxHue)
	{
		if (value < -1000)      { pddPort->hue = -1000; }
		else if (value > 1000)  { pddPort->hue = 1000;  }
		else                    { pddPort->hue = value; }

		Xvhw_Get_Levels(pddPort->phw, &levels);
		levels.hue = pddPort->hue;
		Xvhw_Suspend(pddPort->phw, 1);
		Xvhw_Set_Levels(pddPort->phw, &levels);
		Xvhw_Resume(pddPort->phw, 0);
	}
  	else if (attribute == attr_atoms.XvplxNvramControl)
  	{
        Xvhw_Control(pddPort->phw, "NVRAM_CONTROL", value);
  	}
	else if (attribute == attr_atoms.XvplxRasterOps)
	{
		set_rasterops( pPort, value );
	}
	else if (attribute == attr_atoms.XvplxSaturation)
	{
		if (value < -1000)      { pddPort->saturation = -1000; }
		else if (value > 1000)  { pddPort->saturation = 1000;  }
		else                    { pddPort->saturation = value; }
		
		Xvhw_Get_Levels(pddPort->phw, &levels);
		levels.saturation = pddPort->saturation;
		Xvhw_Suspend(pddPort->phw, 1);
		Xvhw_Set_Levels(pddPort->phw, &levels);
		Xvhw_Resume(pddPort->phw, 0);
	}
	else if (attribute == attr_atoms.XvplxTest)	/* #### FOR TESTING STUFF */
	{
		Xvhw_Control(pddPort->phw, "TEST", value);
	}
	else if (attribute == attr_atoms.XvplxZoom)	
	{
		PGIpvSetZoom(pPort->pAdaptor, value);
	}
	else if ( (attribute == attr_atoms.XvropsInvertFields)   ||
			  (attribute == attr_atoms.XvropsFlipHorizontal) ||
			  (attribute == attr_atoms.XvropsFlipVertical)   )
	{
	}
	else
	{
		DEBUGF(0x40, (fd, 
			"PGIpvSetPortAttribute BadMatch: port id: %x unknown attr: %x \n",
			pPort->id, attribute)
		);
		return BadMatch;
	}

	XvdiSendPortNotify(pPort, attribute, value);

	DEBUGF(0x40, (fd, 
		"PGIpvSetPortAttribute exit: port id: %x attr: %x \n",
		pPort->id, attribute)
	);
	return Success;
}


/*----------------------------------------------------------------------------*
 |"PGIpvSetZoom"
 |
 |  Set the zoom factor for an adaptor. Reprogram each port with zoom factor.
 *----------------------------------------------------------------------------*/
static void PGIpvSetZoom(pAdaptor, value)
	XvAdaptorPtr pAdaptor;
	INT32     	 value;
{
	int			  iEncode;	/* Element in encoding array accessing. */
	int			  iPort;	/* # of port currently processing. */
	XvEncodingID  old_id;	/* Old encoding id for port. */
	int			  options;	/* Configuration of card we are using. */
	XvddPortPtr   pddPort;	/* Device dependent port information. */
	XvEncodingPtr pEncode;  /* Encoding adding pScreen ptr to. */
	XvPortPtr	  pPort;	/* Port currently processing. */

	pPort = pAdaptor->pPorts;
	for ( iPort = 0; iPort < pAdaptor->nPorts; iPort++, pPort++ )
	{
		pddPort = (XvddPortPtr)pPort->devPriv.ptr;

		if (value < 1)       { value = 1; }
		else if (value > 4)  { value = 4; }
		Xvhw_Suspend(pddPort->phw, 1);
		Xvhw_Control(pddPort->phw, "SET_ZOOM", value);
		Xvhw_Resume(pddPort->phw, 0);

		Xvhw_Control(pddPort->phw, "OPTION_CODE", &options);
		switch ( options )
		{
		  default:
			pPort->nEncodings = nPGIpvBaseEncodings;
			pPort->pEncodings = PGIpvBaseEncodings;
			pPort->type = XvInputMask;
			break;
		  case OPT_0_ZOOM2:
			pPort->nEncodings = nPGIpvZoom2BaseEncodings;
			pPort->pEncodings = PGIpvZoom2BaseEncodings;
			pPort->type = XvInputMask;
			break;
		  case OPT_1:
			pPort->nEncodings = nPGIpvOpt1Encodings;
			pPort->pEncodings = PGIpvOpt1Encodings;
			pPort->type = XvInputMask|XvOutputMask;
			break;
		  case OPT_1_ZOOM2:
			pPort->nEncodings = nPGIpvZoom2Opt1Encodings;
			pPort->pEncodings = PGIpvZoom2Opt1Encodings;
			pPort->type = XvInputMask|XvOutputMask;
			break;
#ifdef __hpux
		  case OPT_2:
			pPort->nEncodings = nPGIpvOpt2Encodings;
			pPort->pEncodings = PGIpvOpt2Encodings;
			pPort->type = XvInputMask|XvOutputMask;
			break;
		  case OPT_2_ZOOM2:
			pPort->nEncodings = nPGIpvZoom2Opt2Encodings;
			pPort->pEncodings = PGIpvZoom2Opt2Encodings;
			pPort->type = XvInputMask|XvOutputMask;
			break;
#endif /* __hpux */
		}
		pEncode = pPort->pEncodings;
		for (iEncode = 0; iEncode < pPort->nEncodings; iEncode++)
		{
			pEncode->pScreen = pAdaptor->pScreen;
			pEncode++;
		}


		/*  SET ENCODING ID TO ITS EQUIVALENT IN THIS ZOOM FACTOR. */

		old_id = pPort->pEncoding->id;
		pEncode = pPort->pEncodings;
		for (iEncode = 0; iEncode < pPort->nEncodings; iEncode++)
		{
			if (pEncode->id == (XvEncodingID)old_id)
			{
				pPort->pEncoding = pEncode;
				break;
			}
			pEncode++;
		}
	}
}


/*----------------------------------------------------------------------------*
 |"RasterOps Compatibilty Code"
 |
 |  Code to allow old RasterOps applications to run with new libraries.
 *----------------------------------------------------------------------------*/

void new_to_rops_encoding( p_encoding )
	int*	p_encoding;	/* -> encoding to be translated. */
{
	int		iEncoding;	/* Element in new translation vector accessing. */

	for ( iEncoding = 0; iEncoding < nTranslates; iEncoding++ )
	{
		if ( *p_encoding == translate_encoding[iEncoding].new )
		{
			*p_encoding = translate_encoding[iEncoding].rops;
			return;
		}
	}
}

void rops_to_new_encoding( p_encoding )
	int*	p_encoding;	/* -> encoding to be translated. */
{
	int		iEncoding;	/* Element in translation vector accessing. */

	for ( iEncoding = 0; iEncoding < nTranslates; iEncoding++ )
	{
		if ( *p_encoding == translate_encoding[iEncoding].rops )
		{
			*p_encoding = translate_encoding[iEncoding].new;
			return;
		}
	}
}

set_rasterops(pPort, enableRasterOps)
	XvPortPtr	pPort;
{
	int			  iEncode;	/* Element in encoding list examining. */
	XvEncodingPtr pEncode;	/* Encoding record translating id of. */
	
	switch ( enableRasterOps )
	{
	  case 0:
		if ( RasterOps )
		{
			pEncode = pPort->pEncodings;
			for ( iEncode = 0; iEncode < pPort->nEncodings; iEncode++ )
			{
				rops_to_new_encoding(&pEncode->id);
				pEncode++;
			}
			RasterOps = 0;
		}
		break;
	  default:
		if ( !RasterOps )
		{
			pEncode = pPort->pEncodings;
			for ( iEncode = 0; iEncode < pPort->nEncodings; iEncode++ )
			{
				new_to_rops_encoding(&pEncode->id);
				pEncode++;
			}
			RasterOps = 1;
		}
		break;
	}
}
