/*
 *   Copyright (c) 1994 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 */
/*
** File:
**
**  xvhw_PGIpv.c: Implementation of hardware level xv extension support for
**                Parallax PowerVideo overlay card.
**
** Author:
**
**   Andy Goodrich (Parallax)
**
** Revisions:
**
**   25.01.94 Goodrich
**     - Initial coding.
**
** $Log: xvhw_PGIpv.c,v $
 * Revision 1.16  1996/07/25  18:40:52  dave
 * fixed one off enable/disable
 *
 * Revision 1.15  1996/07/22  19:28:49  dave
 * fixed multiboard probs.
 *
 * Revision 1.14  96/04/19  23:50:10  23:50:10  dist (Software Distribution Account)
 * added call to ccube_init to do compress at server startup
 * 
 * Revision 1.13  1996/04/03  22:23:52  dist
 * fixed various things
 *
 * Revision 1.12  95/12/28  05:06:03  05:06:03  dist ()
 * Andy Goodrich: Removed kludge in O_HWL_O_HWL call that was used to 
 *                bring up hedge.
 * 
 * Revision 1.11  1995/12/22  19:17:22  dist
 * Andy Goodrich: Changed call to plxv_disp_blank to use transfer vector
 *                entry O_PV700_visibility in screen saver code.
 *
 * Revision 1.10  1995/12/11  15:09:48  dist
 * Andy Goodrich: Checkin / merging of hedge changes.
 *
 * Revision 1.9  95/12/11  15:02:07  15:02:07  dist ()
 * Andy Goodrich: Checkin / merging of hedge changes.
 * 
 * Revision 1.7  95/07/21  15:52:01  15:52:01  dist ()
 * Andy Goodrich: Added a chmod(666) to plxv0 file when it is stored.
 * 
 * Revision 1.6  95/07/18  12:29:39  12:29:39  dist ()
 * Andy Goodrich: Added code to set plxv0 nvram file to have read-write 
 *                permission to everyone and to produce an error if it
 *                is not write-able.
 * 
 * Revision 1.5  95/07/12  13:36:34  13:36:34  dist ()
 * Andy Goodrich: added ability to get a resource for individual adaptors when
 *                there is more than one card present.
 * 
 * Revision 1.4  95/06/14  11:55:19  11:55:19  dist ()
 * Andy Goodrich: Changes to use linebytes rather than xsize for strides.
 * 
** Revision 1.3  95/06/01  11:05:12  11:05:12  dist ()
** Andy Goodrich: Added kludge to handle release 1 on AIX platform.
**                (We subtract off current positon of drawable_area from
**                x,y to get real position of zoomed 2 image for 
**                get_image routine.
** 
** Revision 1.2  95/05/25  17:03:05  17:03:05  dist ()
** Andy Goodrich - Combined HP/AIX version for HP 9.1 code freeze.
** 
** Revision 1.1  95/05/12  13:44:58  13:44:58  dist ()
** Initial revision
** 
** Revision 1.1  1995/04/17  19:49:34  dist
** Initial revision
**
*/
#include <stdio.h>                  /* Standard io defs, e.g., stderr. */
#include <sys/types.h>              /* General type defs. */
#include <sys/param.h>              /* General system parameters and limits. */
#include <sys/time.h>               /* General timer parameters. */

#include <sys/buf.h>                /* Input / Output buffer defs. */
#include <sys/errno.h>              /* Kernel error defs. */
#include <sys/mman.h>               /* Virtual memory related macros. */

#include "Xvddx_PGI.h"              /* Ddx specific user definitions. */
#include "objects_internal.h"		/* Pick up hardware interface objects. */
#include "xvhw_PGIpv.h"             /* Definition for Pip Object. */
#ifdef _AIX
#include "pvioctl.h"                /* Pick up PV_POWER. */
#endif /* _AIX */


/*  XV_DIAG bit definitions:
 *
 *    0x01 - normal one-shot diagnostics
 *    0x80 - server initialization tracing
 */
#define XV_DIAG 1  
#include "xvdebug.h"                /* Pick up debugging macros. */


enum {                          /* Values for direction field: */
    XvDrawableToSignal = 0,
    XvImageToDrawable,
    XvSignalToDrawable
};


/*  Wait codes for pv700_wait_for_change:
 */
#define WAIT_FOR_ON  1
#define WAIT_FOR_OFF 2

int debug_flags = 0; /* For dynamically debugging via XV_PLX_TEST attribute. */

enum {                  /* Miscellaneous device parameters: */
    NTSC_BLANK       = 43,
    NTSC_HEIGHT      = 525,
    NTSC_HEIGHT_HALF = 241,
    NTSC_RATE        = 60,
    NTSC_SLEEP       = 35000,
    NTSC_X_ZERO      = 0,
    NTSC_Y_ZERO      = 44,
    NTSC_WIDTH       = 640,
    PAL_BLANK        = 50,
    PAL_HEIGHT       = 625,
    PAL_HEIGHT_HALF  = 287,
    PAL_RATE         = 60,
    PAL_SLEEP        = 50000,
    PAL_WIDTH        = 768,
    PAL_X_ZERO       = 0,
    PAL_Y_ZERO       = 50,
};


static O_Draw_Gc gc_disable = {   /* Drawing context for disabling visibility */
    0,   				/* Color for accelerator operations. */
    0,  				/* Background color to use with 1-bit ops. */
    0,   				/* Foreground color to use with 1-bit ops. */
    0,   				/* = 1 if using transparency mode, 0 if not. */
    0,   				/* Byte to read 8-bit ops (0=r,1=g,2=b,3=tags)*/
    PLXV_ROP_COPY,      /* Raster operation bits. */
    0,   				/* Read mask for comparing wid bits w/value. */
    0,   				/* Value to compare w/wid bits in word. */
    0,   				/* Write mask for setting wid values. */
    0,   				/* Value to write in to wid bits of word. */
    PLX_VISIBLE << 24   /* 32-bit write mask for data written. */
};

static O_Draw_Gc gc_remove = {   /* Drawing context for removing a window. */
    0,   				/* Color for accelerator operations. */
    0,  				/* Background color to use with 1-bit ops. */
    0,   				/* Foreground color to use with 1-bit ops. */
    0,   				/* = 1 if using transparency mode, 0 if not. */
    0,   				/* Byte to read 8-bit ops (0=r,1=g,2=b,3=tags)*/
    PLXV_ROP_COPY,      /* Raster operation bits. */
    0,   				/* Read mask for comparing wid bits w/value. */
    0,   				/* Value to compare w/wid bits in word. */
    0,   				/* Write mask for setting wid values. */
    0,   				/* Value to write in to wid bits of word. */
    (PLX_VISIBLE << 24) | PLXV_WMASK_24 /* 32-bit write mask for data written */
};


/*
 *  FORWARD REFERENCES:
 */
static int  pv700_control();               /* Perform control function. */
static void pv700_dump();                  /* Dump pip object contents. */
static int  pv700_fill_true_color();       /* Fill true color memory. */
static int  pv700_fill_video_enable();     /* Fill video enable memory. */
static void pv700_get_levels();            /* Get the picture levels. */
static void pv700_get_drawable_area();     /* Get the pip output area. */
static int  pv700_get_image();             /* Get image data. */
static void pv700_get_signal_area();       /* Get the pip signal area. */
static void pv700_get_signal_type();       /* Get the pip signal(e.g., rgb). */
static int  pv700_get_true_color();        /* Get true color data. */
static int  pv700_init();                  /* Initialize pip. */
static int  pv700_off();                   /* Turn off pip. */
static int  pv700_on();                    /* Turn on pip. */
static int  pv700_one_shot();              /* Turn on pip for frame. */
static int  pv700_put_image();             /* Put image data. */
static int  pv700_resume();                /* Pop pip "off" stack. */
static void pv700_set_levels();            /* Set the picture levels. */
static void pv700_set_drawable_area();     /* Set pip output origin,size&rect*/
static void pv700_set_signal_area();       /* Set the pip signal area. */
static void pv700_set_signal_type();       /* Set the pip signal(e.g., rgb). */
static int  pv700_status();                /* Return pip operational status. */
static int  pv700_suspend();               /* Push pip "off" stack. */
static void pv700_terminate();             /* Terminate use of a pip object. */
HwPGIpv* 	xv_pv_open();                  /* Open a device. */

#ifdef _AIX
#define XV_ETC_PREFIX "/usr/lpp/parallax/etc"
#else /* !_AIX */
#define XV_ETC_PREFIX "/opt/parallax/etc"
#endif /* _AIX */


/*----------------------------------------------------------------------------*
 |"pv700_control"
 | 
 |  Perform a control function on a pip object. Note the string tests in this
 |  routine are in order of expected occurence for optimization, not in 
 |  alphabetical order.
 |
 |      = 1 if success
 |      = 0 if failure
 *----------------------------------------------------------------------------*/
static pv700_control(p_me, command, arg)
    HwPGIpv*    p_me;      /* -> control block for this xv instance. */
    char*       command;   /* -> command to perform. */
    caddr_t     arg;       /* =  optional argument. */
{
    Xvhw_Drw_Area*   area;          /* Current output area. */
	FILE*			 fd;			/* File descriptor for resource file. */
    short*           halfword;      /* Alias to point to halfwords. */
    int              input_status;  /* Input status register value. */
    int              rc;            /* Return code from calls to routines. */
    int              reg_value;     /* Value of hardware register. */
	char			 rsrc[100]; 	/* Name of resource file. */
    struct timeval   to;            /* Time stamp for performing wait. */


    if ( !strcmp(command, "OPTION_CODE") )
    {
		if ( p_me->p_context->p_outputs[PORT_A] ) 
		{
        	switch( p_me->zoom )
			{
			  default: *(int*)arg = OPT_1; 		 break;
			  case 2:  *(int*)arg = OPT_1_ZOOM2; break;
			  case 4:  *(int*)arg = OPT_1_ZOOM4; break;
			}
		}
		else if ( p_me->p_context->p_inputs[PORT_B] )
		{
        	switch( p_me->zoom )
			{
			  default: *(int*)arg = OPT_2;       break;
			  case 2:  *(int*)arg = OPT_2_ZOOM2; break;
			  case 4:  *(int*)arg = OPT_2_ZOOM4; break;
			}
		}
		else
		{
        	switch( p_me->zoom )
			{
			  default: *(int*)arg = 0;           break;
			  case 2:  *(int*)arg = OPT_0_ZOOM2; break;
			  case 4:  *(int*)arg = OPT_0_ZOOM4; break;
			}
		}
    }

	else if ( !strcmp(command, "CCUBE") )
    {
        *((int *)arg) = p_me->ccube;
    }

    else if ( !strcmp(command, "SEPARATE_FB") )
    {
        *((int *)arg) = 1;
    }

    else if ( !strcmp(command, "CROP") )
    {
        p_me->crop = (int)arg;
    }

    else if ( !strcmp(command, "ERASE") )
    {
        pv700_fill_video_enable(p_me, 0, 0, 
            p_me->public.video_pitch, p_me->public.video_height, 0x00);
    }

    else if ( !strcmp(command, "GET_CHANNEL") )
    {
        *((int *)arg) = 0;
        if ( p_me->p_channels->o_chan == p_me ) *((int *)arg) = 3;
        if ( p_me->p_channels->v_chan == p_me ) *((int *)arg) = 1;
        if ( p_me->p_channels->w_chan == p_me ) *((int *)arg) = 2;
	}

    else if ( !strcmp(command, "GET_PORT#") )
    {
        *((int *)arg) = p_me->public.adaptor_port_number;
    }

    else if ( !strcmp(command, "GET_WID") )
    {
        *(int*)arg = p_me->wid;
    }

    else if ( !strcmp(command, "GET_ZOOM") )
    {
        *(int*)arg = p_me->zoom;
    }

    else if ( !strcmp(command, "NVRAM_CONTROL") )
    {
        switch( (int)arg )
        {
          case XV_PLX_NVRAM_READ:
			sprintf(rsrc, "%s/%s", XV_ETC_PREFIX, 
				&p_me->p_context->p_mmap->name[strlen("/dev/")]);
			if ( (fd = fopen(rsrc, "r")) == 0 ) return 0;
            O_Rsrc_load_from_file(p_me->p_context->p_rsrc, fd);
			fclose(fd);
            break;
          case XV_PLX_NVRAM_READ_INIT:
			sprintf(rsrc, "%s/%s", XV_ETC_PREFIX, 
				&p_me->p_context->p_mmap->name[strlen("/dev/")]);
			if ( (fd = fopen(rsrc, "r")) == 0 ) return 0;
            O_Rsrc_load_from_file(p_me->p_context->p_rsrc, fd);
			fclose(fd);
            O_Rsrc_store_to_context(p_me->p_context->p_rsrc, p_me->p_context);
          case XV_PLX_NVRAM_INIT_FROM:
            O_PV700_init_hw(p_me->p_context, 0);
            break;
          case XV_PLX_NVRAM_WRITE:
			O_Rsrc_load_from_context(p_me->p_context->p_rsrc, p_me->p_context);
			sprintf(rsrc, "%s/%s", XV_ETC_PREFIX, 
				&p_me->p_context->p_mmap->name[strlen("/dev/")]);
			if ( (fd = fopen(rsrc, "w")) == 0 )
			{
				ERRORF((fd, "Unable to write to context file %s\n", rsrc));
				return 0;
			}
			O_Rsrc_store_to_file(p_me->p_context->p_rsrc, fd);
			chmod(rsrc, 0666);
			fclose(fd);
            break; 
		  case XV_PLX_NVRAM_INIT_DISP:
			O_PV700_init_disp(p_me->p_context);
			O_PV700_init_pll(p_me->p_context, 0);
		    break;	
        }
    }

    else if ( !strcmp(command, "NVRAM_GET_MAIN_SIZE") )
    {
        *((int*)arg) = sizeof(XV_controller_nvram);
    }

    else if ( !strcmp(command, "NVRAM_GET_MAIN") )
    {
        *((XV_controller_nvram**)arg) = &p_me->p_context->main;
    }

    else if ( !strcmp(command, "NVRAM_GET_ZOOM_SIZE") )
    {
        *((int*)arg) = sizeof(XV_zoom_nvram);
    }

    else if ( !strcmp(command, "NVRAM_GET_ZOOM") )
    {
        *((XV_zoom_nvram**)arg) = &p_me->p_context->zoom[0];
    }

    else if ( !strcmp(command, "NVRAM_SET_MAIN") )
    {
        p_me->p_context->main = *(XV_controller_nvram*)arg;
    }

    else if ( !strcmp(command, "NVRAM_SET_ZOOM") )
    {
        p_me->p_context->zoom[0] = *(XV_zoom_nvram*)arg;
    }

    else if ( !strcmp(command, "SET_CHANNEL") )
    {
		switch ( (int)arg )
		{
		  case 0:  p_me->chan_affinity = 0;   break;
		  case 1:  p_me->chan_affinity = 'v'; break;
		  case 2:  p_me->chan_affinity = 'w'; break;
		}
    }

    else if ( !strcmp(command, "SET_ZOOM") )
    {
        if ( p_me->zoom != (int)arg )
		{
			p_me->zoom = (int)arg;
			if ( p_me->public.adaptor_port_number == 0 )
			{
				O_PV700_set_zoom(p_me->p_context, (int)arg);
				O_Draw_set_gc(p_me->p_context->p_draw, &gc_remove);
				O_Draw_fill_box(p_me->p_context->p_draw, 
					0, 0, p_me->public.video_pitch, p_me->public.video_height);
			}
		}
    }

    else if ( !strcmp(command, "TEST") )
    {
		switch( (int)arg >> 24 )
		{
		  case 0:
			O_Output_set_register(p_me->p_context->p_outputs[((int)arg)>>24],
				((int)arg & 0x0000ff00) >> 8, (int)arg & 0xff);
			break;
		  default:
			O_Input_set_register(p_me->p_context->p_inputs[((int)arg)>>24],
				((int)arg & 0x0000ff00) >> 8, (int)arg & 0xff);
			break;
		}
    }

    else if ( !strcmp(command, "VISIBILITY") )
    {
#ifdef _AIX
        ioctl(p_me->p_context->fd_ccube, PV_POWER, &arg);
#else
        O_PV700_visibility(p_me->p_context, (int)arg ? 0 : 1);
#endif
    }

    else
    {
        return 0;
    }

    return 1;
}

/*----------------------------------------------------------------------------*
 |"pv700_dump"
 | 
 |  Dump the contents of a pip object to a file
 |
 *----------------------------------------------------------------------------*/
static void pv700_dump(p_me, fd)
    HwPGIpv*    p_me;      /* -> control block for this xv instance. */
    FILE*       fd;        /* -> file descriptor to dump to. */
{

    if ( fd == 0 ) fd = stderr;

    fprintf(fd, "Pip Object at address 0x%08x:\n", p_me);

    fprintf(fd, "  Flags:");
    if ( p_me->public.attr.debug )           fprintf(fd, " DEBUG");
    if ( p_me->public.attr.stop_for_update ) fprintf(fd, " STOP_FOR_UPDATE");
    if ( p_me->public.attr.separate_fb )     fprintf(fd, " SEPARATE_FB");
    fprintf(fd, "\n");

    fprintf(fd, "Transfer vector:\n");
    fprintf(fd, "    Routine:             Expected:  Actual:\n");
    fprintf(fd, "    -------              --------   ------ \n");
    fprintf(fd, "    control              0x%08x 0x%08x\n", 
        pv700_control, p_me->public.control);
    fprintf(fd, "    dump                 0x%08x 0x%08x\n", 
        pv700_dump, p_me->public.dump);
    fprintf(fd, "    fill_true_color      0x%08x 0x%08x\n", 
        pv700_fill_true_color, p_me->public.fill_true_color);
    fprintf(fd, "    fill_video_enable    0x%08x 0x%08x\n",
        pv700_fill_video_enable, p_me->public.fill_video_enable);
    fprintf(fd, "    get_levels           0x%08x 0x%08x\n", 
        pv700_get_levels, p_me->public.get_levels);
    fprintf(fd, "    get_drawable_area 0x%08x 0x%08x\n", 
        pv700_get_drawable_area, p_me->public.get_drawable_area);
    fprintf(fd, "    get_image            0x%08x 0x%08x\n", 
        pv700_get_image, p_me->public.get_image);
    fprintf(fd, "    get_signal_area      0x%08x 0x%08x\n", 
        pv700_get_signal_area, p_me->public.get_signal_area);
    fprintf(fd, "    get_signal_type      0x%08x 0x%08x\n", 
        pv700_get_signal_type, p_me->public.get_signal_type);
    fprintf(fd, "    initialize           0x%08x 0x%08x\n", 
        pv700_init, p_me->public.initialize);
    fprintf(fd, "    off                  0x%08x 0x%08x\n", 
        pv700_off, p_me->public.off);
    fprintf(fd, "    on                   0x%08x 0x%08x\n", 
        pv700_on, p_me->public.on);
    fprintf(fd, "    one_shot             0x%08x 0x%08x\n", 
        pv700_one_shot, p_me->public.one_shot);
    fprintf(fd, "    put_image            0x%08x 0x%08x\n", 
        pv700_put_image, p_me->public.put_image);
    fprintf(fd, "    resume               0x%08x 0x%08x\n", 
        pv700_resume, p_me->public.resume);
    fprintf(fd, "    set_levels           0x%08x 0x%08x\n", 
        pv700_set_levels, p_me->public.set_levels);
    fprintf(fd, "    set_drawable_area 0x%08x 0x%08x\n", 
        pv700_set_drawable_area, p_me->public.set_drawable_area);
    fprintf(fd, "    set_signal_area      0x%08x 0x%08x\n", 
        pv700_set_signal_area, p_me->public.set_signal_area);
    fprintf(fd, "    set_signal_type      0x%08x 0x%08x\n", 
        pv700_set_signal_type, p_me->public.set_signal_type);
    fprintf(fd, "    status               0x%08x 0x%08x\n", 
        pv700_status, p_me->public.status);
    fprintf(fd, "    suspend              0x%08x 0x%08x\n", 
        pv700_suspend, p_me->public.suspend);
    fprintf(fd, "    terminate            0x%08x 0x%08x\n", 
        pv700_terminate, p_me->public.terminate);

}

/*----------------------------------------------------------------------------*
 |"pv700_fill_true_color"
 | 
 |  Fill a rectangular region in the true color memory.
 *----------------------------------------------------------------------------*/
static pv700_fill_true_color(p_me, left, top, width, height, color)
    HwPGIpv*    p_me;       /* -> control block for this xv instance. */
    int         left;       /* =  left side of region to fill in. */
    int         top;        /* =  top side of region to fill in. */
    int         width;      /* =  width of region to fill in. */
    int         height;     /* =  height of region to fill in. */
    u_int       color;      /* =  color to fill pixels with. */
{
	O_Draw_Gc	gc;		/* Drawing context for filling in true color. */

	gc = gc_remove;
	gc.write_mask = PLXV_WMASK_24;
	gc.color = color;
	O_Draw_set_gc(p_me->p_context->p_draw, &gc);
	O_Draw_fill_box(p_me->p_context->p_draw, left, top, width, height);
}

/*----------------------------------------------------------------------------*
 |"pv700_fill_video_enable"
 | 
 |  Fill a rectangular region in the video enable memory. Note we must fill
 |  both the tag memory and true color memory if enable is zero since this 
 |  hardware always sums the output of its frame buffer with the host's 
 |  frame buffer. If enable is one we fill only the tag memory since there
 |  may already be active video where we are filling.
 |
 |  Note that we have conveniently chosen the values of p_me->wid to be
 |  the tag value to use.
 |
 |      = 1 if success
 |      = 0 if rectangle is not within boundaries of video enable memory
 *----------------------------------------------------------------------------*/
static pv700_fill_video_enable(p_me, left, top, width, height, type)
    HwPGIpv*    p_me;       /* -> control block for this xv instance. */
    int         left;       /* =  left side of region to fill in. */
    int         top;        /* =  top side of region to fill in. */
    int         width;      /* =  width of region to fill in. */
    int         height;     /* =  height of region to fill in. */
    int         type;       /* =  enable if non-zero, disable if zero. */  
{
    int     zoom_round; /* Amount to add to round zoom numbers. */
    int     zoom_shift; /* Amount to shift values for zoom. */

    zoom_round = p_me->zoom - 1; 
    zoom_shift = p_me->zoom >> 1; 
    if ( zoom_shift > 0 )
    {
        left = (left+zoom_round) >> zoom_shift;
        top = (top+zoom_round) >> zoom_shift;
        width = (width+zoom_round) >> zoom_shift;
        height = (height+zoom_round) >> zoom_shift;
    }
    if ( width <= 0 || height <= 0 ) 
    {
        return 0;
    }

	switch( type )
	{
      case PGI_Fill_Disable:	/* Disable video display, leave image. */
		O_Draw_set_gc(p_me->p_context->p_draw, &gc_disable);
		O_Draw_fill_box(p_me->p_context->p_draw, left, top, width+1, height+1);
		break;

	  default:  
	  case PGI_Fill_Enable:  /* Enable exposed area of new video. */
		O_Draw_set_gc(p_me->p_context->p_draw, &p_me->gc_enable);
		O_Draw_fill_box(p_me->p_context->p_draw, left, top, width, height);
		break;

	  case PGI_Fill_Exposure:  /* Enable exposed area of existing video. */
		O_Draw_set_gc(p_me->p_context->p_draw, &p_me->gc_exposure);
		O_Draw_fill_box(p_me->p_context->p_draw, left, top, width, height);
		break;

      case PGI_Fill_Remove:	/* Remove video and image from buffer. */
		O_Draw_set_gc(p_me->p_context->p_draw, &gc_remove);
		O_Draw_fill_box(p_me->p_context->p_draw, left, top, width+1, height+1);
		break;
	}

    return 1;
}

/*----------------------------------------------------------------------------*
 |"pv700_get_compressed_image"
 |
 |  Compress data from the true color buffer. This routine is always
 |  called for W channel port, not V!! NOTE: we set the type if the W
 |  channel is being used for input. This is a kludge to get around 
 |  some setting during compression that messes up input.
 |
 |    = # of bytes in compressed image
 *----------------------------------------------------------------------------*/
static int pv700_get_compressed_image( p_me, n_buf, p_buf,
    src_x, src_y, src_w, src_h, dst_x, dst_y, dst_w, dst_h)

    HwPGIpv*    p_me;    /* -> control block for this xv instance. */
    int         n_buf;   /* =  # of bytes in 'p_buf'. */
    char*       p_buf;   /* -> where to put compressed data. */
    int         src_x;   /* =  starting x position in frame buffer to compress*/
    int         src_y;   /* =  starting y position in frame buffer to compress*/
    int         src_w;   /* =  width of data in frame buffer to compress. */
    int         src_h;   /* =  height of data in frame buffer to compress. */
    int         dst_x;   /* =  starting x position in image to save. */
    int         dst_y;   /* =  starting y position in image to save. */
    int         dst_w;   /* =  width of data in image to save. */
    int         dst_h;   /* =  height of data in image to save. */
{
    int     	n_data;  	/* =  # of bytes in compressed image. */
	HwPGIpv*	p_wchan;	/* Port attached to w channel. */
	int			set_type;	/* =  1 if w channel is on input. */
	int			zoom;		/* Amount to shift coordinates for zoom factor. */


    /* SUSPEND VIDEO ON CHANNEL W.  */  

	if ( p_wchan = p_me->p_channels->w_chan )
    {
		set_type = (p_wchan->video_frozen == 0 && 
			p_wchan->video_off_semaphore == 0 ) ? 1 : 0;
		Xvhw_Suspend(p_wchan, 1);
	}
		

    /*
     *  COMPRESS THE IMAGE FROM THE FRAME BUFFER.
     */
	zoom = p_me->zoom >> 1;
    n_data = C3GetImage(p_me->p_context,
        src_x>>zoom, src_y>>zoom, src_w>>zoom, src_h>>zoom,
		dst_x>>zoom, dst_y>>zoom, dst_w>>zoom, dst_h>>zoom,
        1, p_buf, n_buf, p_me->q_compression, p_me->huffman);



    /*  RESUME VIDEO ON CHANNEL W.  */  

	if ( p_wchan )
	{
        Xvhw_Resume(p_wchan, 0);
		if ( set_type ) Xvhw_Set_Signal_Type(p_wchan, &p_wchan->signal); 
	}

    return n_data;
}

/*----------------------------------------------------------------------------*
 |"pv700_get_drawable_area"
 |
 |  Return the current output area information.
 *----------------------------------------------------------------------------*/
static void pv700_get_drawable_area( p_me, p_area )
    HwPGIpv*		p_me;   /* -> control block for this xv instance. */
    Xvhw_Drw_Area*	p_area; /* -> where to put current output area. */
{
    Xvhw_Drw_Area*  p_drw;  /* Fast access to p_me->drawable_area. */
    int             zoom;   /* Zoom factor in use. */

    p_drw = &p_me->drawable_area;
    p_area->fields_inverted = p_drw->fields_inverted;
	p_area->direction = p_drw->direction;


    /*  RETURN NORMAL RECTANGLE TO CALLER. THIS MEANS WE MAY HAVE OF
     *  UNFLIP COORDINATES WE USE INTERNALLY.
     */  
    zoom = p_me->zoom >> 1; 
    if ( p_area->vertical_flip = p_drw->vertical_flip )
    {
        p_area->top = (p_drw->top + p_drw->height + 1) << zoom;
        p_area->height = (-p_drw->height) << zoom;
    }   
    else
    {   
        p_area->top = (p_drw->top) << zoom;
        p_area->height = (p_drw->height) << zoom;
    }   
    if ( p_area->horizontal_flip = p_drw->horizontal_flip )
    {
        p_area->left = (p_drw->left + p_drw->width + 1) << zoom;
        p_area->width = (-p_drw->width) << zoom;
    }
    else
    {
        p_area->left = p_drw->left << zoom;
        p_area->width = p_drw->width << zoom;   
    }
}
    
#ifdef _AIX

/*----------------------------------------------------------------------------*
 |"pv700_get_image"
 |
 |  Return the data from the true color buffer. Note that we must keep track
 |  of the portions of the requested area which are off screen. For those
 |  pixels we return a value of 0x00ff0000. There are 3 strips of data in 
 |  each direction. The ones for the x direction are:
 |
 |    (1) left of screen:  x_bgn_left  ---> x_end_left  = 0x00ff0000
 |    (2) on screen:       x_bgn_on    ---> x_end_on    = pixel value
 |    (3) right of screen: x_bgn_right ---> x_end_right = 0x00ff0000
 |
 |  The strips in the y direction are:
 |
 |    (1) above screen: y_bgn_above ---> y_end_above = 0x00ff0000
 |    (2) on screen:    y_bgn_on    ---> y_end_on    = value wrt x strips
 |    (3) below screen: y_bgn_below ---> y_end_below = 0x00ff0000
 |
 |    = size of image read
 *----------------------------------------------------------------------------*/
static int pv700_get_image( p_me, x, y, w, h, buf )
    HwPGIpv*		p_me;    /* -> control block for this xv instance. */
    int  			x;       /* X coordinate of data to write. */
    int  			y;       /* Y coordinate of data to write. */
    int             w;       /* =  width of data to be returned. */
    int             h;       /* =  height of data to be returned. */
    u_int*			buf;     /* -> buffer to place data into. */
{
    int  dma_received;  /* # of bytes to received in dma. */
    int  dma_wanted;    /* # of bytes to wanted from dma. */
    int  i_x;           /* Current x pixel to get. */
    int  i_y;           /* Current y line to get. */
	int	 stride;		/* Stride between lines in pv700 memory. */
    int  x_bgn_left;    /* Starting x of image off-screen to left. */
    int  x_bgn_right;   /* Starting x of image off-screen to right. */
    int  x_bgn_on;      /* Starting x of image on-screen. */
    int  x_end_on;      /* Ending x of image to get which is on-screen. */
    int  x_end_left;    /* Ending of pixels off screen to the left. */
    int  x_end_right;   /* Ending of pixels off screen to the right. */
    int  y_bgn_above;   /* Starting y of image off sceen to top. */
    int  y_bgn_below;   /* Starting y of image off sceen to bottom. */
    int  y_bgn_on;      /* Starting y of image on sceen. */
    int  y_end_above;   /* Ending y of image off screen to top. */
    int  y_end_below;   /* Ending y of image off screen to bottom. */
    int  y_end_on;      /* Ending y of image on the screen. */
    int  zoom;          /* Current zoom factor. */


    dma_wanted = 4 * w * h;
	stride = p_me->p_context->main.linebytes;
    zoom = p_me->zoom >> 1;
	if ( zoom )
	{
		x = x - p_me->drawable_area.left;
		y = y - p_me->drawable_area.top;
	}

    if ( debug_flags & 1 )
    {
        plxv_dma_setup(p_me->p_context, x, y, w, h);
        dma_received = read(p_me->fd_image, buf, dma_wanted);
        return dma_received;
    }

    /*  CALCULATE THREE STRIPS IN THE X AND Y DIRECTIONS (SEE NOTES ABOVE) */
     
    x_bgn_left = ( x < 0 ) ? x : 0;
    x_bgn_on = ( x < 0 ) ? 0 : x;
    x_end_right =  ( x+w > p_me->fb_pitch ) ? x+w : 0;
    x_end_on = ( x+w < p_me->fb_pitch ) ? x+w : p_me->fb_pitch;

    if ( y < 0 )
    {
        y_bgn_below = y;
        if ( y+h < 0 )
        {
            y_end_below = y+h;
            y_bgn_on = 0;
            y_end_on = 0;
            y_bgn_above = 0;
            y_end_above = 0;
        }
        else if ( y+h < p_me->fb_height )
        {
            y_end_below = 0;
            y_bgn_on = 0;
            y_end_on = y+h;
            y_bgn_above = 0;
            y_end_above = 0;
        }

        else /* y+h >= p_me->fb_height */
        {
            y_end_below = 0;
            y_bgn_on = 0;
            y_end_on = p_me->fb_height;
            y_bgn_above = p_me->fb_height;
            y_end_above = y+h;
        }
    }

    else if ( y < p_me->fb_height )
    {
        y_bgn_on = y;
        y_bgn_below = 0;
        y_end_below = 0;
        if ( y+h < p_me->fb_height )
        {
            y_end_on = y+h;
            y_bgn_above = 0;
            y_end_above = 0;
        }
        else
        {
            y_end_on = p_me->fb_height;
            y_bgn_above = p_me->fb_height;
            y_end_above = y+h;
        }
    }
    else /* y >= p_me->fb_height */
    {
        y_bgn_above = 0;
        y_end_above = 0;
        y_bgn_on = 0;
        y_end_on = 0;
        y_bgn_below = y;
        y_end_below = y+h;
    }

    /* GET THE PORTION OF THE IMAGE WHICH IS ABOVE THE SCREEN (IF ANY) */

    for (i_y = y_bgn_above; i_y < y_end_above; i_y++ )
    {
        for ( i_x = x; i_x < x+w; i_x++ )
        {
            *buf++ = 0x00ff0000;
        }
    }

    /* GET THE PORTION OF THE IMAGE WHICH IS ON SCREEN IN THE Y DIRECTION
     *
     *  We process three strips in the x direction, off screen to the left,
     *  on screen, and off screen to the right.
     */
    for (i_y = y_bgn_on; i_y < y_end_on ; i_y++)
    {
        for (i_x = x_bgn_left; i_x < 0; i_x++ )
        {
            *buf++ = 0x00ff0000;
        }
        for (i_x = x_bgn_on; i_x < x_end_on; i_x++)
        {
            *buf++ = p_me->p_context->m24base[i_x + (i_y * stride)];
        }
        for (i_x = p_me->fb_pitch; i_x < x_end_right; i_x++ )
        {
            *buf++ = 0x00ff0000;
        }
    }

    /* GET THE PORTION OF THE IMAGE WHICH IS BELOW THE SCREEN (IF ANY) */

    for (i_y = y_bgn_below; i_y < y_end_below; i_y++ )
    {
        for ( i_x = x; i_x < x+w; i_x++ )
        {
            *buf++ = 0x00ff0000;
        }
    }

    return dma_wanted;
}
    
#else /* !_AIX */
/*----------------------------------------------------------------------------*
 |"pv700_get_image"
 |
 |  Return the data from the true color buffer. Note that we must keep track
 |  of the portions of the requested area which are off screen. For those
 |  pixels we return a value of 0x00ff0000. There are 3 strips of data in 
 |  each direction. The ones for the x direction are:
 |
 |    (1) left of screen:  x_bgn_left  ---> x_end_left  = 0x00ff0000
 |    (2) on screen:       x_bgn_on    ---> x_end_on    = pixel value
 |    (3) right of screen: x_bgn_right ---> x_end_right = 0x00ff0000
 |
 |  The strips in the y direction are:
 |
 |    (1) above screen: y_bgn_above ---> y_end_above = 0x00ff0000
 |    (2) on screen:    y_bgn_on    ---> y_end_on    = value wrt x strips
 |    (3) below screen: y_bgn_below ---> y_end_below = 0x00ff0000
 |
 |    = size of image read
 *----------------------------------------------------------------------------*/
static int pv700_get_image( p_me, p_geom, buf )
    HwPGIpv*		p_me;    /* -> control block for this xv instance. */
	Xvhw_Image*		p_geom;	 /* -> image geometry information. */
    u_int*			buf;     /* -> buffer to place data into. */
{
    int  dma_received;  /* # of bytes to received in dma. */
    int  dma_wanted;    /* # of bytes to wanted from dma. */
    int  i_x;           /* Current x pixel to get. */
    int  i_y;           /* Current y line to get. */
	int	 stride;		/* Stride between lines in pv700 memory. */
    int  x;       		/* X coordinate of data to return adjusted by zoom. */
    int  y;       		/* Y coordinate of data to return adjusted by zoom. */
    int  x_bgn_left;    /* Starting x of image off-screen to left. */
    int  x_bgn_right;   /* Starting x of image off-screen to right. */
    int  x_bgn_on;      /* Starting x of image on-screen. */
    int  x_end_on;      /* Ending x of image to get which is on-screen. */
    int  x_end_left;    /* Ending of pixels off screen to the left. */
    int  x_end_right;   /* Ending of pixels off screen to the right. */
    int  y_bgn_above;   /* Starting y of image off sceen to top. */
    int  y_bgn_below;   /* Starting y of image off sceen to bottom. */
    int  y_bgn_on;      /* Starting y of image on sceen. */
    int  y_end_above;   /* Ending y of image off screen to top. */
    int  y_end_below;   /* Ending y of image off screen to bottom. */
    int  y_end_on;      /* Ending y of image on the screen. */
    int  zoom;          /* Current zoom factor. */


    dma_wanted = 4 * p_geom->image_w * p_geom->image_h;
	stride = p_me->p_context->main.linebytes;
    zoom = p_me->zoom >> 1;
	x = (p_geom->drawable_x >> zoom) + p_geom->offset_x;
	y = (p_geom->drawable_y >> zoom) + p_geom->offset_y;

    if ( debug_flags & 1 )
    {
        plxv_dma_setup(p_me->p_context, x, y, p_geom->image_w, p_geom->image_h);
        dma_received = read(p_me->fd_image, buf, dma_wanted);
        return dma_received;
    }

    /*  CALCULATE THREE STRIPS IN THE X AND Y DIRECTIONS (SEE NOTES ABOVE) */
     
    x_bgn_left = ( x < 0 ) ? x : 0;
    x_bgn_on = ( x < 0 ) ? 0 : x;
    x_end_right = (x+p_geom->image_w > p_me->fb_pitch) ? x+p_geom->image_w : 0;
    x_end_on = (x+p_geom->image_w < p_me->fb_pitch) ? 
		x+p_geom->image_w : p_me->fb_pitch;

    if ( y < 0 )
    {
        y_bgn_below = y;
        if ( y+p_geom->image_h < 0 )
        {
            y_end_below = y+p_geom->image_h;
            y_bgn_on = 0;
            y_end_on = 0;
            y_bgn_above = 0;
            y_end_above = 0;
        }
        else if ( y+p_geom->image_h < p_me->fb_height )
        {
            y_end_below = 0;
            y_bgn_on = 0;
            y_end_on = y+p_geom->image_h;
            y_bgn_above = 0;
            y_end_above = 0;
        }

        else /* y+p_geom->image_h >= p_me->fb_height */
        {
            y_end_below = 0;
            y_bgn_on = 0;
            y_end_on = p_me->fb_height;
            y_bgn_above = p_me->fb_height;
            y_end_above = y+p_geom->image_h;
        }
    }

    else if ( y < p_me->fb_height )
    {
        y_bgn_on = y;
        y_bgn_below = 0;
        y_end_below = 0;
        if ( y+p_geom->image_h < p_me->fb_height )
        {
            y_end_on = y+p_geom->image_h;
            y_bgn_above = 0;
            y_end_above = 0;
        }
        else
        {
            y_end_on = p_me->fb_height;
            y_bgn_above = p_me->fb_height;
            y_end_above = y+p_geom->image_h;
        }
    }
    else /* y >= p_me->fb_height */
    {
        y_bgn_above = 0;
        y_end_above = 0;
        y_bgn_on = 0;
        y_end_on = 0;
        y_bgn_below = y;
        y_end_below = y+p_geom->image_h;
    }

    /* GET THE PORTION OF THE IMAGE WHICH IS ABOVE THE SCREEN (IF ANY) */

    for (i_y = y_bgn_above; i_y < y_end_above; i_y++ )
    {
        for ( i_x = x; i_x < x+p_geom->image_w; i_x++ )
        {
            *buf++ = 0x00ff0000;
        }
    }

    /* GET THE PORTION OF THE IMAGE WHICH IS ON SCREEN IN THE Y DIRECTION
     *
     *  We process three strips in the x direction, off screen to the left,
     *  on screen, and off screen to the right.
     */
    for (i_y = y_bgn_on; i_y < y_end_on ; i_y++)
    {
        for (i_x = x_bgn_left; i_x < 0; i_x++ )
        {
            *buf++ = 0x00ff0000;
        }
        for (i_x = x_bgn_on; i_x < x_end_on; i_x++)
        {
            *buf++ = p_me->p_context->m24base[i_x + (i_y * stride)];
        }
        for (i_x = p_me->fb_pitch; i_x < x_end_right; i_x++ )
        {
            *buf++ = 0x00ff0000;
        }
    }

    /* GET THE PORTION OF THE IMAGE WHICH IS BELOW THE SCREEN (IF ANY) */

    for (i_y = y_bgn_below; i_y < y_end_below; i_y++ )
    {
        for ( i_x = x; i_x < x+p_geom->image_w; i_x++ )
        {
            *buf++ = 0x00ff0000;
        }
    }

    return dma_wanted;
}
#endif /* _AIX */
    
/*----------------------------------------------------------------------------*
 |"pv700_get_levels"
 |
 |  Return the picture levels (brightness, contrast, hue, saturation).
 *----------------------------------------------------------------------------*/
static void pv700_get_levels( p_me, levels )
    HwPGIpv*	 p_me;   /* -> control block for this xv instance. */
    Xvhw_Levels* levels; /* -> where to put current levels. */
{
    levels->brightness = p_me->levels.brightness;
    levels->contrast = p_me->levels.contrast;
    levels->hue = p_me->levels.hue;
    levels->saturation = p_me->levels.saturation;
}
    
/*----------------------------------------------------------------------------*
 |"pv700_get_signal_area"
 |
 |  Return the current output area information.
 |
 |  (1) We want to return a value which reflects the fact that 0,0 should 
 |      be the origin of the normal video signal for the timing regimen 
 |      being used. So we need to adjust the left and top values by the
 |      offsets specified in the tv private data area.
 |  (2) The area specified by source_start and source_end values includes
 |      those end points, so always add 1 to the width and 2 to the height
 |      before returning them.
 *----------------------------------------------------------------------------*/
static void pv700_get_signal_area( p_me, p_signal )
    HwPGIpv*		p_me;      /* -> control block for this xv instance. */
    Xvhw_Sig_Area*	p_signal;  /* -> where to put signal area info. */
{
    int zoom;   /* Zoom factor in use. */

    zoom = p_me->zoom >> 1;

    p_signal->left = p_me->signal_area.left << zoom;
    p_signal->top = p_me->signal_area.top << zoom;
    p_signal->width = p_me->signal_area.width << zoom;
    p_signal->height = p_me->signal_area.height << zoom;
}

/*----------------------------------------------------------------------------*
 |"pv700_get_signal_type"
 |
 |  Get the type and timing of signal being used. Update the information
 |  in the object's signal fields from the hardware, and then return that
 |  information to the caller. Note, the timing cannot be determined from
 |  reading the hardware, so use the information already in the record.
 *----------------------------------------------------------------------------*/
static void pv700_get_signal_type(p_me, p_signal)
    HwPGIpv*	 p_me;      /* -> pip object. */
    Xvhw_Signal* p_signal;  /* -> signal specification. */
{
    *p_signal = p_me->signal;
}
    
/*----------------------------------------------------------------------------*
 |"pv700_init"
 |
 |  Initialize pip hardware to an operable state. After this
 |  call the pip will be off. Enable it with a call to pv700_on.
 *----------------------------------------------------------------------------*/
static int pv700_init(p_me, signal)
    HwPGIpv*	 p_me;      /* -> pip object. */
    Xvhw_Signal* signal;    /* -> signal type information. */
{
    (*p_me->public.suspend)(p_me, 1);   /* Initialize to an off state. */
    (*p_me->public.set_signal_type)(p_me, signal);

    if ( p_me->p_context->main.linebytes != 1280 )
    {
        fprintf(stderr, "### Expected size of 1280 but got size %d\n", 
            p_me->p_context->main.linebytes);
    }


    /*  CLEAR OUT THE TAG BITS.  */

	O_Draw_set_gc(p_me->p_context->p_draw, &gc_remove);
	O_Draw_fill_box(p_me->p_context->p_draw, 0, 0,
        p_me->public.video_pitch, p_me->public.video_height);
    return 1;
}

/*----------------------------------------------------------------------------*
 |"pv700_off"
 |
 |  Turn off video digitization but not mixing. Set the video_frozen flag so 
 |  that operations on the  resume/suspend semaphore will not cause video to 
 |  come on. (This requires an explicit call to pv700_on.) Return its previous 
 |  operational state. Note we use a one shot to turn off the pip, to leave 
 |  a good image.
 |
 |      = 1 if pip was operating at time of call
 |      = 0 if pip was halted at time of call
 *----------------------------------------------------------------------------*/
static int pv700_off(p_me, wait_for_inactive)
    HwPGIpv*    p_me;               /* -> device instance dependent data. */
    int         wait_for_inactive;  /* =  1 if wait for pip to turn off */
{
	Chan_pv*	p_chans;		/* Fast access to p_me->p_channels. */
    int         prior_state;    /* Previous state of pip operations. */

    if ( p_me->video_frozen ) return 0;

    prior_state = (p_me->video_off_semaphore <= 0) ? 1 : 0;
	p_chans = p_me->p_channels;

	if      ( p_chans->o_chan == p_me ) 
	{
      	plxv_flash_w_off(p_me->p_context); 
        if(p_me->p_context->p_outputs[p_me->video_port])
			O_Output_off(p_me->p_context->p_outputs[p_me->video_port]);
		p_chans->o_chan = 0;
		if ( wait_for_inactive ) pv700_wait_for_change(p_me, WAIT_FOR_OFF);
	}
	else if ( p_chans->v_chan == p_me )
	{
      	plxv_flash_v_off(p_me->p_context); 
		p_chans->v_chan = 0;
		if ( wait_for_inactive ) pv700_wait_for_change(p_me, WAIT_FOR_OFF);
	}
	else if ( p_chans->w_chan == p_me )
	{
      	plxv_flash_w_off(p_me->p_context); 
		p_chans->w_chan = 0;
		if ( wait_for_inactive ) pv700_wait_for_change(p_me, WAIT_FOR_OFF);
	}
    p_me->video_frozen = 1;

    return prior_state;

}


/*----------------------------------------------------------------------------*
 |"pv700_on"
 |
 |  Turn on video digitization and mixing. Set the video_frozen flag to zero so
 |  that suspend/resumes will work. Initialize suspend/resume semaphore to its
 |  neutral state. Return the previous operational state.
 |  Note that we set the video cross over point in this code since other
 |  code will change it out from under us (e.g., pv700_put_compressed_image.)
 |
 |    = 1 if wait_for_active is zero, or pip is now running
 |    = 0 if pip did not turn on (hardware error)
 *----------------------------------------------------------------------------*/
static int pv700_on(p_me, wait_for_active)
    HwPGIpv*    p_me;            /* -> device instance dependent data. */
    int         wait_for_active; /* = 1 if should wait for pip to go active. */
{
	Chan_pv*		p_chans;	/* Fast access to p_me->p_chansnels. */
    Xvhw_Drw_Area*  p_dst; 		/* Output area specification. */
    Xvhw_Sig_Area*  p_src; 		/* Signal area specification. */

    p_dst = &p_me->drawable_area;
    p_src = &p_me->signal_area;
    if ( ( (int)p_dst->top >= (int)p_me->fb_height ) ||
         ( (int)p_src->width <= 0 ) ||
         ( (int)p_src->height <= 0 ) )
    {
        return 0;
    }
    p_me->video_frozen = 0;
    p_me->video_off_semaphore = 0;
	p_chans = p_me->p_channels;

	if ( p_chans->o_chan == p_me ) p_chans->o_chan = 0;
	if ( p_chans->v_chan == p_me ) p_chans->v_chan = 0;
	if ( p_chans->w_chan == p_me ) p_chans->w_chan = 0;

	switch ( p_me->drawable_area.direction )
	{

	  /*  STARTING UP OUTPUT FOR THIS PORT. */

	  case XvDrawableToSignal:
		p_chans->o_chan = p_me;
		p_chans->w_chan = 0;

		plxv_xpoint_w(p_me->p_context, OUTPUT_B, 1);
		plxv_video_w_wmask(p_me->p_context, 0);
	    O_Output_on(p_me->p_context->p_outputs[p_me->video_port]);
		if ( p_me->crop )
		{
			plxv_output_w( p_me->p_context,
				p_dst->left, p_dst->top, p_src->width, p_src->height,
				p_me->x_source_zero, p_me->y_source_zero,
				p_src->width, p_src->height, 0, 0
			);
		}
		else
		{   
			plxv_output_w( p_me->p_context,
				p_dst->left, p_dst->top, p_dst->width, p_dst->height,
				p_src->left+p_me->x_source_zero,
				p_src->top+p_me->y_source_zero,
				p_src->width, p_src->height, 0, 0
			);
		};
		break;


	  /*  STARTING UP CHANNEL V FOR THIS PORT. */

	  default:
		if ( (p_me->chan_affinity=='v') || p_chans->o_chan || !p_chans->v_chan )
		{
			p_chans->v_chan = p_me;
			plxv_video_v_stencil(p_me->p_context, PLX_CHANNEL_MASK, p_me->wid);
			plxv_video_v_wmask(p_me->p_context, PLXV_WMASK_24);
			plxv_xpoint_v(p_me->p_context, p_me->video_port, 1);
			if ( p_me->crop )
			{
				plxv_flash_v_squeeze( p_me->p_context,
					p_me->x_source_zero, p_me->y_source_zero,
					p_src->width, p_src->height,
					p_dst->left, p_dst->top, p_src->width, p_src->height, 0, 0
				);
			}
			else
			{
				plxv_flash_v_squeeze( p_me->p_context,
					p_src->left+p_me->x_source_zero,
					p_src->top+p_me->y_source_zero,
					p_src->width, p_src->height,
					p_dst->left, p_dst->top, p_dst->width, p_dst->height, 0, 0
				);
			}
		}


		/*  STARTING UP CHANNEL W FOR THIS PORT. */

		else
		{
			p_chans->o_chan = 0;
			p_chans->w_chan = p_me;
			plxv_video_w_stencil(p_me->p_context, PLX_CHANNEL_MASK, p_me->wid);
            plxv_video_w_wmask(p_me->p_context, PLXV_WMASK_24);
            plxv_xpoint_w(p_me->p_context, p_me->video_port, 1);
            if ( p_me->crop )
            {
                plxv_flash_w_squeeze( p_me->p_context,
                    p_me->x_source_zero, p_me->y_source_zero,
                    p_src->width, p_src->height,
                    p_dst->left, p_dst->top, p_src->width, p_src->height, 0, 0
                );
            }
            else
            {   
                plxv_flash_w_squeeze( p_me->p_context,
                    p_src->left+p_me->x_source_zero,
                    p_src->top+p_me->y_source_zero,
                    p_src->width, p_src->height,
                    p_dst->left, p_dst->top, p_dst->width, p_dst->height, 0, 0
                );
            }
		}
		break;
	}
	if ( wait_for_active ) pv700_wait_for_change(p_me, WAIT_FOR_ON);

    return 1;
}


/*----------------------------------------------------------------------------*
 |"pv700_one_shot"
 |
 |  Turn pip on for a frame and restart mixing. Upon return the pip will
 |  be off, with the "off" semaphore incremented appropriately.
 |    = 1 if success
 |    = 0 if error detected
 *----------------------------------------------------------------------------*/
static int pv700_one_shot(p_me, wait)
    HwPGIpv*    p_me;  /* -> device instance dependent data. */
    int         wait;  /* =  1 if should wait till video is off. */
{
	Chan_pv*	p_chans;	/* Fast access to p_me->p_channels. */

	p_chans = p_me->p_channels;

	if ( p_chans->v_chan == p_me ) 
	{
		plxv_flash_v_off(p_me->p_context); 
		if ( wait ) pv700_wait_for_change(p_me, WAIT_FOR_OFF);
	}
	else if ( p_chans->w_chan == p_me ) 
	{
		plxv_flash_w_off(p_me->p_context);
		if ( wait ) pv700_wait_for_change(p_me, WAIT_FOR_OFF);
	}
    p_me->video_off_semaphore++;

    return 1;
}


/*----------------------------------------------------------------------------*
 |"pv700_put_compressed_image"
 |
 |  Decompress data into the true color buffer. For the V and W channels the
 |  caller should stop video digitization (i.e., we don't suspend-resume 
 |  in this routine). 
 *----------------------------------------------------------------------------*/
static int pv700_put_compressed_image( p_me, n_buf, p_buf,
    src_x, src_y, src_w, src_h, dst_x, dst_y, dst_w, dst_h)

    HwPGIpv*	p_me;    /* -> control block for this xv instance. */
    int         n_buf;   /* =  # of bytes in 'p_buf'. */
    char*       p_buf;   /* -> data to be decompressed. */
    int         src_x;   /* =  starting x position in 'p_buf' to decompress. */
    int         src_y;   /* =  starting y position in 'p_buf' to decompress. */
    int         src_w;   /* =  width of data in 'p_buf' to decompress. */
    int         src_h;   /* =  height of data in 'p_buf' to decompress. */
    int         dst_x;   /* =  starting x position in frame buffer to write. */
    int         dst_y;   /* =  starting y position in frame buffer to write. */
    int         dst_w;   /* =  width of data in frame buffer to write. */
    int         dst_h;   /* =  height of data in frame buffer to write. */
{
    int     	channel;    /* Hardware library channel number. */
	Chan_pv*	p_chans;	/* Fast access to p_me->p_channels. */
    int     	size;       /* Size of decompressed image. */
	int			zoom;		/* Zoom shift to get to real coordinates. */

	p_chans = p_me->p_channels;

    /*  DETERMINE WHICH CHANNEL TO USE FOR THE DECOMPRESSION. */
     
	if ( (p_me->chan_affinity == 'v') || (p_chans->v_chan == p_me) ||
		  !p_chans->v_chan || p_chans->o_chan 
	)        
	{
        if ( p_chans->v_chan ) Xvhw_Suspend(p_chans->v_chan, 1);
		plxv_video_v_stencil(p_me->p_context, PLX_CHANNEL_MASK, p_me->wid);
        plxv_video_v_wmask(p_me->p_context, PLXV_WMASK_24);
        channel = 0;
	}
	else
	{
        if ( p_chans->w_chan ) Xvhw_Suspend(p_chans->w_chan, 1);
		plxv_video_w_stencil(p_me->p_context, PLX_CHANNEL_MASK, p_me->wid);
        plxv_video_w_wmask(p_me->p_context, PLXV_WMASK_24);
        channel = 1;
    }


    /*  DECOMPRESS THE IMAGE INTO THE FRAME BUFFER.  */

	zoom = p_me->zoom >> 1;
    size = C3PutImage(p_me->p_context,
        src_x, src_y, src_w, src_h, 
		dst_x>>zoom, dst_y>>zoom, dst_w>>zoom, dst_h>>zoom,
        0, p_buf, n_buf, p_me->q_decompression, p_me->huffman, channel);
 

    /*  RESUME VIDEO ON CHANNEL IF WE SUSPENDED IT */

    switch( channel )
    {
      case 0:	if ( p_chans->v_chan ) Xvhw_Resume( p_chans->v_chan, 0 );
      case 1:	if ( p_chans->w_chan ) Xvhw_Resume( p_chans->w_chan, 0 );
    }

    return 1;
}

#ifdef _AIX

/*----------------------------------------------------------------------------*
 |"pv700_put_image"
 |
 |  Place data into the true color buffer. This routine assumes caller is
 |  handling suspension and resumption of video display.
 *----------------------------------------------------------------------------*/
static int pv700_put_image( p_me, depth, x_win, y_win, x_off, y_off, w, h, buf )
    HwPGIpv*        p_me;    /* -> control block for this xv instance. */
    int             depth;   /* =  depth of image. */
    int             x_win;   /* =  x coordinate of window origin. */
    int             y_win;   /* =  y coordinate of window origin. */
    int             x_off;   /* =  x offset within window. */
    int             y_off;   /* =  y offset within window. */
    int             w;       /* =  width of data to be written. */
    int             h;       /* =  height of data to be written. */
    u_int*          buf;     /* -> buffer to get data from. */
{
    int 			dma_size;   /* # of bytes to transfer in dma. */
	O_Draw_Gc		gc;			/* Drawing context. */
	register u_int*	p_pix;		/* Next pixel to write. */
	int				x;			/* X adjusted for zoom. */
	int 			y;			/* Y adjusted for zoom. */
    int 			zoom;       /* Current zoom factor. */

    O_Draw_get_gc(p_me->p_context->p_draw, &gc);
    gc.wid_mask = 0;
    gc.wid_value = 0;
    switch( depth )
    {
      case 32:
        gc.stencil_mask = 0;
        gc.stencil_value = 0;
        gc.write_mask = PLXV_WMASK_24 | (PLX_CHANNEL_MASK << 24);
        break;
      default:
      case 24:
        gc.stencil_mask = PLX_CHANNEL_MASK;
        gc.stencil_value = p_me->wid;
        gc.stencil_mask = 0;
        gc.stencil_value = 0;
        gc.write_mask = PLXV_WMASK_24;
        break;
    }
    O_Draw_set_gc(p_me->p_context->p_draw, &gc);

    zoom = p_me->zoom >> 1;
	x = (x_win >> zoom) + x_off;
	y = (y_win >> zoom) + y_off;

    if ( debug_flags & 2 )
    {
        plxv_dma_setup(p_me->p_context, x, y, w, h);
        dma_size = 4 * w * h;
        write(p_me->fd_image, buf, dma_size);
    }
    else
    {
        int  i_x;       /* Current x pixel to set. */
        int  i_y;       /* Current y line to set. */
		int  stride;	/* Stride between image lines. */
        int  x_bgn;     /* Starting x of image to render. */
        int  x_end;     /* Ending x of image to render. */
        int  x_prefix;  /* Non-visible x pixels on left to render. */
        int  x_postfix; /* Non-visible x pixels on right to render. */
        int  y_bgn;     /* Starting y of image to render. */
        int  y_end;     /* Ending y of image to render. */
        int  y_prefix;  /* Non-visible y pixels on top to render. */

        y_end = y + h;
        if ( y_end < 0 || y >= p_me->fb_height ) return 1;

        x_end = x + w;
        if ( x_end < 0 || x >= p_me->fb_pitch ) return 1;

        if ( y >= 0 )
        {
            y_bgn = y;
            y_prefix = 0;
        }
        else
        {
            y_bgn = 0;
            y_prefix = -y;
        }

        if ( y_end > p_me->fb_height ) y_end = p_me->fb_height;

        if ( x >= 0 )
        {
            x_bgn = x;
            x_prefix = 0;
        }
        else
        {
            x_bgn = 0;
            x_prefix = -x;
        }

        x_postfix = 0;
        if ( x_end > p_me->fb_pitch )
        {
            x_postfix = x_end - p_me->fb_pitch;
            x_end = p_me->fb_pitch;
        }

		stride = p_me->public.video_pitch;
        p_pix = buf;
		p_pix += (y_prefix * w );
        for ( i_y = y_bgn; i_y < y_end; i_y++ )
        {
            p_pix += x_prefix;
            for ( i_x = x_bgn; i_x < x_end; i_x++ )
            {
                p_me->p_context->m24base[i_x+(stride*i_y)]= *p_pix++;
            }
            p_pix += x_postfix;
        }
    }
    return 1;
}
#else /* !AIX */

/*----------------------------------------------------------------------------*
 |"pv700_put_image"
 |
 |  Place data into the true color buffer. This routine assumes caller is
 |  handling suspension and resumption of video display. Note we treat depths
 |  of 24 bits and 32 bits differently. A depth of 24 bits will obey wid
 |  stencils, a depth of 32 bits is assumed to include a wid value in each
 |  of its pixels.
 |
 |    = 1 on success, 0 on failure
 *----------------------------------------------------------------------------*/
static int pv700_put_image( p_me, p_geom, buf )
    HwPGIpv*        p_me;    /* -> control block for this xv instance. */
	Xvhw_Image*		p_geom;	 /* -> information about image geometry. */
    u_int*          buf;     /* -> buffer to get data from. */
{
    int 			dma_size;   /* # of bytes to transfer in dma. */
	O_Draw_Gc		gc;			/* Drawing context. */
	register u_int*	p_pix;		/* Next pixel to write. */
	int				x;			/* X adjusted for zoom. */
	int 			y;			/* Y adjusted for zoom. */
    int 			zoom;       /* Current zoom factor. */

	O_Draw_get_gc(p_me->p_context->p_draw, &gc);
	gc.wid_mask = 0;
	gc.wid_value = 0;
	switch( p_geom->image_data_depth )
	{
	  case 32:
		gc.stencil_mask = 0;
		gc.stencil_value = 0;
		gc.write_mask = PLXV_WMASK_24 | ((unsigned) PLX_CHANNEL_MASK << 24);
		break;
	  default:
	  case 24:
		gc.stencil_mask = PLX_CHANNEL_MASK;
		gc.stencil_value = p_me->wid;
		gc.stencil_mask = 0;
		gc.stencil_value = 0;
		gc.write_mask = PLXV_WMASK_24;
		break;
	}
	O_Draw_set_gc(p_me->p_context->p_draw, &gc);

    zoom = p_me->zoom >> 1;
	x = (p_geom->drawable_x >> zoom) + p_geom->offset_x;
	y = (p_geom->drawable_y >> zoom) + p_geom->offset_y;

    if ( debug_flags & 2 )
    {
        plxv_dma_setup(p_me->p_context, x, y, p_geom->image_w, p_geom->image_h);
        dma_size = 4 * p_geom->image_w * p_geom->image_h;
        write(p_me->fd_image, buf, dma_size);
    }
    else
    {
        int  i_x;       /* Current x pixel to set. */
        int  i_y;       /* Current y line to set. */
		int  stride;	/* Stride between image lines. */
        int  x_bgn;     /* Starting x of image to render. */
        int  x_end;     /* Ending x of image to render. */
        int  x_prefix;  /* Non-visible x pixels on left to render. */
        int  x_postfix; /* Non-visible x pixels on right to render. */
        int  y_bgn;     /* Starting y of image to render. */
        int  y_end;     /* Ending y of image to render. */
        int  y_prefix;  /* Non-visible y pixels on top to render. */

        y_end = y + p_geom->image_h;
        if ( y_end < 0 || y >= p_me->fb_height ) return 1;

        x_end = x + p_geom->image_w;
        if ( x_end < 0 || x >= p_me->fb_pitch ) return 1;

        if ( y >= 0 )
        {
            y_bgn = y;
            y_prefix = 0;
        }
        else
        {
            y_bgn = 0;
            y_prefix = -y;
        }

        if ( y_end > p_me->fb_height ) y_end = p_me->fb_height;

        if ( x >= 0 )
        {
            x_bgn = x;
            x_prefix = 0;
        }
        else
        {
            x_bgn = 0;
            x_prefix = -x;
        }

        x_postfix = 0;
        if ( x_end > p_me->fb_pitch )
        {
            x_postfix = x_end - p_me->fb_pitch;
            x_end = p_me->fb_pitch;
        }

		stride = p_me->public.video_pitch;
        p_pix = buf;
		p_pix += (y_prefix * p_geom->image_w );
        for ( i_y = y_bgn; i_y < y_end; i_y++ )
        {
            p_pix += x_prefix;
            for ( i_x = x_bgn; i_x < x_end; i_x++ )
            {
                p_me->p_context->m24base[i_x+(stride*i_y)]= *p_pix++;
            }
            p_pix += x_postfix;
        }
    }
    return 1;
}

#endif /* AIX */

/*---------------------------------------------------------------------------*
 |"pv700_resume"
 |  Pop the pip "off" semaphore, and if it goes to zero and video_frozen
 |  is not set, start up the pip and start video mixing again. Sending 
 |  a pop_stack parameter of 1 will start video and clear the semaphore.
 |
 |      = 1 if pip now operating
 |      = 0 if pip still halted
 *---------------------------------------------------------------------------*/
static int pv700_resume(p_me, pop_stack)
    HwPGIpv*	p_me;       /* -> control block for this xv instance. */
    int         pop_stack;  /* =  1 if should pop semaphore stack to active. */
{
    if ( p_me->video_frozen ) return 0;

    p_me->video_off_semaphore = (pop_stack) ? 0 : --p_me->video_off_semaphore;
    if ( p_me->video_off_semaphore <= 0 )
    {
        p_me->video_off_semaphore = 0;
        (*p_me->public.on)(p_me, 1);
    }
    return (p_me->video_off_semaphore) ? 0 : 1;
}

/*----------------------------------------------------------------------------*
 |"pv700_set_drawable_area"
 |
 |  Set the size of the rectangle for the pip output area. 
 *----------------------------------------------------------------------------*/
static void pv700_set_drawable_area(p_me, p_area)
    HwPGIpv*		p_me;      /* -> control block for this xv instance. */
    Xvhw_Drw_Area*  p_area;    /* -> output area specification. */
{
    int     dst_h;          /* Height in frame buffer for video. */
    int     dst_w;          /* Width in frame buffer for video. */
    int     dst_x;          /* Starting x in frame buffer for video. */
    int     dst_y;          /* Starting y in frame buffer for video. */
    int     old_direction;  /* Old direction of video. */
    int     zoom;           /* Zoom factor in use. */

    /*
     *  CALCULATE PARAMETERS NECESSARY TO CALL plxv_flash_v_squeeze
     *  OR plxv_flash_w_squeeze AND PUT THOSE VALUES IN OUR OBJECT.
     */  

    zoom = p_me->zoom >> 1;
    dst_x = p_area->left >> zoom;
    dst_y = p_area->top >> zoom;
    dst_w = p_area->width >> zoom;
    dst_h = p_area->height >> zoom;
    if ( p_area->vertical_flip )
    {
        dst_y = dst_y + dst_h - 1;
        dst_h = - dst_h;
    }   
    if ( p_area->horizontal_flip )
    {
        dst_x = dst_x + dst_w - 1;
        dst_w = -dst_w;
    }
 
    old_direction = p_me->drawable_area.direction;
    p_me->drawable_area.direction = p_area->direction;
    p_me->drawable_area.fields_inverted = p_area->fields_inverted;
    p_me->drawable_area.height = dst_h;
    p_me->drawable_area.horizontal_flip = p_area->horizontal_flip;
    p_me->drawable_area.left = dst_x;
    p_me->drawable_area.top = dst_y;
    p_me->drawable_area.vertical_flip = p_area->vertical_flip;
    p_me->drawable_area.width = dst_w;

    if ( p_me->drawable_area.direction != old_direction )
    {
        if(p_me->signal.type < XV_COMPOSITE); /* avoid errors if not set */
			p_me->signal.type = XV_COMPOSITE;
        if(p_me->signal.timing < XV_NTSC);
			p_me->signal.timing = XV_NTSC;
        pv700_set_signal_type(p_me, &p_me->signal);
    }
}

/*----------------------------------------------------------------------------*
 |"pv700_set_levels"
 |
 |  Set the picture levels (brightness, contrast, hue, saturation).
 *----------------------------------------------------------------------------*/
static void pv700_set_levels( p_me, levels )
    HwPGIpv*	 p_me;   /* -> control block for this xv instance. */
    Xvhw_Levels* levels; /* -> where to get new levels. */
{
	O_Input*	p_input;	/* Input object for this port. */
    int         update; 	/* Set to 1 if need to update levels. */

    update = 0;
    if ( p_me->levels.brightness != levels->brightness )
    {
        p_me->levels.brightness = levels->brightness;
        update = 1;
    }
    if ( p_me->levels.contrast != levels->contrast )
    {
        p_me->levels.contrast = levels->contrast;
        update = 1;
    }
    if ( p_me->levels.hue != levels->hue )
    {
        p_me->levels.hue = levels->hue;
        update = 1;
    }
    if ( p_me->levels.saturation != levels->saturation )
    {
        p_me->levels.saturation = levels->saturation;
        update = 1;
    }
    
    /*
     *  WE NEED TO UPDATE PICTURE LEVELS. MAP THE VALUES FROM THE 
     *  -1000 TO +1000 RANGE INTO THE 0 TO 255 RANGE USED BY THE HARDWARE.
     */
    if ( update && (p_input = p_me->p_context->p_inputs[p_me->video_port]) )
    {
		O_Input_set_levels(p_me->p_context->p_inputs[p_me->video_port],
			(255*(p_me->levels.brightness+1000))/2000,
			(255*(p_me->levels.contrast+1000))/2000,
			(255*(p_me->levels.hue+1000))/2000,
			(255*(p_me->levels.saturation+1000))/2000
		);
	}
}
    
/*----------------------------------------------------------------------------*
 |"pv700_set_signal_area"
 |
 |  Set the size of the rectangle for the signal input area.
 |
 *----------------------------------------------------------------------------*/
static void pv700_set_signal_area(p_me, p_signal)
    HwPGIpv*		p_me;      /* -> control block for this xv instance. */
    Xvhw_Sig_Area*	p_signal;  /* -> new signal input area. */
{
    int zoom;   /* Zoom factor in use. */

    zoom = p_me->zoom >> 1;

    p_me->signal_area.left = p_signal->left >> zoom;
    p_me->signal_area.top = p_signal->top >> zoom;
    p_me->signal_area.width = p_signal->width >> zoom;
    p_me->signal_area.height = p_signal->height >> zoom;
}

/*----------------------------------------------------------------------------*
 |"pv700_set_signal_type"
 |
 |  Set the type and timing of signal being used. Only reprogram if there is 
 |  a change. A value of -1 in any of the fields of "signal" indicates the 
 |  current setting should be used. Note we clear the video memory so the 
 |  user can tell if there is an active video signal or not.
 *----------------------------------------------------------------------------*/
static void pv700_set_signal_type(p_me, signal)
    HwPGIpv*	 p_me;      /* -> pip object. */
    Xvhw_Signal* signal;    /* -> signal specification. */
{
	int		instance;				/* Input instance to use. */
	char*	p_timing;				/* Timing value to use. */
	char*	p_type;					/* Type of encoding to use. */

    p_me->signal = *signal;

    /*  CALCULATE PARAMETERS TO CALL INPUT OR OUTPUT OBJECT WITH: */

	instance = signal->type & XV_PLX_INPUT;
    switch( signal->type & XV_SOURCE_TYPE )
    {
      default:
        ERRORF((fd,"set_signal: bad type %08x, defaulting to COMPOSITE(%08x)\n",
            signal->type, XV_COMPOSITE));
        p_me->signal.type = XV_COMPOSITE;
      case XV_COMPOSITE: p_type = "COMPOSITE"; break;
      case XV_RGB:       p_type = "RGB";       break;
      case XV_YC:        p_type = "YC";        break;
      case XV_YUV:       p_type = "YUV";       break;
    }

    switch( signal->timing & XV_SOURCE_TIMING )
    {
      default:
        ERRORF((fd,
            "set_signal: bad signal timing %08x, defaulting to NTSC(%08x)\n", 
            signal->timing, XV_NTSC));
        p_me->signal.timing = XV_NTSC;
      case XV_NTSC:  
        p_me->x_source_zero = NTSC_X_ZERO;
        p_me->y_source_zero = NTSC_Y_ZERO;
		p_timing = "NTSC";
        break;

      case XV_PAL:   
        p_me->x_source_zero = PAL_X_ZERO;
        p_me->y_source_zero = PAL_Y_ZERO;
		p_timing = "PAL";
        break;

      case XV_SECAM: 
        p_me->x_source_zero = PAL_X_ZERO;
        p_me->y_source_zero = PAL_Y_ZERO;
		p_timing = "SECAM";
        break;
	}

	/*
	 *  DETERMINE IF OUR PORT IS AN INPUT OR OUTPUT CASE AND
	 *  SET THE NEW INFORMATION.
	 */
    switch( p_me->drawable_area.direction )
    {
      case XvSignalToDrawable:
		if ( signal->type & XV_PLX_DMSD )
		{
			if ( p_me->p_context->p_inputs[PORT_B] )
			{
				p_me->video_port = p_me->video_port1;
				O_Input_on( p_me->p_context->p_inputs[PORT_B]);
			}
		}
		else
		{
			p_me->video_port = p_me->video_port0;
		}
		if ( p_me->p_context->p_inputs[p_me->video_port] )
		{
			O_Input_set_type(p_me->p_context->p_inputs[p_me->video_port], 
				p_timing, p_type, instance);
		}
        break;

      case XvDrawableToSignal:
		if ( p_me->p_context->p_outputs[PORT_A] )
		{
			p_me->video_port = PORT_A;
			O_Output_set_type(p_me->p_context->p_outputs[p_me->video_port], 
				p_timing, p_type, instance);
		}
        break;
    }
}

/*----------------------------------------------------------------------------*
 |"pv700_set_compression_table"
 |
 |  Set one of the tables used for compression.
 *----------------------------------------------------------------------------*/
static void pv700_set_compression_table( p_me, n_data, data )
    HwPGIpv*	p_me;   /* -> control block for this xv instance. */
    int         n_data; /* =  length of 'data' in bytes. */
    short*      data;   /* -> data for table. */
{
    int     tab_flag;   /* Flags returned from table creation routine. */
    u_short *table;     /* Table creating. */

    tab_flag = plxv_create_c3table(p_me->p_context, 1, data, n_data, &table);
        
    if ( tab_flag & 0x01 )
    {
        if ( p_me->q_compression ) free(p_me->q_compression);
        p_me->q_compression = table;
    }
    else if ( tab_flag & 0x02 )
    {
        if ( p_me->huffman ) free(p_me->huffman);
        p_me->huffman = table;
    }
}

/*----------------------------------------------------------------------------*
 |"pv700_set_decompression_table"
 |
 |  Set one of the tables used for decompression.
 *----------------------------------------------------------------------------*/
static void pv700_set_decompression_table( p_me, n_data, data )
    HwPGIpv*	p_me;   /* -> control block for this xv instance. */
    int         n_data; /* =  length of 'data' in bytes. */
    short*      data;   /* -> data for table. */
{
    int     tab_flag;   /* Flags returned from table creation routine. */
    u_short *table;     /* Table creating. */

    tab_flag = plxv_create_c3table(p_me->p_context, 0, data, n_data, &table);
        
    if ( tab_flag & 0x01 )
    {
        if ( p_me->q_decompression ) free(p_me->q_decompression);
        p_me->q_decompression = table;
    }
    else if ( tab_flag & 0x02 )
    {
        if ( p_me->huffman ) free(p_me->huffman);
        p_me->huffman = table;
    }
}

/*----------------------------------------------------------------------------*
 |"pv700_status"
 |
 |  Return current status of pip operation.
 |
 |    = status
 *----------------------------------------------------------------------------*/
static int pv700_status(p_me)
    HwPGIpv*  p_me;      /* -> pip object. */
{    
   int	status;		/* Status value to return. */

   switch ( p_me->drawable_area.direction )
	{
	  case XvDrawableToSignal:
		status = XVS_Sync;
		break;
	  default:
		status =  ( p_me->p_context->p_inputs[p_me->video_port] ) ?
			O_Input_status(p_me->p_context->p_inputs[p_me->video_port]) : 0; 
		if ( status & 0x40 )
		{
			status = 0;
		}
		else
		{   
			switch( p_me->signal.timing )
			{
			  case XV_NTSC:
				status = (status & (1<<5)) ? XVS_Sync: 0;
				break;
			  default:
				status = (status & (1<<5)) ? 0: XVS_Sync;
				break;
			}
		}    
		break;
	}
	status |= (p_me->ccube ? XVS_Plx_Ccube : 0);
	status |= (XVS_Input|XVS_Display);
	status |= ( p_me->p_context->p_outputs[PORT_A] ? XVS_Output : 0 );

	return status;
}
    
/*----------------------------------------------------------------------------*
 |"pv700_suspend"
 |
 |  Push the pip "off" semaphore by one. If it is running stop it and
 |  optionally wait for the pip to actually stop running.
 |  Note we do a one shot to turn off the pip so we get a good image. 
 |
 |      = 1 if pip was operating at time of call
 |      = 0 if pip was halted at time of call
 *----------------------------------------------------------------------------*/
static int pv700_suspend(p_me, wait)
    HwPGIpv*   p_me;      /* -> control block for this xv instance. */
    int        wait;      /* =  1 if should wait for pip to stop. */
{
    int             prior_state;    /* State of pip prior to this call. */

    if ( p_me->video_frozen ) 
    {
        prior_state = 0;
    }
    else
    {
        prior_state = (p_me->video_off_semaphore <= 0) ? 1 : 0;
        if ( prior_state )
        {
            (*p_me->public.one_shot)(p_me, wait);
            p_me->video_off_semaphore = 1;
        }
        else
        {
            p_me->video_off_semaphore++;
        }
    }

    return prior_state;
}

/*----------------------------------------------------------------------------*
 |"pv700_terminate" 
 | 
 |  Terminate operations of Pip object: turn off video mixing
 |  and free dynamically allocated storage. If this is port 0 un-map the
 |  card by closing the device's file descriptor.
 *----------------------------------------------------------------------------*/
static void pv700_terminate(p_me) 
    HwPGIpv*	p_me;  /* -> object to terminate. */ 
{    
	DEBUGF(0x80, (fd, "%s: pv700_terminate port %d\n", 
		p_me->p_context->p_mmap->name, p_me->public.adaptor_port_number));

    (*p_me->public.suspend)(p_me, 0); 
    if ( p_me->public.adaptor_port_number == 0 )
	{
		O_PV700_terminate(p_me->p_context);
		free((char*)p_me->p_channels);
	}
    free((char*)p_me);
}    
 
/*----------------------------------------------------------------------------*
 |"pv700_wait_for_change"
 |
 |  Wait for pip to change its status from running to non-running or
 |  vice-versa. If we wait more than 100 milliseconds return an error.
 |  Note we wait one millisecond, but actually we wait 4 milliseconds due to 
 |  scheduler granularity.
 |    = 1 if status changed
 |    = 0 if status failed to change (hardware error)
 *----------------------------------------------------------------------------*/
pv700_wait_for_change(p_me, expected)
    HwPGIpv*   p_me;      /* -> context for object. */
    int        expected;  /* =  expected change, see WAIT_xxx defs above. */
{
    int             sleep_time;     /* How long to wait for one frame. */
    struct timeval  to;             /* Timer value for wait. */

    switch( p_me->signal.timing )
    {
	  default:
      case XV_NTSC:
        sleep_time = NTSC_SLEEP;
        break;
      case XV_PAL:  
      case XV_SECAM:    
        sleep_time = PAL_SLEEP;
        break;
    }
    
    to.tv_sec = 0;      
    to.tv_usec = sleep_time;    
    select(0,0,0,0,&to); 
}
    
/*----------------------------------------------------------------------------*
 |"xv_pv_map"
 |
 |  Given a device name map in a tv card.
 |
 |      -> device mapping info on success
 |      =  0 if failure
 *----------------------------------------------------------------------------*/
Plx_Map_Rec* xv_pv_map(device)
    char *device;   /* -> base name of devices to open. */
{
    int             image_fd;  /* Mapping of memory areas of card. */
    Plx_Map_Rec*    p_map;     /* Map region to be  passed to xv_pv700_open*/
	char			rsrc[100]; /* Name of resource file. */

	DEBUGF(0x80, (fd, "xv_pv_map(%s) entry\n", device));

    if ( !(p_map = (Plx_Map_Rec*)malloc(sizeof(Plx_Map_Rec))) )
    {
        ERRORF((fd, "Unable to allocate storage device %s\n", device)); 
        return 0;
    }
	sprintf(rsrc, "%s/%s", XV_ETC_PREFIX, &device[strlen("/dev/")]);
	DEBUGF(0x80, (fd, "xv_pv_map(%s) calling O_HWL_O_HWL(full, %s, %s)\n", 
		device, device, rsrc));
    p_map->p_context = O_HWL_O_HWL("full", device, rsrc);
    if ( p_map->p_context == 0 ) 
    {
        free(p_map);
        ERRORF((fd, "Unable to open device %s\n", device)); 
        return 0;
    }
    p_map->fd_ccube = p_map->p_context->fd_ccube; 

#if 1
    plxv_ccube_init(p_map->p_context);	/* do one compress at startup */
#endif

	DEBUGF(0x80, (fd, "xv_pv_map(%s) exit\n", device));
    return p_map;
}

static Tv_Private xv_pv700_private_methods = 
{
    pv700_get_compressed_image,
    pv700_put_compressed_image,
    pv700_set_compression_table,
    pv700_set_decompression_table
};

/*----------------------------------------------------------------------------*
 |"xv_pv_open"
 |
 |  Given a file descriptor, open a pip connected to a tv card as a X video 
 |  extension device. 
 |
 |      -> X video extension control block
 |      =  0 if failure
 *----------------------------------------------------------------------------*/
HwPGIpv* xv_pv_open( i_port, p_map )
    int             i_port; /* =  port being opened. */
    Plx_Map_Rec*    p_map;  /* -> mmap object or zero. */
{
	static Chan_pv*    p_channels;	 /* Global channel storage. */
    int                pgi;          /* Index: next plane group to look at. */
    int                pip_size;     /* Size of memory map or zero. */
    int                pip_offset;   /* Offset for memory mapping. */
    HwPGIpv*    	   p_me;         /* X video extension instance cb. */
    Xvhw_Signal        signal;       /* Video signal type and timing. */

	DEBUGF(0x80, (fd, "xv_pv_open(%d,%x) entry\n", i_port, p_map));

    /*
     *  Allocate pip object and fill in its transfer vector.
     */
    if ( !(p_me = (HwPGIpv*)malloc(sizeof(HwPGIpv))) )
    {
        return 0;
    }
	DEBUGF(0x80, (fd, "xv_pv_open(%d,%x) after object allocation\n", 
		i_port, p_map));
    p_me->public.attr.debug = 0;
    p_me->public.attr.stop_for_update = 0;
    p_me->public.attr.separate_fb = 1;
    p_me->public.adaptor_port_number = i_port; 

    p_me->public.control = pv700_control;
    p_me->public.dump = pv700_dump;
    p_me->public.fill_true_color = pv700_fill_true_color;
    p_me->public.fill_video_enable = pv700_fill_video_enable;
    p_me->public.get_levels = pv700_get_levels;
    p_me->public.get_drawable_area = pv700_get_drawable_area;
    p_me->public.get_image = pv700_get_image;
    p_me->public.get_signal_area = pv700_get_signal_area;
    p_me->public.get_signal_type = pv700_get_signal_type;
    p_me->public.initialize = pv700_init;
    p_me->public.off = pv700_off;
    p_me->public.on = pv700_on;
    p_me->public.one_shot = pv700_one_shot;
    p_me->public.put_image = pv700_put_image;
    p_me->public.resume = pv700_resume;
    p_me->public.set_levels = pv700_set_levels;
    p_me->public.set_drawable_area = pv700_set_drawable_area;
    p_me->public.set_signal_area = pv700_set_signal_area;
    p_me->public.set_signal_type = pv700_set_signal_type;
    p_me->public.status = pv700_status;
    p_me->public.suspend = pv700_suspend;
    p_me->public.terminate = pv700_terminate;

    p_me->public.video_pitch =  p_map->p_context->main.linebytes;
    p_me->public.video_height = p_map->p_context->main.ysize;

    p_me->atv = &xv_pv700_private_methods;
    p_me->ccube = O_PV700_ccube_present(p_map->p_context);
	p_me->chan_affinity = 0;
    p_me->crop = 0;
    p_me->drawable_area.fields_inverted = 0;
    p_me->drawable_area.horizontal_flip = 0;
    p_me->drawable_area.vertical_flip = 0;
    p_me->fb_pitch =  p_map->p_context->main.xsize;
    p_me->fb_height = p_map->p_context->main.ysize;
    p_me->fd_ccube = p_map->fd_ccube;
    p_me->fd_image = p_map->fd_image;
    p_me->huffman = 0;
    p_me->p_context = p_map->p_context;
    p_me->q_compression = 0;
    p_me->q_decompression = 0;
    p_me->video_frozen = 0;
#if 0
    p_me->video_off_semaphore = 0;
#else
    p_me->video_off_semaphore = 1;	/* not started yet damnit! */
#endif
    p_me->video_port0 = p_map->p_context->p_inputs[PORT_D] ? PORT_D : PORT_B;
    p_me->video_port1 = p_map->p_context->p_inputs[PORT_B] ? PORT_B : PORT_D;
	p_me->video_port = p_me->video_port0;
    p_me->x_source_zero = NTSC_X_ZERO;
    p_me->y_source_zero = NTSC_Y_ZERO;
	p_me->zoom = 1;


    /*  PERFORM ANY PORT-SPECIFIC PROCESSING (E.G., IF THIS IS THE FIRST 
     *  PORT FOR AN ADAPTOR INITIALIZE THE HARDWARE LIBRARY AND THE TAG BITS.)
     */
    switch( i_port )
    {
      case 0:
		DEBUGF(0x80, (fd, "xv_pv_open processing port 0\n"));
		if ( !(p_channels = (Chan_pv*)malloc(sizeof(Chan_pv))) )
		{
			ERRORF((fd, "Unable to allocate Chan_pv structure\n"));
			return 0;
		}
		p_me->p_channels = p_channels;
        p_me->drawable_area.direction = XvSignalToDrawable;
		p_channels->next_port = 0;
		p_channels->o_chan = 0;
		p_channels->v_chan = 0;
		p_channels->w_chan = 0;

        plxv_video_v_wmask(p_me->p_context, PLXV_WMASK_24);
        plxv_video_w_wmask(p_me->p_context, PLXV_WMASK_24);
        plxv_video_v_stencil(p_me->p_context, 0, 0);
        plxv_video_w_stencil(p_me->p_context, 0, 0);

		DEBUGF(0x80, (fd, "xv_pv_open before call to pv700_init\n"));
		signal.timing = XV_NTSC;
		signal.type = XV_COMPOSITE;
		signal.gray_scale = 0;
		signal.vtr_mode = 0;
		pv700_init(p_me, &signal);
        break;

      default:
		DEBUGF(0x80, (fd, "xv_pv_open processing port %d\n", i_port));
		p_me->p_channels = p_channels;
        p_me->drawable_area.direction = XvSignalToDrawable;
        break;
    }
	DEBUGF(0x80, (fd, "xv_pv_open done processing port %d\n", i_port));
	p_me->wid = i_port << 1;
	p_me->gc_enable.color = 0;
	p_me->gc_enable.color1_bg = 0;
	p_me->gc_enable.color1_fg = 0;
	p_me->gc_enable.f_transparency = 0;
	p_me->gc_enable.readback = 0;
	p_me->gc_enable.rop = PLXV_ROP_COPY;
    p_me->gc_enable.stencil_mask = 0;
    p_me->gc_enable.stencil_value = 0;
    p_me->gc_enable.wid_mask = PLX_CHANNEL_MASK | PLX_VISIBLE;
    p_me->gc_enable.wid_value = p_me->wid | PLX_VISIBLE;
    p_me->gc_enable.write_mask = PLXV_WMASK_ALL;
	p_me->gc_exposure.color = 0;
	p_me->gc_exposure.f_transparency = 0;
	p_me->gc_exposure.rop = PLXV_ROP_COPY;
    p_me->gc_exposure.stencil_mask = 0;
    p_me->gc_exposure.stencil_value = 0;
    p_me->gc_exposure.wid_mask = PLX_CHANNEL_MASK | PLX_VISIBLE;
    p_me->gc_exposure.wid_value = p_me->wid | PLX_VISIBLE;
    p_me->gc_exposure.write_mask = PLXV_WMASK_TAG_ALL;

	DEBUGF(0x80, (fd, "xv_pv_open(%d,%x) returning %x\n", i_port, p_map, p_me));
    return p_me;
}
