/*
 *   Copyright (c) 1994 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 */
/*
** File:
**
**	xvhw_PGI700.h: Hardware layer instance control block for XV video 
**                 extension support of Parallax TV2 board.
**
** Author:
**
**   Andy Goodrich (Parallax)
**
** Revisions:
**
**   25.01.94 Goodrich
**     - Initial coding.
*
* $Log: xvhw_PGIpv.h,v $
 * Revision 1.6  1996/04/03  22:28:46  dist
 * needed ifdef for sun
 *
 * Revision 1.4  95/12/11  14:58:03  14:58:03  dist ()
 * Andy Goodrich: Checkin / merging of hedge changes.
 * 
 * Revision 1.3  95/08/16  09:21:33  09:21:33  dist ()
 * Andy Goodrich: Added next_port field to Chan_pv to implement 
 *                XV_PLX_NEXT_PORT
 * 
 * Revision 1.2  95/05/25  17:03:10  17:03:10  dist ()
 * Andy Goodrich - Combined HP/AIX version for HP 9.1 code freeze.
 * 
 * Revision 1.1  95/05/12  13:45:32  13:45:32  dist ()
 * Initial revision
 * 
 * Revision 1.1  1995/04/17  19:49:39  dist
 * Initial revision
 *
*/
#ifndef xvhw_PGItv2_INCLUDED
#define xvhw_PGItv2_INCLUDED 

#include "xvhw.h"			/* Standard interface to Pip object. */
#include "hw_context.h"    	/* Parallax hardware library definitions. */
#include "plxv_xpoint.h"	/* Pick up PORT_x definitions. */

#define PLX_TV_WIDTH  1280		/* Width of video frame buffer. */
#define PLX_TV_HEIGHT 1024		/* Height of video frame buffer. */

typedef struct _Tv_Private {		 /* Methods specific to this hardware: */
	int	  (*get_compressed_image)(); /* Get a compressed image. */
	int	  (*put_compressed_image)(); /* Put a compressed image. */
	void  (*set_compress_table)();	 /* Set a compression table. */
	void  (*set_decompress_table)(); /* Set a decompression table. */
} Tv_Private;

typedef struct _Plx_Map_Rec {
	int			fd_ccube;	  	/* File descriptor for ccube dma. */
	int			fd_image;	  	/* File descriptor for image dma. */
	XV_struct*	p_context;		/* Hardware library context. */
} Plx_Map_Rec;


typedef struct _Chan_pv {
	int					next_port;	/* Next port to return XV_PLX_NEXT_PORT. */
	struct 	_HwPGIpv*	o_chan;		/* Current output channel port. */
	struct 	_HwPGIpv*	v_chan;		/* Current V channel port. */
	struct 	_HwPGIpv*	w_chan;		/* Current W channel port. */
} Chan_pv;

/* =============================================================================
 * ||                                                                         ||
 * || X Video Extension Object For Communicating With Video Hardware Support. ||
 * ||                                                                         ||
 * ||   Notes:                                                                ||
 * ||     (1) The p_draw field is used to determine if a port is being        ||
 * ||         assigned a new drawable. If so we do a complete erasure when    ||
 * ||         preparing its surface.                                          ||
 * =============================================================================
 */
typedef struct _HwPGIpv {
	XvhwRec   	  	 public;		   /* Public fields: THIS MUST BE FIRST! */
	Tv_Private	  	 *atv;			   /* Private methods. */
	int				 ccube;            /* = 1 if ccube chip present. */
	int				 chan_affinity;	   /* Forced channel affinity (0,'v','w') */
	int				 crop;			   /* = 1 if we crop not squeeze video. */
	Xvhw_Drw_Area 	 drawable_area;    /* Current drawable area */
    int        	  	 fb_height;    	   /* # of lines in host fb. */
	int			  	 fb_pitch;		   /* # of pixels per host fb line. */
	int			  	 fd_ccube;	  	   /* File descriptor for device/ccube. */
	int			  	 fd_image;	  	   /* File descriptor for image dma. */
	O_Draw_Gc		 gc_enable;	       /* Drawing GC to enable new area. */
	O_Draw_Gc		 gc_exposure;	   /* Drawing GC to re-enable old area. */
	u_short		  	 *huffman;		   /* Huffman table. */
	Xvhw_Levels	  	 levels;		   /* Brightness contrast hue & saturation*/
	Chan_pv*		 p_channels;	   /* Channel allocation information. */
	XV_struct*		 p_context;		   /* Hardware library context. */
	void*			 p_draw;		   /* Drawable on this port (see 1 above) */
	u_short*	  	 q_compression;	   /* Q table for compression. */
	u_short*	  	 q_decompression;  /* Q table for decompression. */
	int			  	 screen;		   /* X11 screen xv is for. */
	Xvhw_Signal	  	 signal;		   /* Signal specification. */
	Xvhw_Sig_Area 	 signal_area;	   /* Current pip source (signal) area. */
	int			  	 video_frozen;     /* = 1 if pip explicitly stopped. */
	int			  	 video_off_semaphore; /* Resume / suspend semaphore. */
	int			  	 video_port;	   /* Video port currently utilizing. */
	int			  	 video_port0;	   /* Identifier for input port 0. */
	int			  	 video_port1;	   /* Identifier for input port 0. */
	int			  	 wid;		   	   /* Window id for this object. */
	int			  	 x_source_zero;    /* X bias for 0 in video signal. */
	int			  	 y_source_zero;    /* Y bias for 0 in video signal. */
	int				 zoom;			   /* Zoom factor. */
} HwPGIpv;

enum {							    /* Values associated with channel field: */
	PLX_VISIBLE =      0x01,		/* Power video visibility bit. */
	PLX_CHANNEL_V =    0x02,    	/* Note this is frame buffer tag value! */
	PLX_CHANNEL_W =    0x04,    	/* Note this is frame buffer tag value! */
	PLX_CHANNEL_C1 =   0x08,    	/* CCUBE channel 1. */
	PLX_CHANNEL_C0 =   0x04<<25,    /* CCUBE channel 0. */
	PLX_CHANNEL_C2 =   0x05<<25,    /* CCUBE channel 2. */
	PLX_CHANNEL_C3 =   0x06<<25,    /* CCUBE channel 3. */
	PLX_CHANNEL_C4 =   0x07<<25,    /* CCUBE channel 4. */
#if defined(_AIX) || defined(sun) 
	PLX_CHANNEL_MASK = 0xfe			/* Mask for channel values. */
#else
	PLX_CHANNEL_MASK = 0x0e			/* Mask for channel values. */
#endif
};

#define Xvplx_Get_Compressed_Image(obj, l_data, data, src_x, srcy, src_w, src_h, dst_x, dsty, dst_w, dst_h) \
	(*((HwPGIpv *)obj)->atv->get_compressed_image)(obj, l_data, data, \
		src_x, srcy, src_w, src_h, dst_x, dsty, dst_w, dst_h)

#define Xvplx_Put_Compressed_Image(obj, l_data, data, src_x, srcy, src_w, src_h, dst_x, dsty, dst_w, dst_h) \
	(*((HwPGIpv *)obj)->atv->put_compressed_image)(obj, l_data, data, \
		src_x, srcy, src_w, src_h, dst_x, dsty, dst_w, dst_h)

#define Xvplx_Set_Compression_Table(obj, l_data, data) \
	(*((HwPGIpv *)obj)->atv->set_compress_table)(obj, l_data, data)

#define Xvplx_Set_Decompression_Table(obj, l_data, data) \
	(*((HwPGIpv *)obj)->atv->set_decompress_table)(obj, l_data, data)

enum {                      /* Zoom encoding options. */
    OPT_0_ZOOM2 = OPT_2+1,
    OPT_1_ZOOM2,
    OPT_2_ZOOM2,
    OPT_0_ZOOM4,
    OPT_1_ZOOM4,
    OPT_2_ZOOM4,
};

enum {
	PGI_Fill_Disable = 0x02,	/* Fill video enable to disable viewing. */
	PGI_Fill_Enable = 0xff,		/* Clear memory and enable viewing. */
	PGI_Fill_Exposure = 1,		/* Enable viewing, don't touch image. */
	PGI_Fill_Remove = 0			/* Disable viewing and remove image. */
};

#endif /* xvhw_PGItv2_INCLUDED */
