static char sccsid[] = "@(#)70  1.4  src/UMS/usr/lpp/XV/server/dix/xvddx.c, Xv, ums212 3/6/95 15:10:02";
/*
 *   COMPONENT_NAME: xv
 *
 *   FUNCTIONS: none
 *
 *   ORIGINS: 16, 145, 27
 *
 *   This module contains IBM CONFIDENTIAL code.
 *   OBJECT CODE ONLY SOURCE MATERIALS
 *
 *   (C) COPYRIGHT International Business Machines Corp. 1995
 *   All Rights Reserved
 *   US Government Users Restricted Rights - Use, duplication or
 *   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

/***********************************************************
Copyright 1991 by Digital Equipment Corporation, Maynard, Massachusetts
and the Massachusetts Institute of Technology, Cambridge, Massachusetts.
Copyright 1994 by Parallax Corporation, Santa Clara, California.

                        All Rights Reserved

Permission to use, copy, modify, and distribute this software and its 
documentation for any purpose and without fee is hereby granted, 
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in 
supporting documentation, and that the names of Digital or MIT not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.  

DIGITAL DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
DIGITAL BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.

PARALLAX DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
DIGITAL BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
************************************************************/
/*
** File: 
**
**   xvddx.c --- Xv interface to device-dependent layer.
**
** Authors: 
**
**   David Carver  (Digital Workstation Engineering/Project Athena)
**   Andy Goodrich (Parallax and RasterOps)
**   
** Revisions:
**
**   01 Dec 94 Goodrich
**     - initial coding, collecting routines from former xvddx.c and adding
**       additional ones to make device-dependent layer implementations easier.
**
** $Log: xvddx.c,v $
 * Revision 1.4  1996/04/04  01:48:41  dist
 * no dummy adaptor in hp X11R6
 *
 * Revision 1.3  95/11/03  12:28:54  12:28:54  dist (Software Distribution Account)
 * Andy Goodrich: Check in at start of port to Solaris 2.5.
 * 
 * Revision 1.2  95/05/25  16:59:53  16:59:53  dist ()
 * Andy Goodrich - Combined HP/AIX version for HP 9.1 code freeze.
 * 
*/
#include <stdio.h>
#include <sys/types.h>

/*  XVDIAG bit assignments:
 *
 *    0x01 - normal one-shot debugging
 *    0x40  - window wrapper debugging.
 *    0x80 - initialization call tracing.
 */
/* #define XV_DIAG 0x80   */
#include "xvdebug.h"   
 
#include "X.h"
#include "Xproto.h"
#include "scrnintstr.h"
#include "validate.h"
#include "windowstr.h"
#include "pixmapstr.h"
#include "gcstruct.h"
#include "dixstruct.h"

#include "Xv.h"
#include "Xvproto.h"
#include "xvdix.h"

#define GLOBAL
#include "xvddx.h"
#undef GLOBAL

#include <sys/time.h>

#ifdef _AIX
#include <fcntl.h>
#include <sys/errno.h>
#include <sys/ldr.h>
#endif /* _AIX */

/*  FORWARD REFERENCES: */

static int     XvddAdaptorsConfig();
static int     XvddQueryAdaptors();
int            XvddScreenInit();

#ifndef _AIX

extern XvAdaptorPtr PGIpvAdaptorInit();
extern XvAdaptorPtr DummyAdaptorInit();

static struct {
    char*        type;       /* Type name. */
    XvAdaptorPtr (*config)(); /* Hardware specific adaptor initializer. */
} adaptors[] =
{
#ifndef X11R6
    {"Dummy", DummyAdaptorInit},
#endif
    {"PGItv2", PGIpvAdaptorInit},
};
#define N_ADAPTOR_TYPES (sizeof(adaptors)/sizeof(adaptors[0]))

#else /* _AIX */

#define N_ADAPTOR_TYPES 16

static struct {
    char    type[256];       /* Type name. */
    Bool    (*config)(); /* Hardware specific adaptor initialization routine. */
} adaptors[N_ADAPTOR_TYPES];

static int n_adaptor_types;
#endif /* _AIX */


/*---------------------------------------------------------------------------*
 |"XvddAdaptorsConfig"
 |  
 |  Read the configuration file searching for those adaptors assigned to 
 |  this screen. For each adaptor found call its hardware-specific 
 |  configuration routine to allocate an adaptor record. 
 |
 |		= # of adaptors validated
 *---------------------------------------------------------------------------*/
static int XvddAdaptorsConfig( pScreen, pPadaptors) 
	ScreenPtr		pScreen;	/* -> screen looking for adaptors on. */
	XvAdaptorPtr*	pPadaptors;	/* -> vector for pointers to adaptors. */
{
	int			 c;				 /* Temporary for swallowing comments. */
	char		 device[256];	 /* Name of adaptor's device. */
	FILE*		 fd;			 /* File descriptor for xvconfig. */
	int			 iAdaptor;		 /* Entry in pPadaptor to fill in next. */
	int			 iType;			 /* # of entry in adaptors now accessing. */
#ifdef _AIX
    char         load_file[256]; /* Load file name. */
#endif
	char		 myScreen[20];	 /* Our target screen as a character string. */
	XvAdaptorPtr pAdaptor;		 /* Information about the adaptor. */
	char		 screen[20];	 /* Screen # configuration record is for. */
	char		 type[256];		 /* Type of adaptor. */

	DEBUGF(0x80, (fd, "XvddAdaptorsConfig: entry\n"));

#ifdef _AIX
    if ( !(fd = fopen("/usr/lpp/UMS/lib/xvconfig", "r")) )
#else /* !_AIX */
    if ( !(fd = fopen("/etc/xvconfig", "r")) )
#endif /* _AIX */
    {
        return 0;
    }
	sprintf(myScreen, ":0.%d", pScreen->myNum);

	for ( iAdaptor = 0; ; )
	{
		/*
		 *  PARSE SCREEN NUMBER. IF IT STARTS WITH A # IT IS A COMMENT, SO
		 *  EAT THE REST OF THE LINE.
		 */
		for (;;)
		{
			if ( fscanf(fd, "%s", screen) != 1 ) 
			{
				fclose(fd);
				DEBUGF(0x80,(fd,"XvddAdaptorsConfig: exit rc=%d \n", iAdaptor));
				return iAdaptor;
			}
			if ( screen[0] != '#' ) break;
			for ( c = fgetc(fd); c != EOF && c != '\n'; c = fgetc(fd) ) ;
		}


		/*
		 *  PARSE TYPE DESIGNATION AND DEVICE NAME. IF THE SCREEN IS NOT
		 *  THE ONE WE ARE LOOKING FOR TRY AGAIN.
		 */
		if ( fscanf(fd, "%s", type) != 1 )
		{
			ERRORF((fd, "Unable to get type field for screen %s\n", screen));
			DEBUGF(0x80, (fd, "XvddAdaptorsConfig: exit rc=%d \n", iAdaptor));
			fclose(fd);
			return iAdaptor;
		}
		if ( fscanf(fd, "%s", device) != 1 )
		{
			ERRORF((fd, "Unable to get device field for screen %s type %s\n", 
				screen, type));
			DEBUGF(0x80, (fd, "XvddAdaptorsConfig: exit rc=%d \n", iAdaptor));
			fclose(fd);
			return iAdaptor;
		}
#ifdef _AIX
        if ( fscanf(fd, "%s", load_file) != 1 )
        {
            ERRORF((fd,"Unable to get loadfile field for screen %s type %s\n",
                screen, type));
            DEBUGF(0x80, (fd, "XvddAdaptorConfig: exit rc=%d \n", iAdaptor));
            return iAdaptor;
        }
#endif /* _AIX */

		if ( strcmp(myScreen, screen) ) continue;

		/*
		 *  LOOP LOOKING UP TYPE DESIGNATION IN OUR TYPE TABLE.
		 */
		for ( iType = 0; iType < N_ADAPTOR_TYPES; iType++ )
		{
			if ( !strcmp(type, adaptors[iType].type) )
			{
				pAdaptor = (*adaptors[iType].config)(pScreen, device);
				if ( pAdaptor ) break;
			}
		}
#ifdef _AIX
        if ( iType != n_adaptor_types )
        {
            if(n_adaptor_types == N_ADAPTOR_TYPES) {
                ERRORF((fd, "Overflow adaptor table\n"));
                DEBUGF(0x80, (fd, "Overflow adaptor table\n"));
                return iAdaptor;
            }
            strcpy(adaptors[iType].type, type);

            adaptors[iType].config =
                    load(load_file, (uint) NULL,
                "/lib:/usr/lib:/usr/lpp/X11/bin:/usr/lpp/UMS/lib");
            if(!adaptors[iType].config) {
                ERRORF((fd, "Unable to load %s errno %d\n", load_file, errno));
                DEBUGF(0x80, (fd, "Unable to load %s errno %d\n",
                            load_file, errno));
                if(!fork()) {
                    int efd;
                    char *buffer[1024];
                    buffer[0] = "execerror";
                    buffer[1] = load_file;
                    loadquery(L_GETMESSAGES, &buffer[2], sizeof buffer-8);
                    close(2);
                    do{
                            efd = open("/tmp/xv_log", O_WRONLY|O_APPEND);
                    } while(efd < 2);
                    execvp("/usr/sbin/execerror", buffer);
                    ERRORF((fd, "execerror failure\n"));
                    DEBUGF(0x80, (fd, "execerror failure\n"));
                }
                return iAdaptor;
            }
            pAdaptor = (*adaptors[iType].config)(pScreen, device);

            n_adaptor_types++;

            pPadaptors[iAdaptor] = pAdaptor;
            iAdaptor++;
        }
#else /* !_AIX */
        if ( iType != N_ADAPTOR_TYPES )
        {
            pPadaptors[iAdaptor] = pAdaptor;
            iAdaptor++;
        }
#endif /* _AIX */
		else
		{
			ERRORF((fd, "Unable to find %s\n", device));
			DEBUGF(0x80, (fd, "XvddAdaptorsConfig: exit rc=%d \n", iAdaptor));
			fclose(fd);
			return iAdaptor;
		}
        DEBUGF(0x80, (fd, "XvddAdaptorsConfig: count = %d \n", iAdaptor));
    }
    return iAdaptor;
}


/*----------------------------------------------------------------------------*
 |"XvddQueryAdaptors"
 |
 |  Transfer vector routine to get a pointer to and the count of adaptors
 |  associated with a screen.
 *----------------------------------------------------------------------------*/
static int XvddQueryAdaptors(pScreen, p_nAdaptors, p_pPadaptors)
	ScreenPtr    	pScreen;
	int*            p_nAdaptors;
	XvAdaptorPtr**  p_pPadaptors;
{
	XvScreenPtr pxvScrn;

	pxvScrn = (XvScreenPtr) pScreen->devPrivates[XvScreenIndex].ptr;

	*p_nAdaptors = pxvScrn->nAdaptors;
	*p_pPadaptors = pxvScrn->pPadaptors;

	return (Success);
}


/*----------------------------------------------------------------------------*
 |"XvddScreenInit"
 |
 |  Initialize the Xv structures associated with the screen whose pointer
 |  we were passed. XvddScreenInit is called once for each screen in the 
 |  system, whether it has an active Xv device or not. If a screen does not 
 |  have an Xv device (i.e., XvddAdaptorsConfig returns a failure) a return is 
 |  made without filling in any structures.
 |
 |    = Success on success
 |    = 0 if error is detected
 *----------------------------------------------------------------------------*/
XvddScreenInit(pScreen)
    ScreenPtr pScreen;
{
	int			  nAdaptors;	/* # of adpators found for this screen. */
	XvAdaptorPtr* pPadaptors;	/* Vector of adaptors which have been inited. */
    XvScreenPtr   pxvScrn;		/* Private screen record for XV. */
    int           status;

    DEBUGF(0x80, (fd, "XvddScreenInit: entry\n"));


    /*  DETERMINE WHICH ADAPTORS ARE PRESENT FOR THIS SCREEN.  */

    pScreen->devPrivates[XvScreenIndex].ptr = 0;    /* In case of failure. */
	if ( !(pPadaptors = (XvAdaptorPtr*)xalloc(sizeof(XvAdaptorPtr)*100)) )
	{
		return BadAlloc;
    }
    nAdaptors = XvddAdaptorsConfig(pScreen, pPadaptors);
    DEBUGF(0x80, (fd, "XvddScreenInit: after XvddAdaptorsConfig %d adaptors\n",        nAdaptors));
    if ( !nAdaptors )
    {
		xfree((char*)pPadaptors);
        return -1;
    }


    /* ALL XV DD SCREEN INIT PROCS NEED TO CALL THE DI SCREEN INIT PROC FIRST */

    XvScreenInit(pScreen);
    pxvScrn = (XvScreenPtr) pScreen->devPrivates[XvScreenIndex].ptr;
    DEBUGF(0x80, (fd, "XvddScreenInit: after XvScreenInit call\n"));

    pxvScrn->nAdaptors = nAdaptors;
    pxvScrn->pPadaptors = pPadaptors;
	pxvScrn->ddQueryAdaptors = XvddQueryAdaptors;

    DEBUGF(0x80, (fd, "XvddScreenInit: exit\n"));
	return Success;
}
