static char sccsid[] = "@(#)71  1.2  src/UMS/usr/lpp/XV/server/dix/xvdisp.c, Xv, ums212 2/28/95 15:36:04";
/*
 *   COMPONENT_NAME: xv
 *
 *   FUNCTIONS: none
 *
 *   ORIGINS: 16, 145, 27
 *
 *   This module contains IBM CONFIDENTIAL code.
 *   OBJECT CODE ONLY SOURCE MATERIALS
 *
 *   (C) COPYRIGHT International Business Machines Corp. 1995
 *   All Rights Reserved
 *   US Government Users Restricted Rights - Use, duplication or
 *   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

/***********************************************************
Copyright 1991 by Digital Equipment Corporation, Maynard, Massachusetts,
and the Massachusetts Institute of Technology, Cambridge, Massachusetts.
Copyright 1994 by Parallax Corporation, Santa Clara, California.

                        All Rights Reserved

Permission to use, copy, modify, and distribute this software and its 
documentation for any purpose and without fee is hereby granted, 
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in 
supporting documentation, and that the names of Digital or MIT not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.  

DIGITAL DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
DIGITAL BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.

******************************************************************/
/*
** File: 
**
**   xvdisp.c --- Xv server extension dispatch module.
**
** Author: 
**
**   David Carver (Digital Workstation Engineering/Project Athena)
**
** Revisions:
**
**   08.04.94 Goodrich
**      - added Control request which allows piggyback of arbitrary
**        device specific requests on top of Xv protocol 
**
**   04.02.94 Goodrich
**      - added GetLongPortAttribute and SetLongPortAttribute requests
**      - alphabetized routines and declarations
**
**   11.06.91 Carver
**     - changed SetPortControl to SetPortAttribute
**     - changed GetPortControl to GetPortAttribute
**     - changed QueryBestSize
**
**   15.05.91 Carver
**     - version 2.0 upgrade
**
**   24.01.91 Carver
**     - version 1.4 upgrade
**
 * $Log:	xvdisp.c,v $
 * Revision 1.4  95/12/11  15:25:18  15:25:18  dist ()
 * Andy Goodrich: Merge of hedge changes into existing HP/AIX source.
 * 
 * Revision 1.3  95/11/03  12:29:35  12:29:35  dist ()
 * Andy Goodrich: Check in at start of port to Solaris 2.5.
 * 
 * Revision 1.2  95/05/25  17:01:03  17:01:03  dist ()
 * Andy Goodrich - Combined HP/AIX version for HP 9.1 code freeze.
 * 
*/

#include <stdio.h>

/*  XV_DIAG bit assignments:
 *
 *    0x01 - normal on-shot debugging
 *    0x40 - trace requests 
 */
/* #define XV_DIAG 0x41    */
#include <xvdebug.h>

#include "X.h"
#include "Xproto.h"
#include "misc.h"
#include "scrnintstr.h"
#include "windowstr.h"
#include "pixmapstr.h"
#include "gcstruct.h"
#include "dixstruct.h"
#include "resource.h"
#include "opaque.h"

#include "Xv.h"
#include "Xvproto.h"
#include "xvdix.h"

/* EXTERNAL */

extern void (* ReplySwapVector[256]) ();

int request_trace = 0;		/* = 1 if should produce request trace. */

/* INTERNAL */

static int ProcXvControl();
#ifndef _AIX 
static int ProcXvGetImage();
#endif /* !_AIX #### */
static int ProcXvGetLongPortAttribute();
static int ProcXvGetPortAttribute();
static int ProcXvGetStill();
static int ProcXvGetVideo();
static int ProcXvGrabPort();
#ifndef _AIX 
static int ProcXvPutImage();
#endif /* !_AIX #### */
static int ProcXvPutStill();
static int ProcXvPutVideo();
static int ProcXvQueryAdaptors();
static int ProcXvQueryBestSize();
static int ProcXvQueryEncodings();
static int ProcXvQueryExtension();
static int ProcXvSelectPortNotify();
static int ProcXvSelectVideoNotify();
static int ProcXvSetLongPortAttribute();
static int ProcXvSetPortAttribute();
static int ProcXvStopVideo();
static int ProcXvUngrabPort();

static int SProcXvControl();
#ifndef _AIX 
static int SProcXvGetImage();
#endif /* !_AIX #### */
static int SProcXvGetLongPortAttribute();
static int SProcXvGetPortAttribute();
static int SProcXvGetVideo();
static int SProcXvGetStill();
static int SProcXvGrabPort();
static int SProcXvQueryAdaptors();
static int SProcXvQueryBestSize();
static int SProcXvQueryEncodings();
static int SProcXvQueryExtension();
#ifndef _AIX 
static int SProcXvPutImage();
#endif /* !_AIX #### */
static int SProcXvPutStill();
static int SProcXvPutVideo();
static int SProcXvSelectPortNotify();
static int SProcXvSelectVideoNotify();
static int SProcXvSetLongPortAttribute();
static int SProcXvSetPortAttribute();
static int SProcXvStopVideo();
static int SProcXvUngrabPort();

static int SWriteAdaptorInfo();
static int SWriteEncodingInfo();
static int SWriteFormat();
#ifndef _AIX 
static int SWriteGetImageReply();
#endif /* !_AIX #### */
static int SWriteGetLongPortAttributeReply();
static int SWriteGetPortAttributeReply();
static int SWriteGrabPortReply();
static int SWriteQueryAdaptorsReply();
static int SWriteQueryBestSizeReply();
static int SWriteQueryEncodingsReply();
static int SWriteQueryExtensionReply();
static int SWriteQueryVideoReply();

#define _AllocatePort(_i,_p) \
  ((_p)->id != _i) ? (* (_p)->pAdaptor->ddAllocatePort)(_i,_p,&_p) : Success

#define _WriteAdaptorInfo(_c,_d) \
  if ((_c)->swapped) SWriteAdaptorInfo(_c, _d); \
  else WriteToClient(_c, sz_xvAdaptorInfo, _d)

#define _WriteEncodingInfo(_c,_d) \
  if ((_c)->swapped) SWriteEncodingInfo(_c, _d); \
  else WriteToClient(_c, sz_xvEncodingInfo, _d)

#define _WriteFormat(_c,_d) \
  if ((_c)->swapped) SWriteFormat(_c, _d); \
  else WriteToClient(_c, sz_xvFormat, _d)

#define _WriteGetImageReply(_c,_d) \
  if ((_c)->swapped) SWriteGetImageReply(_c, _d); \
  else WriteToClient(_c, sz_xvGetImageReply, _d)

#define _WriteGetPortAttributeReply(_c,_d) \
  if ((_c)->swapped) SWriteGetPortAttributeReply(_c, _d); \
  else WriteToClient(_c, sz_xvGetPortAttributeReply, _d)

#define _WriteGetLongPortAttributeReply(_c,_d) \
  if ((_c)->swapped) SWriteGetLongPortAttributeReply(_c, _d); \
  else WriteToClient(_c, sz_xvGetLongPortAttributeReply, _d)

#define _WriteGrabPortReply(_c,_d) \
  if ((_c)->swapped) SWriteGrabPortReply(_c, _d); \
  else WriteToClient(_c, sz_xvGrabPortReply, _d)

#define _WriteQueryAdaptorsReply(_c,_d) \
  if ((_c)->swapped) SWriteQueryAdaptorsReply(_c, _d); \
  else WriteToClient(_c, sz_xvQueryAdaptorsReply, _d)

#define _WriteQueryBestSizeReply(_c,_d) \
  if ((_c)->swapped) SWriteQueryBestSizeReply(_c, _d); \
  else WriteToClient(_c, sz_xvQueryBestSizeReply, _d)

#define _WriteQueryEncodingsReply(_c,_d) \
  if ((_c)->swapped) SWriteQueryEncodingsReply(_c, _d); \
  else WriteToClient(_c, sz_xvQueryEncodingsReply, _d)

#define _WriteQueryExtensionReply(_c,_d) \
  if ((_c)->swapped) SWriteQueryExtensionReply(_c, _d); \
  else WriteToClient(_c, sz_xvQueryExtensionReply, _d)

/*
** ProcXvDispatch
**
**
**
*/

int
ProcXvDispatch(client)
register ClientPtr client;
{
    REQUEST(xReq);

    UpdateCurrentTime();

	switch (stuff->data) 
    {
      case xv_PutVideo: return(ProcXvPutVideo(client));
#ifndef _AIX 
      case xv_PutImage: return(ProcXvPutImage(client));
#endif /* !AIX #### */
      case xv_Control: return(ProcXvControl(client));
#ifndef _AIX 
      case xv_GetImage: return(ProcXvGetImage(client));
#endif /* !AIX #### */
      case xv_GetLongPortAttribute: return(ProcXvGetLongPortAttribute(client));
      case xv_GetPortAttribute: return(ProcXvGetPortAttribute(client));
      case xv_GetStill: return(ProcXvGetStill(client));
      case xv_GetVideo: return(ProcXvGetVideo(client));
      case xv_GrabPort: return(ProcXvGrabPort(client));
      case xv_PutStill: return(ProcXvPutStill(client));
      case xv_QueryAdaptors: return(ProcXvQueryAdaptors(client));
      case xv_QueryBestSize: return(ProcXvQueryBestSize(client));
      case xv_QueryEncodings: return(ProcXvQueryEncodings(client));
      case xv_QueryExtension: return(ProcXvQueryExtension(client));
      case xv_SelectVideoNotify: return(ProcXvSelectVideoNotify(client));
      case xv_SelectPortNotify: return(ProcXvSelectPortNotify(client));
      case xv_SetLongPortAttribute: return(ProcXvSetLongPortAttribute(client));
      case xv_SetPortAttribute: return(ProcXvSetPortAttribute(client));
      case xv_StopVideo: return(ProcXvStopVideo(client));
      case xv_UngrabPort: return(ProcXvUngrabPort(client));
      default:
      	if (stuff->data < xvNumRequests)
		{
	  		SendErrorToClient(client, XvReqCode, stuff->data, 0, 
			    BadImplementation);
	  		return(BadImplementation);
		}
      	else
		{
	  		DEBUGF(0x40, (fd, "Unknown request %d request\n", stuff->data)); 
	  		SendErrorToClient(client, XvReqCode, stuff->data, 0, BadRequest);
	  		return(BadRequest);
		}
  	}
}


/*---------------------------------------------------------------------------*
 |"ProcXvControl"
 |
 |  Process a Control request. The contents of the request beyond
 |  the header are arbitrary and the device-dependent layer is responsible
 |  for executing and generating the reply to this request. 
 |  Since this request is an optional implementation return a BadRequest
 |  message if the transfer vector entry is zero.
 | 
 |    = Success on success, error code otherwise.
 *---------------------------------------------------------------------------*/
static int
ProcXvControl(client)
    register ClientPtr client;
{
    XvPortPtr               pPort;
    int                     status;

    REQUEST(xvControlReq);
    REQUEST_AT_LEAST_SIZE(xvControlReq);
	DEBUGF(0x40, (fd, "XvControl: attribute=%x, length = %d\n", 
		stuff->attribute, stuff->length));

    if(!(pPort = LOOKUP_PORT(stuff->port, client) ))
    {
        client->errorValue = stuff->port;
        return (_XvBadPort);
    }   

    if ((status = _AllocatePort(stuff->port, pPort)) != Success)
    {
        client->errorValue = stuff->port;
        return (status);
    }   

    if (!ValidAtom(stuff->attribute))
    {
        client->errorValue = stuff->attribute;
        return(BadAtom);
    }
 
    /*
     *  CALL DEVICE-DEPENDENT LAYER TO PERFORM THE REQUESTED OPERATION:
     */
	if ( !pPort->pAdaptor->ddControl )
	{
		return BadMatch;
	}
    status = (* pPort->pAdaptor->ddControl)(client, pPort,
        stuff->attribute, (stuff->length<<2) - sz_xvControlReq, &stuff[1]);
    if (status != Success)
    {
        client->errorValue = stuff->attribute;
        return status;
    }
    return Success;
}


#ifndef _AIX 
/*---------------------------------------------------------------------------*
 |"ProcXvGetImage"
 |
 |  Process a GetImage request. The device-dependent layer is responsible for 
 |  generating the reply to this request. (This can be done in a standard way 
 |  by calling SendXvGetImageReply.)
 |
 |    = Success on success, error code otherwise.
 *---------------------------------------------------------------------------*/
static int
ProcXvGetImage(client)
register ClientPtr client;

{
    register DrawablePtr  pDraw;
    register GCPtr        pGC;
    XvPortPtr 			  pPort;
    int 				  status;

    REQUEST(xvGetImageReq);
    REQUEST_SIZE_MATCH(xvGetImageReq);
    DEBUGF(0x40, (fd, 
	"XvGetImage: draw=%x, gc=%x, port=%x type=%d (%d,%d,%d,%d) (%d,%d,%d,%d)\n",
	  stuff->drawable, stuff->gc, stuff->port, stuff->image_type,
	  stuff->image_x, stuff->image_y, stuff->image_w, stuff->image_h,
	  stuff->drw_x, stuff->drw_y, stuff->drw_w, stuff->drw_h));

    VALIDATE_DRAWABLE_AND_GC(stuff->drawable, pDraw, pGC, client);

    if(!(pPort = LOOKUP_PORT(stuff->port, client) ))
    {
      client->errorValue = stuff->port;
      return (_XvBadPort);
    }

    if ((status = _AllocatePort(stuff->port, pPort)) != Success)
    {
      client->errorValue = stuff->port;
      return (status);
    }

#ifdef CHECKOUT
    if (!(pPort->type & XvInputMask))
    {
      client->errorValue = stuff->port;
      return (BadMatch);
    }
#endif

    status = XVCALL(diMatchPort)(pPort, pDraw);
    if (status != Success)
    {
      return status;
    }

    status = (* pPort->pAdaptor->ddGetImage)(client, pDraw, pPort, pGC,
		stuff->drw_x, stuff->drw_y, stuff->drw_w, stuff->drw_h,
		stuff->image_x, stuff->image_y, stuff->image_w, stuff->image_h,
		stuff->image_type, stuff+1);
    if (status != Success)
    {
        client->errorValue = stuff->image_type;
        return status;
    }
    return Success;

}
#endif /* !_AIX #### */

/*---------------------------------------------------------------------------*
 |"ProcXvGetLongPortAttribute"
 |
 |  Process a GetPortLongPortAttribute request. The device-dependent layer
 |  is responsible for generating the reply to this request. (This can
 |  be done in a standard way by calling SendXvGetLongPortAttributeReply.)
 |
 |    = Success on success, error code otherwise.
 *---------------------------------------------------------------------------*/
static int
ProcXvGetLongPortAttribute(client)
    register ClientPtr client;
{
    XvPortPtr               pPort;
    xvGetPortAttributeReply rep;
    int                     status;

    REQUEST(xvGetLongPortAttributeReq);
    REQUEST_AT_LEAST_SIZE(xvGetLongPortAttributeReq);
	DEBUGF(0x40, (fd, "XvGetLongPortAttribute: port=%x attribute=%x\n", 
		stuff->port, stuff->attribute));

    if(!(pPort = LOOKUP_PORT(stuff->port, client) ))
    {
        client->errorValue = stuff->port;
        return (_XvBadPort);
    }   

    if ((status = _AllocatePort(stuff->port, pPort)) != Success)
    {
        client->errorValue = stuff->port;
        return (status);
    }   

    if (!ValidAtom(stuff->attribute))
    {
        client->errorValue = stuff->attribute;
        return(BadAtom);
    }
 
    /*
     *  CALL DEVICE-DEPENDENT LAYER TO GET ATTRIBUTE'S VALUE.
     */
    status = (* pPort->pAdaptor->ddGetLongPortAttribute)(client, pPort,
        stuff->attribute);
    if (status != Success)
    {
        client->errorValue = stuff->attribute;
        return status;
    }
    return Success;
}
 

/*---------------------------------------------------------------------------*
 |"ProcXvGetPortAttribute"
 |
 |  Process a GetPortAttribute request. The device-dependent layer
 |  is responsible for returning to this routine a single word of data.
 |
 |    = Success on success, error code otherwise.
 *---------------------------------------------------------------------------*/
static int
ProcXvGetPortAttribute(client)
register ClientPtr client;

{
  XvPortPtr               pPort;
  xvGetPortAttributeReply rep;
  int                     status;
  int                     value;

  REQUEST(xvGetPortAttributeReq);
  REQUEST_SIZE_MATCH(xvGetPortAttributeReq);
  DEBUGF(0x40, (fd, "XvGetPortAttribute: entry port=0x%x attribute=0x%x\n", 
	stuff->port, stuff->attribute));

  if(!(pPort = LOOKUP_PORT(stuff->port, client) ))
    {
      client->errorValue = stuff->port;
      return (_XvBadPort);
    }

  if ((status = _AllocatePort(stuff->port, pPort)) != Success)
    {
      client->errorValue = stuff->port;
      return (status);
    }

  if (!ValidAtom(stuff->attribute))
    {
      client->errorValue = stuff->attribute;
      return(BadAtom);
    }

  status = XVCALL(diGetPortAttribute)(client, pPort, stuff->attribute, &value);
  if (status != Success)
    {
      client->errorValue = stuff->attribute;
      return status;
    }

  rep.type = X_Reply;
  rep.sequenceNumber = client->sequence;
  rep.length = 0;
  rep.value = value;
 
  _WriteGetPortAttributeReply(client, &rep);

  DEBUGF(0x40, (fd,"XvGetPortAttribute: exit value=%d 0x%08x\n", value, value));
  return Success;
}

static int
ProcXvGetStill(client)
register ClientPtr client;

{
  register DrawablePtr pDraw;
  XvPortPtr pPort;
  register GCPtr pGC;
  int status;

  REQUEST(xvGetStillReq);
  REQUEST_SIZE_MATCH(xvGetStillReq);
  DEBUGF(0x40, 
	(fd, "XvGetStill: draw=%x, gc=%x, port=%x (%d,%d,%d,%d) (%d,%d,%d,%d)\n", 
	stuff->drawable, stuff->gc, stuff->port,
	stuff->vid_x, stuff->vid_y, stuff->vid_w, stuff->vid_h,
	stuff->drw_x, stuff->drw_y, stuff->drw_w, stuff->drw_h));

  VALIDATE_DRAWABLE_AND_GC(stuff->drawable, pDraw, pGC, client);

  if(!(pPort = LOOKUP_PORT(stuff->port, client) ))
    {
      client->errorValue = stuff->port;
      return (_XvBadPort);
    }

  if ((status = _AllocatePort(stuff->port, pPort)) != Success)
    {
      client->errorValue = stuff->port;
      return (status);
    }

  if (!(pPort->type & XvOutputMask))
    {
      client->errorValue = stuff->port;
      return (BadMatch);
    }

  status = XVCALL(diMatchPort)(pPort, pDraw);
  if (status != Success)
    {
      return status;
    }

  return XVCALL(diGetStill)(client, pDraw, pPort, pGC,
			    stuff->vid_x, stuff->vid_y,
			    stuff->vid_w, stuff->vid_h,
			    stuff->drw_x, stuff->drw_y,
			    stuff->drw_w, stuff->drw_h);

}

static int
ProcXvGetVideo(client)
register ClientPtr client;

{
  register DrawablePtr pDraw;
  XvPortPtr pPort;
  register GCPtr pGC;
  int status;

  REQUEST(xvGetVideoReq);
  REQUEST_SIZE_MATCH(xvGetVideoReq);
  DEBUGF(0x40, 
	(fd, "XvGetVideo: draw=%x, gc=%x, port=%x (%d,%d,%d,%d) (%d,%d,%d,%d)\n", 
	stuff->drawable, stuff->gc, stuff->port,
	stuff->vid_x, stuff->vid_y, stuff->vid_w, stuff->vid_h,
	stuff->drw_x, stuff->drw_y, stuff->drw_w, stuff->drw_h));

  VALIDATE_DRAWABLE_AND_GC(stuff->drawable, pDraw, pGC, client);

  if(!(pPort = LOOKUP_PORT(stuff->port, client) ))
    {
      client->errorValue = stuff->port;
      return (_XvBadPort);
    }

  if ((status = _AllocatePort(stuff->port, pPort)) != Success)
    {
      client->errorValue = stuff->port;
      return (status);
    }

  if (!(pPort->type & XvOutputMask))
    {
      client->errorValue = stuff->port;
      return (BadMatch);
    }

  status = XVCALL(diMatchPort)(pPort, pDraw);
  if (status != Success)
    {
      return status;
    }

  return XVCALL(diGetVideo)(client, pDraw, pPort, pGC,
			    stuff->vid_x, stuff->vid_y,
			    stuff->vid_w, stuff->vid_h,
			    stuff->drw_x, stuff->drw_y,
			    stuff->drw_w, stuff->drw_h);

}

static int
ProcXvGrabPort(client)
register ClientPtr client;

{
  int result, status;
  XvPortPtr pPort;
  xvGrabPortReply rep;

  REQUEST(xvGrabPortReq);
  REQUEST_SIZE_MATCH(xvGrabPortReq);
  DEBUGF(0x40, (fd, "XvGrabPort: port=%x\n", stuff->port));

  if(!(pPort = LOOKUP_PORT(stuff->port, client) ))
    {
      client->errorValue = stuff->port;
      return (_XvBadPort);
    }

  if ((status = _AllocatePort(stuff->port, pPort)) != Success)
    {
      client->errorValue = stuff->port;
      return (status);
    }

  status = XVCALL(diGrabPort)(client, pPort, stuff->time, &result);

  if (status != Success)
    {
      return status;
    }

  rep.type = X_Reply;
  rep.sequenceNumber = client->sequence;
  rep.length = 0;
  rep.result = result;

  _WriteGrabPortReply(client, &rep);

  return Success;

}

static int
ProcXvPutStill(client)
register ClientPtr client;

{
  register DrawablePtr pDraw;
  XvPortPtr pPort;
  register GCPtr pGC;
  int status;

  REQUEST(xvPutStillReq);
  REQUEST_SIZE_MATCH(xvPutStillReq);
  DEBUGF(0x40, 
	(fd, "XvPutStill: draw=%x, gc=%x, port=%x (%d,%d,%d,%d) (%d,%d,%d,%d)\n", 
	stuff->drawable, stuff->gc, stuff->port,
	stuff->vid_x, stuff->vid_y, stuff->vid_w, stuff->vid_h,
	stuff->drw_x, stuff->drw_y, stuff->drw_w, stuff->drw_h));

  VALIDATE_DRAWABLE_AND_GC(stuff->drawable, pDraw, pGC, client);

  if(!(pPort = LOOKUP_PORT(stuff->port, client) ))
    {
      client->errorValue = stuff->port;
      return (_XvBadPort);
    }

  if ((status = _AllocatePort(stuff->port, pPort)) != Success)
    {
      client->errorValue = stuff->port;
      return (status);
    }

  if (!(pPort->type & XvInputMask))
    {
      client->errorValue = stuff->port;
      return (BadMatch);
    }

  status = XVCALL(diMatchPort)(pPort, pDraw);
  if (status != Success)
    {
      return status;
    }

  return XVCALL(diPutStill)(client, pDraw, pPort, pGC,
			    stuff->vid_x, stuff->vid_y,
			    stuff->vid_w, stuff->vid_h,
			    stuff->drw_x, stuff->drw_y,
			    stuff->drw_w, stuff->drw_h);

}

#ifndef _AIX 
static int
ProcXvPutImage(client)
register ClientPtr client;

{
  	register DrawablePtr pDraw;
  	XvPortPtr pPort;
  	register GCPtr pGC;
  	int status;
	
  	REQUEST(xvPutImageReq);
  	REQUEST_AT_LEAST_SIZE(xvPutImageReq);
	
  	DEBUGF(0x41, (fd, 
	  "XvPutImage: draw=%x gc=%x port=%x type=%d (%d,%d,%d,%d) (%d,%d,%d,%d)\n",
		stuff->drawable, stuff->gc, stuff->port, stuff->image_type,
		stuff->image_x, stuff->image_y, stuff->image_w, stuff->image_h,
		stuff->drw_x, stuff->drw_y, stuff->drw_w, stuff->drw_h));
	
  	VALIDATE_DRAWABLE_AND_GC(stuff->drawable, pDraw, pGC, client);
	
  	if( !(pPort = LOOKUP_PORT(stuff->port, client)) )
	{
      	client->errorValue = stuff->port;
      	return (_XvBadPort);
	}
	
  	if ((status = _AllocatePort(stuff->port, pPort)) != Success)
	{
      	client->errorValue = stuff->port;
      	return (status);
	}
	
  	if (!(pPort->type & XvInputMask))
	{
      	client->errorValue = stuff->port;
      	return (BadMatch);
	}
	
  	status = XVCALL(diMatchPort)(pPort, pDraw);
  	if (status != Success)
	{
      	return status;
	}
	
  	status = (* pPort->pAdaptor->ddPutImage)(client, pDraw, pPort, pGC,
		stuff->drw_x, stuff->drw_y, stuff->drw_w, stuff->drw_h,
		stuff->image_x, stuff->image_y, stuff->image_w, stuff->image_h,
		stuff->image_type, stuff->length - sz_xvPutImageReq, stuff+1);
	return status;
}
#endif /* !AIX #### */

static int
ProcXvPutVideo(client)
register ClientPtr client;

{
  register DrawablePtr pDraw;
  XvPortPtr pPort;
  register GCPtr pGC;
  int status;

  REQUEST(xvPutVideoReq);
  REQUEST_SIZE_MATCH(xvPutVideoReq);
  DEBUGF(0x40, 
	(fd, "XvPutVideo: draw=%x, gc=%x, port=%x (%d,%d,%d,%d) (%d,%d,%d,%d)\n", 
	stuff->drawable, stuff->gc, stuff->port,
	stuff->vid_x, stuff->vid_y, stuff->vid_w, stuff->vid_h,
	stuff->drw_x, stuff->drw_y, stuff->drw_w, stuff->drw_h));

  VALIDATE_DRAWABLE_AND_GC(stuff->drawable, pDraw, pGC, client);

  if(!(pPort = LOOKUP_PORT(stuff->port, client) ))
    {
      client->errorValue = stuff->port;
      return (_XvBadPort);
    }

  if ((status = _AllocatePort(stuff->port, pPort)) != Success)
    {
      client->errorValue = stuff->port;
      return (status);
    }

  if (!(pPort->type & XvInputMask))
    {
      client->errorValue = stuff->port;
      return (BadMatch);
    }

  status = XVCALL(diMatchPort)(pPort, pDraw);
  if (status != Success)
    {
      return status;
    }

  return XVCALL(diPutVideo)(client, pDraw, pPort, pGC,
			    stuff->vid_x, stuff->vid_y,
			    stuff->vid_w, stuff->vid_h,
			    stuff->drw_x, stuff->drw_y,
			    stuff->drw_w, stuff->drw_h);

}

static int
ProcXvQueryAdaptors(client)
register ClientPtr client;

{
  xvFormat format;
  xvEncodingInfo einfo;
  xvAdaptorInfo ainfo;
  xvQueryAdaptorsReply rep;
  int totalSize;
  int na;
  int	iAdaptor;
  XvAdaptorPtr pa;
  int nf;
  XvFormatPtr pf;
  WindowPtr pWin;
  ScreenPtr pScreen;
  XvScreenPtr pxvs;
  int		  iPort;

  REQUEST(xvQueryAdaptorsReq);
  REQUEST_SIZE_MATCH(xvQueryAdaptorsReq);
  DEBUGF(0x40, (fd, "XvQueryAdaptors: entry\n"));

  if(!(pWin = (WindowPtr)LookupWindow(stuff->window, client) ))
    {
      client->errorValue = stuff->window;
	  DEBUGF(0x40, (fd, "XvQueryAdaptors: exit rc=BadWindow\n"));
      return (BadWindow);
    }

  pScreen = pWin->drawable.pScreen;
  pxvs = (XvScreenPtr)pScreen->devPrivates[XvScreenIndex].ptr;

  if (!pxvs)
    {
      rep.type = X_Reply;
      rep.sequenceNumber = client->sequence;
      rep.num_adaptors = 0;
      rep.length = 0;

      _WriteQueryAdaptorsReply(client, &rep);

	  DEBUGF(0x40, (fd, "XvQueryAdaptors: exit rc=No screen private area\n"));
      return Success;
    }

  DEBUGF(1, (fd, "XvQueryAdaptors: before ddQueryAdaptors\n"));
  (* pxvs->ddQueryAdaptors)(pScreen, &pxvs->nAdaptors, &pxvs->pPadaptors);
  DEBUGF(1, (fd, "XvQueryAdaptors: after ddQueryAdaptors\n"));

  rep.type = X_Reply;
  rep.sequenceNumber = client->sequence;
  rep.num_adaptors = pxvs->nAdaptors;

  /* CALCULATE THE TOTAL SIZE OF THE REPLY IN BYTES */

  totalSize = pxvs->nAdaptors * sz_xvAdaptorInfo;

  /* FOR EACH ADPATOR ADD UP THE BYTES FOR ENCODINGS AND FORMATS */

  DEBUGF(1, (fd, "XvQueryAdaptors: before size calculation\n"));
  na = pxvs->nAdaptors;
  for ( iAdaptor = 0; iAdaptor < na; iAdaptor++ )
    {
	  pa = pxvs->pPadaptors[iAdaptor];
      totalSize += (strlen(pa->name) + 3) & ~3;
      totalSize += pa->nFormats * sz_xvFormat;
    }

  rep.length = totalSize >> 2;

  DEBUGF(1, (fd, "XvQueryAdaptors: before WriteQueryAdaptors reply\n"));
  _WriteQueryAdaptorsReply(client, &rep);
  DEBUGF(1, (fd, "XvQueryAdaptors: after WriteQueryAdaptors reply\n"));

  na = pxvs->nAdaptors;
  for ( iAdaptor = 0; iAdaptor < na; iAdaptor++ )
    {
      pa = pxvs->pPadaptors[iAdaptor];

      ainfo.base_id = pa->base_id;
      ainfo.num_ports = pa->nPorts;
      ainfo.type = pa->pPorts->type;
	  for ( iPort = 1; iPort < pa->nPorts; iPort++ )
	  {
		  ainfo.type |= pa->pPorts[iPort].type; /* #### FIX THIS SOME DAY. */
	  }
      ainfo.name_size = strlen(pa->name);
      ainfo.num_formats = pa->nFormats;

	  DEBUGF(1, (fd, "XvQueryAdaptors: before WriteAdaptorInfo adaptor %d\n", iAdaptor));
      _WriteAdaptorInfo(client, &ainfo);
	  DEBUGF(1, (fd, "XvQueryAdaptors: after WriteAdaptorInfo adaptor %d\n", iAdaptor));
	  DEBUGF(1, (fd, "XvQueryAdaptors: writing adaptor %s \n", pa->name));

      WriteToClient(client, strlen(pa->name), pa->name);
	  DEBUGF(1, (fd, "XvQueryAdaptors: after WriteToClient adaptor %d\n", iAdaptor));

      nf = pa->nFormats;
      pf = pa->pFormats;
      while (nf--)
	{
	  format.depth = pf->depth;
	  format.visual = pf->visual;
	  _WriteFormat(client, &format);
	  pf++;
	}

    }

  DEBUGF(0x40, (fd, "XvQueryAdaptors: exit rc=%d(No client exception)\n", 
	client->noClientException));
  return (client->noClientException);

}

static int
ProcXvQueryBestSize(client)
register ClientPtr client;

{
  int status;
  CARD16 actual_width, actual_height;
  XvPortPtr pPort;
  xvQueryBestSizeReply rep;
  REQUEST(xvQueryBestSizeReq);
  REQUEST_SIZE_MATCH(xvQueryBestSizeReq);
  DEBUGF(0x40, (fd, "XvQueryBestSize: port=%x motion=%x (%d,%d) (%d,%d)\n",
	stuff->port, stuff->motion, stuff->vid_w, stuff->vid_h,
	stuff->drw_w, stuff->drw_h));

  if(!(pPort = LOOKUP_PORT(stuff->port, client) ))
    {
      client->errorValue = stuff->port;
	  DEBUGF(0x40, (fd, "XvQueryBestSize: exit rc=BadPort\n"));
      return (_XvBadPort);
    }

  if ((status = _AllocatePort(stuff->port, pPort)) != Success)
    {
      client->errorValue = stuff->port;
	  DEBUGF(0x40, (fd, "XvQueryBestSize: exit rc=%d\n", status));
      return (status);
    }

  rep.type = X_Reply;
  rep.sequenceNumber = client->sequence;
  rep.length = 0;

  (* pPort->pAdaptor->ddQueryBestSize)(client, pPort, stuff->motion,
				       stuff->vid_w, stuff->vid_h, 
				       stuff->drw_w, stuff->drw_h, 
				       &actual_width, &actual_height);

  rep.actual_width = actual_width;
  rep.actual_height = actual_height;
 
  _WriteQueryBestSizeReply(client, &rep);

  DEBUGF(0x40, (fd, "XvQueryBestSize: exit rc=Success\n"));
  return Success;
}

static int
ProcXvSelectPortNotify(client)
register ClientPtr client;

{
  int status;
  XvPortPtr pPort;
  REQUEST(xvSelectPortNotifyReq);
  REQUEST_SIZE_MATCH(xvSelectPortNotifyReq);
  DEBUGF(0x40, (fd, "XvSelectPortNotify: port=%x\n", stuff->port));

  if(!(pPort = LOOKUP_PORT(stuff->port, client) ))
    {
      client->errorValue = stuff->port;
	  DEBUGF(0x40, (fd, "XvSelectPortNotify: exit rc=BadPort\n"));
      return (_XvBadPort);
    }

  if ((status = _AllocatePort(stuff->port, pPort)) != Success)
    {
      client->errorValue = stuff->port;
	  DEBUGF(0x40, (fd, "XvSelectPortNotify: exit rc=%d\n", status));
      return (status);
    }

  DEBUGF(0x40, (fd, "XvSelectPortNotify: success exit via SelectPortNotify\n"));
  return XVCALL(diSelectPortNotify)(client, pPort, stuff->onoff);

}

static int
ProcXvQueryEncodings(client)
register ClientPtr client;

{
  xvEncodingInfo einfo;
  xvQueryEncodingsReply rep;
  int totalSize;
  XvPortPtr pPort;
  int ne;
  XvEncodingPtr pe;
  int status;

  REQUEST(xvQueryEncodingsReq);
  REQUEST_SIZE_MATCH(xvQueryEncodingsReq);
  DEBUGF(0x40, (fd, "XvQueryEncodings: port=%x\n", stuff->port));

  if(!(pPort = LOOKUP_PORT(stuff->port, client) ))
    {
      client->errorValue = stuff->port;
	  DEBUGF(0x40, (fd, "XvQueryEncodings: exit rc = BadPort\n"));
      return (_XvBadPort);
    }

  if ((status = _AllocatePort(stuff->port, pPort)) != Success)
    {
      client->errorValue = stuff->port;
	  DEBUGF(0x40, (fd, "XvQueryEncodings: exit rc = %d\n", status));
      return (status);
    }

  rep.type = X_Reply;
  rep.sequenceNumber = client->sequence;
  rep.num_encodings = pPort->nEncodings;

  /* FOR EACH ENCODING ADD UP THE BYTES FOR ENCODING NAMES */

  ne = pPort->nEncodings;
  pe = pPort->pEncodings;
  totalSize = ne * sz_xvEncodingInfo;
  while (ne--)
    {
      totalSize += (strlen(pe->name) + 3) & ~3;
      pe++;
    }

  rep.length = totalSize >> 2;

  _WriteQueryEncodingsReply(client, &rep);

  ne = pPort->nEncodings;
  pe = pPort->pEncodings;
  do
    {
      einfo.encoding = pe->id;
      einfo.name_size = strlen(pe->name);
      einfo.width = pe->width;
      einfo.height = pe->height;
      einfo.rate.numerator = pe->rate.numerator;
      einfo.rate.denominator = pe->rate.denominator;
      _WriteEncodingInfo(client, &einfo);
      WriteToClient(client, strlen(pe->name), pe->name);
      pe++;
    } while (--ne);

  DEBUGF(0x40, (fd, "XvQueryEncodings: exit rc = noClientException\n"));
  return (client->noClientException);

}

static int
ProcXvQueryExtension(client)
register ClientPtr client;

{
  xvQueryExtensionReply rep;
  REQUEST(xvQueryExtensionReq);
  REQUEST_SIZE_MATCH(xvQueryExtensionReq);

  DEBUGF(0x40, (fd, "XvQueryExtension: entry\n"));

  rep.type = X_Reply;
  rep.sequenceNumber = client->sequence;
  rep.length = 0;
  rep.version = XvVersion;
  rep.revision = XvRevision;

  _WriteQueryExtensionReply(client, &rep);

  DEBUGF(0x40, (fd, "XvQueryExtension: exit\n"));

  return Success;

}

static int
ProcXvSelectVideoNotify(client)
register ClientPtr client;

{
  register DrawablePtr pDraw;
  XvPortPtr pPort;

  REQUEST(xvSelectVideoNotifyReq);
  REQUEST_SIZE_MATCH(xvSelectVideoNotifyReq);
  DEBUGF(0x40, (fd, "XvSelectVideoNotify: draw=%x %s\n", 
	stuff->drawable, (char*)( stuff->onoff ? "on" : "off") ));

  if(!(pDraw = (DrawablePtr)LOOKUP_DRAWABLE(stuff->drawable, client) ))
    {
      client->errorValue = stuff->drawable;
      return (BadWindow);
    }

  return XVCALL(diSelectVideoNotify)(client, pDraw, stuff->onoff);

}

static int
ProcXvSetLongPortAttribute(client)
register ClientPtr client;

{
  register  len;
  XvPortPtr pPort;
  int       status;

  REQUEST(xvSetLongPortAttributeReq);
  REQUEST_AT_LEAST_SIZE(xvSetLongPortAttributeReq);
  DEBUGF(0x40, (fd, "XvSetLongPortAttribute: port=%x attr=%x len=%d\n", 
	stuff->port, stuff->attribute, stuff->attr_len));

  if(!(pPort = LOOKUP_PORT(stuff->port, client) ))
    {
      client->errorValue = stuff->port;
      return (_XvBadPort);
    }

  if ((status = _AllocatePort(stuff->port, pPort)) != Success)
    {
      client->errorValue = stuff->port;
      return (status);
    }

  if (!ValidAtom(stuff->attribute))
    {
      client->errorValue = stuff->attribute;
      return(BadAtom);
    }

  status = (* pPort->pAdaptor->ddSetLongPortAttribute)(client, pPort,
	stuff->attribute, stuff->attr_len, &stuff[1]);

  if (status == BadMatch) 
      client->errorValue = stuff->attribute;
  else
      client->errorValue = 0xdeadbeaf;

  return status;

}

static int
ProcXvSetPortAttribute(client)
register ClientPtr client;

{
  int status;
  register len;
  XvPortPtr pPort;

  REQUEST(xvSetPortAttributeReq);
  REQUEST_SIZE_MATCH(xvSetPortAttributeReq);
  DEBUGF(0x40, (fd, "XvSetPortAttribute: port=%x attr=%x value=0x%x (%d)\n", 
	stuff->port, stuff->attribute, stuff->value, stuff->value));

  if(!(pPort = LOOKUP_PORT(stuff->port, client) ))
    {
      client->errorValue = stuff->port;
      return (_XvBadPort);
    }

  if ((status = _AllocatePort(stuff->port, pPort)) != Success)
    {
      client->errorValue = stuff->port;
      return (status);
    }

  if (!ValidAtom(stuff->attribute))
    {
      client->errorValue = stuff->attribute;
      return(BadAtom);
    }

  status = XVCALL(diSetPortAttribute)(client, pPort, 
				    stuff->attribute, stuff->value);

  if (status == BadMatch) 
      client->errorValue = stuff->attribute;
  else
      client->errorValue = stuff->value;

  return status;

}

static int
ProcXvStopVideo(client)
register ClientPtr client;

{
  int status;
  register DrawablePtr pDraw;
  XvPortPtr pPort;

  REQUEST(xvStopVideoReq);
  REQUEST_SIZE_MATCH(xvStopVideoReq);
  DEBUGF(0x40, (fd, "XvStopVideo: draw= %x port=%x\n", 
	stuff->drawable, stuff->port));

  if(!(pPort = LOOKUP_PORT(stuff->port, client) ))
    {
      client->errorValue = stuff->port;
      return (_XvBadPort);
    }

  if ((status = _AllocatePort(stuff->port, pPort)) != Success)
    {
      client->errorValue = stuff->port;
      return (status);
    }

  if(!(pDraw = LOOKUP_DRAWABLE(stuff->drawable, client) ))
    {
      client->errorValue = stuff->drawable;
      return (BadDrawable);
    }

  return XVCALL(diStopVideo)(client, pPort, pDraw);

}

static int
ProcXvUngrabPort(client)
register ClientPtr client;

{
  int status;
  XvPortPtr pPort;

  REQUEST(xvGrabPortReq);
  REQUEST_SIZE_MATCH(xvGrabPortReq);
  DEBUGF(0x40, (fd, "XvUngrabPort: port=%x\n", stuff->port));

  if(!(pPort = LOOKUP_PORT(stuff->port, client) ))
    {
      client->errorValue = stuff->port;
      return (_XvBadPort);
    }

  if ((status = _AllocatePort(stuff->port, pPort)) != Success)
    {
      client->errorValue = stuff->port;
      return (status);
    }

  return XVCALL(diUngrabPort)(client, pPort, stuff->time);

}

#ifndef _AIX 
/*---------------------------------------------------------------------------*
 |"SendXvGetImageReply"
 |
 |  Send the X reply for a GetImage request. Note we write data if we
 |  were given a non-null pointer to data. If we are given a non-zero
 |  length and a null pointer the caller will supply the data.
 |
 |    = Success
 *---------------------------------------------------------------------------*/
int SendXvGetImageReply(client, image_width, image_height, nImage, pImage)
    register ClientPtr  client;
	int					image_width;
	int					image_height;
	int					nImage;
    char                *pImage;
{
    xvGetPortAttributeReply rep;

    rep.type = X_Reply;
    rep.sequenceNumber = client->sequence;
    rep.length = (nImage + 3) >> 2;
    _WriteGetImageReply(client, &rep);

    if ( pImage )
    {
        WriteToClient(client, nImage, pImage);
    }

    return Success;
}
#endif /* !_AIX #### */


/*---------------------------------------------------------------------------*
 |"SendXvGetLongPortAttributeReply"
 |
 |  Send the X reply for a GetPortLongPortAttribute request.
 |    = Success
 *---------------------------------------------------------------------------*/
int SendXvGetLongPortAttributeReply(client, length, pValue)
    register ClientPtr  client;
    int                 length;
    char                *pValue;
{
    xvGetPortAttributeReply rep;

    rep.type = X_Reply;
    rep.sequenceNumber = client->sequence;
    rep.length = (length + 3) >> 2;
    _WriteGetLongPortAttributeReply(client, &rep);

    if ( length )
    {
        WriteToClient(client, length, pValue);
    }

    return Success;
}


/* Swapped Procs */

int
SProcXvDispatch(client)
register ClientPtr client;
{
  REQUEST(xReq);

  UpdateCurrentTime();

  switch (stuff->data) 
  {
    case xv_PutVideo: return(SProcXvPutVideo(client));
#ifndef _AIX 
    case xv_PutImage: return(SProcXvPutImage(client));
#endif /* !_AIX #### */

    case xv_Control: return(SProcXvControl(client));
#ifndef _AIX 
    case xv_GetImage: return(SProcXvGetImage(client));
#endif /* !_AIX #### */
    case xv_GetLongPortAttribute: return(SProcXvGetLongPortAttribute(client));
    case xv_GetPortAttribute: return(SProcXvGetPortAttribute(client));
    case xv_GetStill: return(SProcXvGetStill(client));
    case xv_GetVideo: return(SProcXvGetVideo(client));
    case xv_GrabPort: return(SProcXvGrabPort(client));
    case xv_QueryAdaptors: return(SProcXvQueryAdaptors(client));
    case xv_QueryBestSize: return(SProcXvQueryBestSize(client));
    case xv_QueryEncodings: return(SProcXvQueryEncodings(client));
    case xv_QueryExtension: return(SProcXvQueryExtension(client));
    case xv_PutStill: return(SProcXvPutStill(client));
    case xv_SelectPortNotify: return(SProcXvSelectPortNotify(client));
    case xv_SelectVideoNotify: return(SProcXvSelectVideoNotify(client));
    case xv_SetLongPortAttribute: return(SProcXvSetLongPortAttribute(client));
    case xv_SetPortAttribute: return(SProcXvSetPortAttribute(client));
    case xv_StopVideo: return(SProcXvStopVideo(client));
    case xv_UngrabPort: return(SProcXvUngrabPort(client));
    default:
      if (stuff->data < xvNumRequests)
	{
	  SendErrorToClient(client, XvReqCode, stuff->data, 0, 
			    BadImplementation);
	  return(BadImplementation);
	}
      else
	{
	  SendErrorToClient(client, XvReqCode, stuff->data, 0, BadRequest);
	  return(BadRequest);
	}
  }
}

static int
SProcXvControl(client)
register ClientPtr client;

{
  register char n;
  REQUEST(xvControlReq);
  swaps(&stuff->length, n);
  swapl(&stuff->port, n);
  swapl(&stuff->attribute, n);
  return ProcXvControl(client);
}

#ifndef _AIX 
static int
SProcXvGetImage(client)
register ClientPtr client;

{
  register char n;
  REQUEST(xvGetImageReq);
  swaps(&stuff->length, n);
  swapl(&stuff->port, n);
  swapl(&stuff->drawable, n);
  swapl(&stuff->gc, n);
  swaps(&stuff->drw_x, n);
  swaps(&stuff->drw_y, n);
  swaps(&stuff->drw_w, n);
  swaps(&stuff->drw_h, n);
  swaps(&stuff->image_x, n);
  swaps(&stuff->image_y, n);
  swaps(&stuff->image_w, n);
  swaps(&stuff->image_h, n);
  swaps(&stuff->image_type, n);
  return ProcXvGetImage(client);
}
#endif /* !_AIX #### */

static int
SProcXvGetLongPortAttribute(client)
register ClientPtr client;

{
  register char n;
  REQUEST(xvGetLongPortAttributeReq);
  swaps(&stuff->length, n);
  swapl(&stuff->port, n);
  swapl(&stuff->attribute, n);
  return ProcXvGetLongPortAttribute(client);
}

static int
SProcXvGetPortAttribute(client)
register ClientPtr client;

{
  register char n;
  REQUEST(xvGetPortAttributeReq);
  swaps(&stuff->length, n);
  swapl(&stuff->port, n);
  swapl(&stuff->attribute, n);
  return ProcXvGetPortAttribute(client);
}

static int
SProcXvGetStill(client)
register ClientPtr client;

{
  register char n;
  REQUEST(xvGetStillReq);
  swaps(&stuff->length, n);
  swapl(&stuff->port, n);
  swapl(&stuff->drawable, n);
  swapl(&stuff->gc, n);
  swaps(&stuff->vid_x, n);
  swaps(&stuff->vid_y, n);
  swaps(&stuff->vid_w, n);
  swaps(&stuff->vid_h, n);
  swaps(&stuff->drw_x, n);
  swaps(&stuff->drw_y, n);
  swaps(&stuff->drw_w, n);
  swaps(&stuff->drw_h, n);
  return ProcXvGetStill(client);
}

static int
SProcXvGetVideo(client)
register ClientPtr client;

{
  register char n;
  REQUEST(xvGetVideoReq);
  swaps(&stuff->length, n);
  swapl(&stuff->port, n);
  swapl(&stuff->drawable, n);
  swapl(&stuff->gc, n);
  swaps(&stuff->vid_x, n);
  swaps(&stuff->vid_y, n);
  swaps(&stuff->vid_w, n);
  swaps(&stuff->vid_h, n);
  swaps(&stuff->drw_x, n);
  swaps(&stuff->drw_y, n);
  swaps(&stuff->drw_w, n);
  swaps(&stuff->drw_h, n);
  return ProcXvGetVideo(client);
}

static int
SProcXvGrabPort(client)
register ClientPtr client;

{
  register char n;
  REQUEST(xvGrabPortReq);
  swaps(&stuff->length, n);
  swapl(&stuff->port, n);
  swapl(&stuff->time, n);
  return ProcXvGrabPort(client);
}

#ifndef _AIX 
static int
SProcXvPutImage(client)
register ClientPtr client;

{
  register char n;
  REQUEST(xvPutImageReq);
  swaps(&stuff->length, n);
  swapl(&stuff->port, n);
  swapl(&stuff->drawable, n);
  swapl(&stuff->gc, n);
  swaps(&stuff->drw_x, n);
  swaps(&stuff->drw_y, n);
  swaps(&stuff->drw_w, n);
  swaps(&stuff->drw_h, n);
  swaps(&stuff->image_x, n);
  swaps(&stuff->image_y, n);
  swaps(&stuff->image_w, n);
  swaps(&stuff->image_h, n);
  swaps(&stuff->image_type, n);
  return ProcXvPutImage(client);
}
#endif /* !_AIX #### */

static int
SProcXvPutStill(client)
register ClientPtr client;

{
  register char n;
  REQUEST(xvPutStillReq);
  swaps(&stuff->length, n);
  swapl(&stuff->port, n);
  swapl(&stuff->drawable, n);
  swapl(&stuff->gc, n);
  swaps(&stuff->vid_x, n);
  swaps(&stuff->vid_y, n);
  swaps(&stuff->vid_w, n);
  swaps(&stuff->vid_h, n);
  swaps(&stuff->drw_x, n);
  swaps(&stuff->drw_y, n);
  swaps(&stuff->drw_w, n);
  swaps(&stuff->drw_h, n);
  return ProcXvPutStill(client);
}

static int
SProcXvPutVideo(client)
register ClientPtr client;

{
  register char n;
  REQUEST(xvPutVideoReq);
  swaps(&stuff->length, n);
  swapl(&stuff->port, n);
  swapl(&stuff->drawable, n);
  swapl(&stuff->gc, n);
  swaps(&stuff->vid_x, n);
  swaps(&stuff->vid_y, n);
  swaps(&stuff->vid_w, n);
  swaps(&stuff->vid_h, n);
  swaps(&stuff->drw_x, n);
  swaps(&stuff->drw_y, n);
  swaps(&stuff->drw_w, n);
  swaps(&stuff->drw_h, n);
  return ProcXvPutVideo(client);
}

static int
SProcXvQueryAdaptors(client)
register ClientPtr client;

{
  register char n;
  REQUEST(xvQueryAdaptorsReq);
  swaps(&stuff->length, n);
  swapl(&stuff->window, n);
  return ProcXvQueryAdaptors(client);
}

static int
SProcXvQueryBestSize(client)
register ClientPtr client;

{
  register char n;
  REQUEST(xvQueryBestSizeReq);
  swaps(&stuff->length, n);
  swapl(&stuff->port, n);
  swaps(&stuff->vid_w, n);
  swaps(&stuff->vid_h, n);
  swaps(&stuff->drw_w, n);
  swaps(&stuff->drw_h, n);
  return ProcXvQueryBestSize(client);
}

static int
SProcXvQueryEncodings(client)
register ClientPtr client;

{
  register char n;
  REQUEST(xvQueryEncodingsReq);
  swaps(&stuff->length, n);
  swapl(&stuff->port, n);
  return ProcXvQueryEncodings(client);
}

static int
SProcXvQueryExtension(client)
register ClientPtr client;

{
  register char n;
  REQUEST(xvQueryExtensionReq);
  swaps(&stuff->length, n);
  return ProcXvQueryExtension(client);
}

static int
SProcXvSelectPortNotify(client)
register ClientPtr client;

{
  register char n;
  REQUEST(xvSelectPortNotifyReq);
  swaps(&stuff->length, n);
  swapl(&stuff->port, n);
  return ProcXvSelectPortNotify(client);
}

static int
SProcXvSelectVideoNotify(client)
register ClientPtr client;

{
  register char n;
  REQUEST(xvSelectVideoNotifyReq);
  swaps(&stuff->length, n);
  swapl(&stuff->drawable, n);
  return ProcXvSelectVideoNotify(client);
}

static int
SProcXvSetLongPortAttribute(client)
register ClientPtr client;

{
  register char n;
  REQUEST(xvSetLongPortAttributeReq);
  swaps(&stuff->length, n);
  swapl(&stuff->port, n);
  swapl(&stuff->attribute, n);
  swapl(&stuff->attr_len, n);
  return ProcXvSetLongPortAttribute(client);
}

static int
SProcXvSetPortAttribute(client)
register ClientPtr client;

{
  register char n;
  REQUEST(xvSetPortAttributeReq);
  swaps(&stuff->length, n);
  swapl(&stuff->port, n);
  swapl(&stuff->attribute, n);
  swapl(&stuff->value, n);
  return ProcXvSetPortAttribute(client);
}

static int
SProcXvStopVideo(client)
register ClientPtr client;

{
  register char n;
  REQUEST(xvStopVideoReq);
  swaps(&stuff->length, n);
  swapl(&stuff->port, n);
  swapl(&stuff->drawable, n);
  return ProcXvStopVideo(client);
}

static int
SProcXvUngrabPort(client)
register ClientPtr client;

{
  register char n;
  REQUEST(xvUngrabPortReq);
  swaps(&stuff->length, n);
  swapl(&stuff->port, n);
  swapl(&stuff->time, n);
  return ProcXvUngrabPort(client);
}

static int
SWriteAdaptorInfo(client, pAdaptor)
register ClientPtr client;
xvAdaptorInfo *pAdaptor;

{
  register char n;

  swapl(&pAdaptor->base_id, n);
  swaps(&pAdaptor->name_size, n);
  swaps(&pAdaptor->num_ports, n);
  swaps(&pAdaptor->num_formats, n);

  (void)WriteToClient(client, sz_xvAdaptorInfo, (char *)pAdaptor);

}

static int
SWriteEncodingInfo(client, pEncoding)
register ClientPtr client;
xvEncodingInfo *pEncoding;

{
  register char n;
  
  swapl(&pEncoding->encoding, n);
  swaps(&pEncoding->name_size, n);
  swaps(&pEncoding->width, n);
  swaps(&pEncoding->height, n);
  swapl(&pEncoding->rate.numerator, n);
  swapl(&pEncoding->rate.denominator, n);
  (void)WriteToClient(client, sz_xvEncodingInfo, (char *)pEncoding);
}

static int
SWriteFormat(client, pFormat)
register ClientPtr client;
xvFormat *pFormat;

{
  register char n;

  swapl(&pFormat->visual, n);
  (void)WriteToClient(client, sz_xvFormat, (char *)pFormat);
}


#ifndef _AIX 
static int
SWriteGetImageReply(client, rep)
    register ClientPtr  client;
    xvGetImageReply* 	rep;

{
  register char n;

  swaps(&rep->sequenceNumber, n);
  swapl(&rep->length, n);
  swapl(&rep->image_width, n);
  swapl(&rep->image_height, n);

  (void)WriteToClient(client, sz_xvGetImageReply, (char *)rep);

}
#endif /* !_AIX #### */

static int
SWriteGetLongPortAttributeReply(client, rep)
    register ClientPtr      client;
    xvGetLongPortAttributeReply *rep;

{
  register char n;

  swaps(&rep->sequenceNumber, n);
  swapl(&rep->length, n);

  (void)WriteToClient(client, sz_xvGetLongPortAttributeReply, (char *)rep);

}

static int
SWriteGetPortAttributeReply(client, rep)
register ClientPtr client;
xvGetPortAttributeReply *rep;

{
  register char n;

  swaps(&rep->sequenceNumber, n);
  swapl(&rep->length, n);
  swapl(&rep->value, n);

  (void)WriteToClient(client, sz_xvGetPortAttributeReply, (char *)rep);

}

static int
SWriteGrabPortReply(client, rep)
register ClientPtr client;
xvGrabPortReply *rep;

{
  register char n;

  swaps(&rep->sequenceNumber, n);
  swapl(&rep->length, n);

  (void)WriteToClient(client, sz_xvGrabPortReply, (char *)rep);

}

static int
SWriteQueryAdaptorsReply(client, rep)
register ClientPtr client;
xvQueryAdaptorsReply *rep;

{
  register char n;

  swaps(&rep->sequenceNumber, n);
  swapl(&rep->length, n);
  swaps(&rep->num_adaptors, n);
  
  (void)WriteToClient(client, sz_xvQueryAdaptorsReply, (char *)rep);

}

static int
SWriteQueryBestSizeReply(client, rep)
register ClientPtr client;
xvQueryBestSizeReply *rep;

{
  register char n;

  swaps(&rep->sequenceNumber, n);
  swapl(&rep->length, n);
  swaps(&rep->actual_width, n);
  swaps(&rep->actual_height, n);

  (void)WriteToClient(client, sz_xvQueryBestSizeReply, (char *)rep);
}

static int
SWriteQueryEncodingsReply(client, rep)
register ClientPtr client;
xvQueryEncodingsReply *rep;

{
  register char n;

  swaps(&rep->sequenceNumber, n);
  swapl(&rep->length, n);
  swaps(&rep->num_encodings, n);
  
  (void)WriteToClient(client, sz_xvQueryEncodingsReply, (char *)rep);

}

static int
SWriteQueryExtensionReply(client, rep)
register ClientPtr client;
xvQueryExtensionReply *rep;

{
  register char n;

  swaps(&rep->sequenceNumber, n);
  swapl(&rep->length, n);
  swaps(&rep->version, n);
  swaps(&rep->revision, n);
  
  (void)WriteToClient(client, sz_xvQueryExtensionReply, (char *)rep);

}

