static char sccsid[] = "@(#)75  1.3  src/UMS/usr/lpp/XV/server/dix/xvwrappers.c, Xv, ums212 3/22/95 13:42:02";
/*
 *   COMPONENT_NAME: xv
 *
 *   FUNCTIONS: none
 *
 *   ORIGINS: 16, 145, 27
 *
 *   This module contains IBM CONFIDENTIAL code.
 *   OBJECT CODE ONLY SOURCE MATERIALS
 *
 *   (C) COPYRIGHT International Business Machines Corp. 1995
 *   All Rights Reserved
 *   US Government Users Restricted Rights - Use, duplication or
 *   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

/***********************************************************
Copyright 1991 by Digital Equipment Corporation, Maynard, Massachusetts,
and the Massachusetts Institute of Technology, Cambridge, Massachusetts.

                        All Rights Reserved

Permission to use, copy, modify, and distribute this software and its 
documentation for any purpose and without fee is hereby granted, 
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in 
supporting documentation, and that the names of Digital or MIT not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.  

DIGITAL DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
DIGITAL BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.

******************************************************************/
/*
** File: 
**
**   xvwrappers.c --- Xv server extension wrappers for X functions.
**   
** Author: 
**
**   David Carver (Digital Workstation Engineering/Project Athena)
**   Andy Goodrich (Parallax Graphics)
**
** Revisions:
**
**   02 Dec 94 Goodrich
**     - initial coding (moving code from xvmain.c and ddx/mi/xvddx.c with
**       significant rewriting).
**    
 * $Log:	xvwrappers.c,v $
 * Revision 1.4  95/12/11  15:26:16  15:26:16  dist ()
 * Andy Goodrich: Merge of hedge changes into existing HP/AIX source.
 * 
 * Revision 1.3  95/11/03  12:29:41  12:29:41  dist ()
 * Andy Goodrich: Check in at start of port to Solaris 2.5.
 * 
 * Revision 1.2  95/05/25  17:01:22  17:01:22  dist ()
 * Andy Goodrich - Combined HP/AIX version for HP 9.1 code freeze.
 * 
*/

#include <stdio.h>

#include "X.h"
#include "Xproto.h"
#include "misc.h"
#include "os.h"
#include "scrnintstr.h"
#include "windowstr.h"
#include "pixmapstr.h"
#include "gc.h"
#include "extnsionst.h"
#include "dixstruct.h"
#include "gcstruct.h"
#include "resource.h"
#include "opaque.h"
#include "input.h"

#include "Xv.h"
#include "Xvproto.h"
#include "xvdix.h"

/*  XV_DIAG BIT MEANINGS:
 *
 *    0x01 - normal one-shot debugging
 *    0x20 - pixmap wrapper debugging
 *    0x40 - wrapper debugging
 *    0x80 - initialization debugging
 */
/* #define XV_DIAG 0x40      */
#include "xvdebug.h"


/* FORWARD REFERENCES: */

Bool    	   CreateResourceTypes();
Bool           XvCloseScreen();
static void    XvClipNotify();
static void    XvCopyWindow();
static int     XvQueryAdaptors();
static Bool    XvSaveScreen();
static Bool    XvUnrealizeWindow();
static void    XvWindowExposures();

extern XvScreenPtr XvScreens[MAXSCREENS];


/*--------------------------------------------------------------------------*
 |"XvScreenInit"
 |
 |  Xv device-independent screen initialization. Allocate screen private
 |  storage for an Xv screen record and wrap the screen functions Xv needs
 |  to intercept.
 |
 |    = Success on success, error code on failure.
 *--------------------------------------------------------------------------*/
int XvScreenInit(pScreen)
    ScreenPtr pScreen;
{

    XvScreenPtr pxvs;	/* Screen record filling in. */

	DEBUGF(0x80, (fd, "XvScreenInit: entry stack @ 0x%08x\n", &pxvs));

#ifdef NOTDEF
    if (XvScreenGeneration != serverGeneration)
    {
		DEBUGF(0x80, (fd, "XvScreenInit: before CreateResourceTypes\n"));
        if (!CreateResourceTypes())
        {
            ErrorF("XvScreenInit: Unable to allocate resource types\n");
            return BadAlloc;
        }
		DEBUGF(0x80, (fd, "XvScreenInit: after CreateResourceTypes\n"));
        XvScreenIndex = AllocateScreenPrivateIndex ();
		DEBUGF(0x80, (fd, "XvScreenInit: after AllocateScreenPrivateIndex\n"));
        if (XvScreenIndex < 0)
        {
            ErrorF("XvScreenInit: Unable to allocate screen private index\n");
            return BadAlloc;
        }
        XvScreenGeneration = serverGeneration; 
    }
#endif

    /* ALLOCATE SCREEN PRIVATE RECORD */
  
	DEBUGF(0x80, (fd, "XvScreenInit: before xalloc\n"));
    pxvs = (XvScreenPtr) xalloc (sizeof (XvScreenRec));
    if (!pxvs)
    {
        ErrorF("XvScreenInit: Unable to allocate screen private structure\n");
        return BadAlloc;
    }
  
	DEBUGF(0x80, (fd, "XvScreenInit: before setting up wrappers\n"));
    XvScreens[pScreen->myNum] = pxvs;

    pxvs->ClipNotify = pScreen->ClipNotify;
    pxvs->CloseScreen   = pScreen->CloseScreen;
    pxvs->CopyWindow = pScreen->CopyWindow;
    pxvs->DestroyPixmap = pScreen->DestroyPixmap;
    pxvs->DestroyWindow = pScreen->DestroyWindow;
    pxvs->SaveScreen = pScreen->SaveScreen;
    pxvs->UnrealizeWindow = pScreen->UnrealizeWindow;
    pxvs->WindowExposures = pScreen->WindowExposures;
  
    pScreen->ClipNotify = XvClipNotify;
    pScreen->CloseScreen   = XvCloseScreen;
    pScreen->CopyWindow = XvCopyWindow;
    pScreen->DestroyPixmap = XvDestroyPixmap;
    pScreen->DestroyWindow = XvDestroyWindow;
    pScreen->SaveScreen = XvSaveScreen;
    pScreen->UnrealizeWindow = XvUnrealizeWindow;
    pScreen->WindowExposures = XvWindowExposures; 

    pScreen->devPrivates[XvScreenIndex].ptr = (pointer)pxvs;

	DEBUGF(0x80, (fd, "XvScreenInit: exiting stack @ 0x%08x\n", &pxvs));
    return Success;
}


/*----------------------------------------------------------------------------*
 |"XvClipNotify"
 |
 |  Screen transfer vector wrapper routine to indicate a clipping change.
 *----------------------------------------------------------------------------*/static void
XvClipNotify(pWin, dx, dy)
    WindowPtr pWin;
    int dx,dy;
{

    int             iAdapt;     /* Entry in pxvScrn->pPadaptors accessing. */
    int             iPort;      /* Entry in pxvAdapt->pPorts  processing. */
    ScreenPtr       pScreen;    /* Screen containing window being exposed. */
    XvAdaptorPtr    pxvAdapt;   /* Adaptor currently checking ports of. */
    XvPortPtr       pxvPort;    /* Port currently checking for video restart. */    XvScreenPtr     pxvScrn;    /* Xv screen info for window being exposed. */

	DEBUGF(0x40, (fd, "XvClipNotify: entry stack @ 0x%08x\n", &iAdapt));

    pScreen = pWin->drawable.pScreen;
    pxvScrn=(XvScreenPtr)pWin->drawable.pScreen->devPrivates[XvScreenIndex].ptr;    if (!pxvScrn) return; /* THIS IS ONLY NEEDED IF WE ARE USING miClipNotify */


    /*  UNWRAP CLIP NOTIFICATION ROUTINE AND CALL IT (IF IT EXISTS),
     *  THEN RE-WRAP IT.
     */  
    pScreen->ClipNotify = pxvScrn->ClipNotify;
    if (pScreen->ClipNotify) (* pScreen->ClipNotify)(pWin, dx, dy);
    pScreen->ClipNotify = XvClipNotify;


    /*  LOOP THROUGH ADAPTORS ASSOCIATED WITH THIS SCREEN CALLING THE ADAPTOR
     *  LEVEL ClipNotify SUPPORT.
     */  
    for ( iAdapt = 0; iAdapt < pxvScrn->nAdaptors; iAdapt++ )
    {
        pxvAdapt = pxvScrn->pPadaptors[iAdapt];
        (* pxvAdapt->ddClipNotify)(pxvAdapt, pWin, dx, dy);
    }

	DEBUGF(0x40, (fd, "XvClipNotify: exit stack @ 0x%08x\n", &iAdapt));
}


/*--------------------------------------------------------------------------*
 |"XvCloseScreen"
 |
 |  Xv wrapper for the CloseScreen screen transfer vector routine.
 |
 |    = TRUE on success, FALSE on error
 *--------------------------------------------------------------------------*/
Bool XvCloseScreen(iScreen, pScreen)
    int       iScreen;
    ScreenPtr pScreen;
{
    int           	iAdapt;
    XvAdaptorPtr  	pxvAdap;
    XvScreenPtr 	pxvs;

	DEBUGF(0x40, (fd, "XvCloseScreen: entry stack @ 0x%08x\n", &iAdapt));

    pxvs = (XvScreenPtr) pScreen->devPrivates[XvScreenIndex].ptr;
  
    pScreen->ClipNotify = pxvs->ClipNotify;
    pScreen->CopyWindow = pxvs->CopyWindow;
    pScreen->CloseScreen   = pxvs->CloseScreen;
    pScreen->DestroyPixmap = pxvs->DestroyPixmap;
    pScreen->DestroyWindow = pxvs->DestroyWindow;
    pScreen->SaveScreen = pxvs->SaveScreen;
    pScreen->UnrealizeWindow = pxvs->UnrealizeWindow;
    pScreen->WindowExposures = pxvs->WindowExposures;

#ifndef ibm
    XvImageTerm();
#endif  /* !ibm */

    for ( iAdapt=0; iAdapt < pxvs->nAdaptors; iAdapt++ )
    {
        pxvAdap = pxvs->pPadaptors[iAdapt];
        if (pxvAdap->pFormats) xfree(pxvAdap->pFormats);
        (* pxvAdap->ddTerminate)(pxvAdap);
    }

    xfree(pxvs->pPadaptors);
    pxvs->pPadaptors = (XvAdaptorPtr*)NULL;

    xfree(pxvs);

    XvScreens[iScreen] = 0; 

    pScreen->devPrivates[XvScreenIndex].ptr = (pointer)NULL;

	DEBUGF(0x40, (fd, "XvCloseScreen: exit stack @ 0x%08x\n", &iAdapt));
    return (*pScreen->CloseScreen)(iScreen, pScreen);
}


/*----------------------------------------------------------------------------*
 |"XvCopyWindow"
 |
 |  Screen transfer vector wrapper routine to copy a window.
 *----------------------------------------------------------------------------*/static void
#ifdef __hpux
XvCopyWindow(pWin, lastposition, pRegionSrc, foo1, foo2)
    char *foo1, foo2;
#else /* ! __hpux */
XvCopyWindow(pWin, lastposition, pRegionSrc)
#endif
    WindowPtr pWin;
    DDXPointRec lastposition;
    RegionPtr pRegionSrc;
{
    int             iAdapt;     /* Entry in pxvScrn->pPadaptors accessing. */
    int             iPort;      /* Entry in pxvAdapt->pPorts processing. */
    WindowPtr       pChild;
    ScreenPtr       pScreen;    /* Screen containing window being exposed. */
    XvAdaptorPtr    pxvAdapt;   /* Adaptor currently checking ports of. */
    XvPortPtr       pxvPort;    /* Port currently checking for video restart. */    
	XvScreenPtr     pxvScrn;    /* Xv screen info for window being exposed. */

	DEBUGF(0x40, (fd, "XvCopyWindow: entry stack @ 0x%08x\n", &iAdapt));

    pScreen = pWin->drawable.pScreen;
    pxvScrn=(XvScreenPtr)pWin->drawable.pScreen->devPrivates[XvScreenIndex].ptr;


    /*  UNWRAP COPYWINDOW AND CALL IT SO THE REAL PROCESSING GETS DONE,
     *  THEN REWRAP IT.
     */  
    pScreen->CopyWindow = pxvScrn->CopyWindow;
#ifdef __hpux
    (* pScreen->CopyWindow)(pWin, lastposition, pRegionSrc, foo1, foo2);
#else  /* !__hpux */
    (* pScreen->CopyWindow)(pWin, lastposition, pRegionSrc);
#endif /* __hpux */
    pScreen->CopyWindow = XvCopyWindow;


    /*  LOOP THROUGH ADAPTORS ASSOCIATED WITH THIS SCREEN CALLING THE ADAPTOR
     *  LEVEL CopyWindow ROUTINE SO VIDEO MAY BE RESTARTED ON THE ADAPTOR'S
     *  PORTS WHERE NEEDED.
     */  
    for ( iAdapt = 0; iAdapt < pxvScrn->nAdaptors; iAdapt++ )
    {
        pxvAdapt = pxvScrn->pPadaptors[iAdapt];
        (* pxvAdapt->ddCopyWindow)(pxvAdapt, pWin, lastposition, pRegionSrc);
    }

	DEBUGF(0x40, (fd, "XvCopyWindow: exit stack @ 0x%08x\n", &iAdapt));
}


/*--------------------------------------------------------------------------*
 |"XvDestroyPixmap"
 |
 |  Xv wrapper for the DestroyPixmap screen transfer vector routine.
 |
 |    = TRUE on success, FALSE on error
 *--------------------------------------------------------------------------*/
Bool XvDestroyPixmap(pPix)
    PixmapPtr pPix;

{
	int			 iAdaptor;
    int          na;
    int          np;
    XvAdaptorPtr pa;
    XvPortPtr    pp;
    XvScreenPtr  pxvs;
    ScreenPtr    pScreen;
    Bool         status;

	DEBUGF(0x20, (fd, "XvDestroyPixmap: entry pixmap=0x%08x stack @ 0x%08x\n", 
		pPix, &iAdaptor));

    pScreen = pPix->drawable.pScreen;

    SCREEN_PROLOGUE(pScreen, DestroyPixmap);

    pxvs = (XvScreenPtr)pScreen->devPrivates[XvScreenIndex].ptr;
	status = (* pScreen->DestroyPixmap)(pPix); 

    /* CHECK TO SEE IF THIS PORT IS IN USE */

    na = pxvs->nAdaptors;
    for ( iAdaptor = 0; iAdaptor < na; iAdaptor++ )
    {
		pa = pxvs->pPadaptors[iAdaptor];
        np = pa->nPorts;
        pp = pa->pPorts;

        while (np--)
        {
            if (pp->pDraw == (DrawablePtr)pPix)
            {
                XvdiSendVideoNotify(pp, pp->pDraw, XvPreempted);

#ifndef _AIX
                (void)(* pp->pAdaptor->ddDetachPort)((ClientPtr)NULL, pp, 
                                                  pp->pDraw);
#else   /* _AIX #### */
			    (void)(* pp->pAdaptor->ddStopVideo)((ClientPtr)NULL, pp,
													pp->pDraw);
#endif /* _AIX #### */
                pp->pDraw = (DrawablePtr)NULL;
                pp->client = (ClientPtr)NULL;
                pp->time = currentTime;
            }
            pp++;
        }
    }

/* ####    status = (* pScreen->DestroyPixmap)(pPix); */

    SCREEN_EPILOGUE(pScreen, DestroyPixmap, XvDestroyPixmap);

	DEBUGF(0x20, (fd, "XvDestroyPixmap: exit stack @ 0x%08x\n", &iAdaptor));
    return status;

}


/*--------------------------------------------------------------------------*
 |"XvDestroyWindow"
 |
 |  Xv wrapper for the DestroyWindow screen transfer vector routine.
 |
 |    = TRUE on success, FALSE on error
 *--------------------------------------------------------------------------*/
Bool XvDestroyWindow(pWin)
  WindowPtr pWin;
{
	int			 iAdaptor;
    int          na;
    int          np;
    XvAdaptorPtr pa;
    XvPortPtr    pp;
    ScreenPtr    pScreen;
    XvScreenPtr  pxvs;
    Bool         status;

	DEBUGF(0x40, (fd, "XvDestroyWindow: entry stack @ 0x%08x\n", &iAdaptor));

    pScreen = pWin->drawable.pScreen;

    SCREEN_PROLOGUE(pScreen, DestroyWindow);

    pxvs = (XvScreenPtr)pScreen->devPrivates[XvScreenIndex].ptr;


    /* DETACH PORT(S) FROM THIS WINDOW. */

    na = pxvs->nAdaptors;
    for ( iAdaptor = 0; iAdaptor < na; iAdaptor++ )
    {
		pa = pxvs->pPadaptors[iAdaptor];
        np = pa->nPorts;
        pp = pa->pPorts;

        while (np--)
        {
            if (pp->pDraw == (DrawablePtr)pWin)
            {
                XvdiSendVideoNotify(pp, pp->pDraw, XvPreempted);

#ifndef _AIX
                (void)(* pp->pAdaptor->ddDetachPort)((ClientPtr)NULL, pp, 
                                                  pp->pDraw);
#else   /* _AIX #### */
			    (void)(* pp->pAdaptor->ddStopVideo)((ClientPtr)NULL, pp,
													pp->pDraw);
#endif /* _AIX #### */

                pp->pDraw = (DrawablePtr)NULL;
                pp->client = (ClientPtr)NULL;
                pp->time = currentTime;
            }
            pp++;
        }
    }
  
    status = (* pScreen->DestroyWindow)(pWin);

    SCREEN_EPILOGUE(pScreen, DestroyWindow, XvDestroyWindow);

	DEBUGF(0x40, (fd, "XvDestroyWindow: exit stack @ 0x%08x\n", &iAdaptor));
    return status;

}


/*----------------------------------------------------------------------------*
 |"XvSaveScreen"
 |
 |  Screen transfer vector wrapper routine to handle screen saver mode.
 *----------------------------------------------------------------------------*/
#ifdef _AIX
static Bool XvSaveScreen(pScreen, mode, phase)
    ScreenPtr pScreen;	/* -> screen the saver operation is occurring on. */
    int 	  mode;		/* =  mode of screen saver: 0 - off, 1 - on. */
    int       phase;
#else
static Bool XvSaveScreen(pScreen, mode)
    ScreenPtr pScreen;	/* -> screen the saver operation is occurring on. */
    int 	  mode;		/* =  mode of screen saver: 0 - off, 1 - on. */
#endif
{
    int			 iAdaptor;	/* # of adaptor now processing. */
    int			 nAdaptor;	/* # of adaptors supported for this screen. */
	XvAdaptorPtr pAdaptor;	/* Context for adaptor how processing. */
	XvScreenPtr  pxvs;		/* Xv context (private storage) for this screen. */
	int			 rc;		/* Return code from real Save Screen routine. */


	/*  LOOP THROUGH THE ADAPTORS CALLING THE SCREEN SAVE OPERATION 
     *  FOR THOSE WHICH HAVE ONE. 
     */
    pxvs = (XvScreenPtr)pScreen->devPrivates[XvScreenIndex].ptr;
	nAdaptor = pxvs->nAdaptors;
    for(iAdaptor = 0; iAdaptor < nAdaptor; ++iAdaptor)
	{
		pAdaptor = pxvs->pPadaptors[iAdaptor];
#ifdef _AIX
		(* pAdaptor->ddSaveScreen)(pAdaptor, mode, phase);
#else
		(* pAdaptor->ddSaveScreen)(pAdaptor, mode);
#endif
	}

    /*  UNWRAP SAVE SCREEN NOTIFICATION ROUTINE AND CALL IT (IF IT EXISTS),
     *  THEN RE-WRAP IT.
     */
    if ( pScreen->SaveScreen = pxvs->SaveScreen )
	{
#ifdef _AIX
	    rc = (*pScreen->SaveScreen)(pScreen, mode, phase);
#else
    	rc = (*pScreen->SaveScreen)(pScreen, mode);
#endif
    	pScreen->SaveScreen = XvSaveScreen;
	}
	return rc;
}



/*----------------------------------------------------------------------------*
 |"XvUnrealizeWindow"
 |
 |  Screen transfer vector wrapper routine to handle removal of a window
 |  from the screen.
 *----------------------------------------------------------------------------*/
static Bool XvUnrealizeWindow(pWin)
    WindowPtr pWin;
{
    int             iAdapt;     /* Entry in pxvScrn->pPadaptors accessing. */
    int             iPort;      /* Entry in pxvAdapt->pPorts  processing. */
    ScreenPtr       pScreen;    /* Screen containing window being exposed. */
    XvAdaptorPtr    pxvAdapt;   /* Adaptor currently checking ports of. */
    XvPortPtr       pxvPort;    /* Port currently checking for video restart. */    
	XvScreenPtr     pxvScrn;    /* Xv screen info for window being exposed. */
    int             status;     /* Return code from real UnrealizeWindow. */

	DEBUGF(0x40, (fd, "XvUnrealizeWindow: entry stack @ 0x%08x\n", &iAdapt));

    pScreen = pWin->drawable.pScreen;
    pxvScrn=(XvScreenPtr)pWin->drawable.pScreen->devPrivates[XvScreenIndex].ptr;    


    /*  UNWRAP UNREALIZE WINDOW ROUTINE AND CALL REAL ONE, THEN
     *  RE-WRAP IT.
     */  
    pScreen->UnrealizeWindow = pxvScrn->UnrealizeWindow;
    status = (* pScreen->UnrealizeWindow)(pWin);
    pScreen->UnrealizeWindow = XvUnrealizeWindow;


    /*  LOOP THROUGH ADAPTORS ASSOCIATED WITH THIS SCREEN CALLING THE 
     *  ADAPTOR LEVEL UnRealizeWindow ROUTINE SO THAT WE CAN HALT VIDEO
     *  ON ANY OF THE ADAPTOR'S PORTS WHICH CORRESPOND TO OUR WINDOW.
     */  
	for ( iAdapt = 0; iAdapt < pxvScrn->nAdaptors; iAdapt++ )
    {
        pxvAdapt = pxvScrn->pPadaptors[iAdapt];
        (* pxvAdapt->ddUnrealizeWindow)(pxvAdapt, pWin);
    }

	DEBUGF(0x40, (fd, "XvUnrealizeWindow: exit stack @ 0x%08x\n", &iAdapt));
    return status;

}


/*----------------------------------------------------------------------------*
 |"XvWindowExposures"
 |
 |  Screen transfer vector wrapper routine to handle window exposures.
 |  For all windows temporarily 'un-wrap' and call the real exposure routine.
 |  If the window being exposed is the one in which video is playing restart
 |  video after setting its new geometry. 
 *----------------------------------------------------------------------------*/
static void

#if defined(__hpux)
XvWindowExposures(pWin, pReg, pOtherReg, foo1)
    WindowPtr pWin;
    RegionPtr pReg;
    RegionPtr pOtherReg;
    char *foo1;
#else
XvWindowExposures(pWin, pReg, pOtherReg)
    WindowPtr pWin;
    RegionPtr pReg;
    RegionPtr pOtherReg;
#endif /* defined(__hpux) */
{
    int             iAdapt;     /* Entry in pxvScrn->pPadaptors accessing. */
    int             iPort;      /* Entry in pxvAdapt->pPorts  processing. */
    ScreenPtr       pScreen;    /* Screen containing window being exposed. */
    XvAdaptorPtr    pxvAdapt;   /* Adaptor currently checking ports of. */
    XvPortPtr       pxvPort;    /* Port currently checking for video restart. */
	XvScreenPtr     pxvScrn;    /* Xv screen info for window being exposed. */

	DEBUGF(0x40, (fd, "XvWindowExposures: entry win = 0x%08x, stack @ 0x%08x\n",
		pWin, &iAdapt));

    pScreen = pWin->drawable.pScreen;
    pxvScrn=(XvScreenPtr)pWin->drawable.pScreen->devPrivates[XvScreenIndex].ptr;


    /*  UNWRAP WINDOW EXPOSURE ROUTINE AND CALL IT SO NORMAL PROCESSING
     *  GETS DONE. THEN RE-WRAP IT.
     */  
    pScreen->WindowExposures = pxvScrn->WindowExposures;
#if defined(__hpux)
    (* pScreen->WindowExposures)(pWin, pReg, pOtherReg, foo1);
#else
    (* pScreen->WindowExposures)(pWin, pReg, pOtherReg);
#endif  /* defined(__hpux) */
	pxvScrn->WindowExposures = pScreen->WindowExposures;
    pScreen->WindowExposures = XvWindowExposures; 


    /*  LOOP THROUGH ADAPTORS ASSOCIATED WITH THIS SCREEN CALLING THEIR
     *  WindowExposures ROUTINE SO THAT WE CAN RESTART VIDEO ON THE 
     *  ADAPTOR'S PORTS WHERE NEEDED.
     */  
	for ( iAdapt = 0; iAdapt < pxvScrn->nAdaptors; iAdapt++ )
    {
        pxvAdapt = pxvScrn->pPadaptors[iAdapt];
        (* pxvAdapt->ddWindowExposures)(pxvAdapt, pWin, pReg, pOtherReg);
    }


	/*  RE-WRAP WINDOW EXPOSURE NOTIFICATION TRANSFER VECTOR ENTRY. */

	DEBUGF(0x40, (fd, "XvWindowExposures: exit stack @ 0x%08x\n", &iAdapt));
}
