/*
 * All rights reserved.
 */

#ident	"@(#)tvfour.c	1.19	97/08/07 PLX"




#include <sys/debug.h>
#include <sys/types.h>
#include <sys/buf.h>
#include <sys/errno.h>
#include <sys/conf.h>
#include <sys/file.h>
#include <sys/uio.h>
#include <sys/map.h>
#include <sys/cred.h>
#include <sys/open.h>
#include <sys/stat.h>
#include <sys/kmem.h>
#include <sys/cmn_err.h>
#include <sys/kmem.h>
#include <sys/conf.h>
#include <sys/ddi.h>
#include <sys/devops.h>
#include <sys/sunddi.h>
#include <sys/modctl.h>

#include <sys/visual_io.h>
#ifdef _DDICT
#define	NBBY 8
#endif

#include <sys/fbio.h>
#include "tvfour.h"

#define PV_DISP_ONOFF 0x5005
#define PV_RES_SET    0x5006
#define BRD_RESET     0x5001
#define MODEBITSA 0x101
#define MODEBITSB 0x102
#define PLL_CNTRL 0x103


int	pvip_debug = 1;


/* configuration options */




/* Data access requirements. */
#ifdef _LITTLE_ENDIAN
static struct ddi_device_acc_attr endian_attr = {
	DDI_DEVICE_ATTR_V0,
	DDI_STRUCTURE_LE_ACC,
	DDI_STRICTORDER_ACC
};
#else /* _LITTLE_ENDIAN */
static struct ddi_device_acc_attr endian_attr = {
	DDI_DEVICE_ATTR_V0,
	DDI_STRUCTURE_BE_ACC,
	DDI_STRICTORDER_ACC
};
#endif /* _LITTLE_ENDIAN */
/*
 #define NO_SWAP
*/

#ifdef NO_SWAP
static struct ddi_device_acc_attr endian_attr = {
	DDI_DEVICE_ATTR_V0,
	DDI_NEVERSWAP_ACC,
	DDI_STRICTORDER_ACC
};
#endif


#define	getprop(devi, name, def)	\
		ddi_getprop(DDI_DEV_T_ANY, (devi), \
		DDI_PROP_DONTPASS, (name), (def))

/* config info */

static int	tvfour_open(dev_t *, int, int, cred_t *);
static int	tvfour_close(dev_t, int, int, cred_t *);
static int	tvfour_ioctl(dev_t, int, int, int, cred_t *, int *);
static int	tvfour_mmap(dev_t, off_t, int);
static int      tvfour_read(dev_t , struct uio *, cred_t *);
static int      tvfour_write(dev_t , struct uio *, cred_t *);
static int      tvfour_ccube_strategy(struct buf *);
static void	tvfour_minphys(struct buf *);

static struct cb_ops tvfour_cb_ops = {
	tvfour_open,		/* open */
	tvfour_close,		/* close */
	tvfour_ccube_strategy,	/* strategy */
	nodev,			/* print */
	nodev,			/* dump */
	tvfour_read,		/* read */
	tvfour_write,		/* write */
	tvfour_ioctl,		/* ioctl */
	nodev,			/* devmap */
	tvfour_mmap,		/* mmap */
	ddi_segmap,		/* segmap */
	nochpoll,		/* poll */
	ddi_prop_op,		/* cb_prop_op */
	0,			/* streamtab */
	D_NEW | D_MP		/* Driver compatibility flag */
};

static int	tvfour_info(dev_info_t *, ddi_info_cmd_t, void *, void **);
static int	tvfour_attach(dev_info_t *, ddi_attach_cmd_t);
static int	tvfour_detach(dev_info_t *, ddi_detach_cmd_t);
static int	tvfour_power(dev_info_t *, int, int);

static struct dev_ops tvfour_ops = {
	DEVO_REV,		/* devo_rev, */
	0,			/* refcnt */
	tvfour_info,		/* info */
	nulldev,		/* identify */
	nulldev,		/* probe */
	tvfour_attach,		/* attach */
	tvfour_detach,		/* detach */
	nodev,			/* reset */
	&tvfour_cb_ops,		/* driver operations */
	(struct bus_ops *) 0,	/* bus operations */
	tvfour_power,		/* power operations */
};

#define	PVIP_CMAP_ENTRIES	256
#define	PVIP_CURSOR_ENTRIES	2

/*
 * Per-context info:
 *	many registers in the tec and fbc do
 *	not need to be saved/restored.
 */


#define	PVIP_VRTIOCTL	1		/* FBIOVERTICAL in effect */
#define	PVIP_VRTCTR	2		/* OWGX vertical retrace counter */


static u_int	tvfour_pagesize;



/* default structure for FBIOGATTR ioctl */
static struct fbgattr	tvfour_attr;

/*
 * handy macros
 */
static void *state_p;	/* opaque handle where all the state hangs */

#define	getsoftc(instance) \
	((tvfour_softc *)ddi_get_soft_state(state_p, (instance)))

#define	btob(n)			ctob(btoc(n))

#define	PVIP_WAIT		500000	/* .5 seconds */



extern u_int tvfour_intr(caddr_t i_handler_arg);

/*
 *
 */

static XV_regmaps XV_promrev1 = {
	{ XVREGNUM, XVBASE, XVBASE, XVSIZE, XVVOFF },
	{ XVREGNUM, C3BASE, C3BASE, C3SIZE, C3VOFF },
	{ XVREGNUM, M1BASE, M1BASE, M1SIZE, M1VOFF },
	{ XVREGNUM, M8BASE, M8BASE, M8SIZE, M8VOFF },
	{ XVREGNUM, M24BASE, M24BASE, M24SIZE, M24VOFF },
	{ XVREGNUM, PROMBASE, PROMBASE, PROMSIZE, PROMVOFF }
};

/* Loadable Driver stuff */

extern struct mod_ops	mod_driverops;

static struct modldrv	tvfour_modldrv = {
	&mod_driverops,		/* Type of module.This one is a driver */
	"sal-xtra pci driver v97/07/21",	/* Name of the module. */
	&tvfour_ops,			/* driver ops */
};

static struct modlinkage	tvfour_modlinkage = {
	MODREV_1,
	(void *) &tvfour_modldrv,
	NULL,
};


int
_init(void)
{
	register int	e;

	DEBUGF(1, (CE_CONT, "! tvfour: compiled %s, %s\n", __TIME__, __DATE__));

	if ((e = ddi_soft_state_init(&state_p,
		sizeof (tvfour_softc), 1)) != 0) {
		DEBUGF(1, (CE_CONT, "! done\n"));
		return (e);
	}

	e = mod_install(&tvfour_modlinkage);

	if (e) {
		ddi_soft_state_fini(&state_p);
		DEBUGF(1, (CE_CONT, "! done\n"));
	}

	DEBUGF(1, (CE_CONT, "! tvfour: _init done rtn=%d\n", e));

	return (e);
}

int _fini(void)
{
	register int	e;

	if ((e = mod_remove(&tvfour_modlinkage)) != 0)
		return (e);

	ddi_soft_state_fini(&state_p);

	return (0);
}

int _info(struct modinfo *modinfop)
{
	return (mod_info(&tvfour_modlinkage, modinfop));
}


#define PVSAL_PCI_CONF_ID  0
#define PVSAL_PCI_CONF_CMD 4
#define PVSAL_PCI_CONF_STATUS 6
#define PVSAL_PCI_CONF_CLASS 8
#define PVSAL_PCI_CONF_BASE 0x10
#define PVSAL_PCI_CONF_LEVEL 0x3d
#define PVSAL_PCI_CONF_INT 0x3c


static int
tvfour_attach(register dev_info_t *devi,
	register ddi_attach_cmd_t cmd)
{
	register tvfour_softc	*softc;
	register int instance;

	char		name[16];
	uint32_t	timestamp[2];
	int		power[2];
	int		level;
	int		vector;
	int		class;
	int		proplen, i;
	int		vendor_id,device_id,status,revision;
	int 		base_addr;
	int		cm;
	u_int		*prom;
	u_int		*TmrInt;
	u_int		*TmrIntRB;
	XV_regs 	*regs;



	switch (cmd) 
	{
	case DDI_ATTACH:


	/*
	tvfour_pagesize = ddi_ptob(devi, 1);
	 * Work out which instance we are, and remember
	 */
	instance = ddi_get_instance(devi);

	/* Allocate softc struct */
	if (ddi_soft_state_zalloc(state_p, instance))
	{
		printf("ddi_soft_state_zalloc failed\n");
		return (DDI_FAILURE);
	}

	softc = getsoftc(instance);

	/* link it in */
	softc->devi = devi;

	ddi_set_driver_private(devi, (caddr_t) softc);

	/* map in the configuration registers */
	if (pci_config_setup(devi, &softc->tvfour_conf_handle)) {
		DEBUGF(2, (CE_CONT,
			"tvfour_attach: ddi_conf_regs (pvip) FAILED\n"));
		return (DDI_FAILURE);
	}
#define CONFIG_DIAGS
#ifdef CONFIG_DIAGS
	vendor_id = pci_config_getw(softc->tvfour_conf_handle, PVSAL_PCI_CONF_ID);
	device_id = pci_config_getw(softc->tvfour_conf_handle, PVSAL_PCI_CONF_ID+2);
	status=pci_config_getw(softc->tvfour_conf_handle, PVSAL_PCI_CONF_STATUS);
	base_addr = pci_config_getl(softc->tvfour_conf_handle,PVSAL_PCI_CONF_BASE);
	level= pci_config_getb(softc->tvfour_conf_handle,PVSAL_PCI_CONF_LEVEL);
	class= pci_config_getl(softc->tvfour_conf_handle,PVSAL_PCI_CONF_CLASS);
	DEBUGF(1,(CE_CONT,"! tvfour_attach vendor_id=0x%x device_id=0x%x status=0x%x base_addr=0x%x level=0x%x class=0x%x\n",
	     vendor_id,device_id,status,base_addr,level,class));
        level=getprop(devi, "interrupt-priorities", 11);
	cmn_err(CE_NOTE,"!value returned from interrupt-priorities=%d\n",
		level);
#endif

	softc->regmap= &XV_promrev1;

	softc->regs= kmem_zalloc(sizeof(XV_regs),KM_SLEEP);
	if(softc->regs == NULL) {
		printf("regs kmem_zalloc failed\n");
		ddi_soft_state_free(state_p,instance);
		return(DDI_FAILURE);
	}
	if (ddi_regs_map_setup(devi, 1, (caddr_t *)&prom,
			softc->regmap->prom.offset, 
			softc->regmap->prom.size,
			&endian_attr, &softc->acc_handle)) {
			return (DDI_FAILURE);
	}
	*prom = 0;
	drv_usecwait(10000);
	*prom = 0xffffffff;
	softc->dmactl= (DMA_CTL *)prom;
	softc->miscmode= (MISC_REGS *)((char *)prom+0x10);

	if (ddi_regs_map_setup(devi, 1, (caddr_t *)&softc->regs,
			softc->regmap->xv.offset,
			softc->regmap->xv.size,
			&endian_attr, &softc->xvreg_hand)) {
			return (DDI_FAILURE);
	}
	softc->xvregs= (XV_hw_regs *)softc->regs;

	PCIV_WREG(softc,IntMask.l,0);
	PCIV_WREG(softc,BIntMask.l,0);

	if (ddi_regs_map_setup(devi, 1, (caddr_t *)&softc->c3regs,
			softc->regmap->c3.offset, 
			softc->regmap->c3.size,
			&endian_attr, &softc->c3reg_hand)) {
			return (DDI_FAILURE);
	}
#if 0
	if (ddi_regs_map_setup(devi, 1, (caddr_t *)&softc->m1addr,
			softc->regmap->m1.offset, 
			softc->regmap->m1.size,
			&endian_attr, &softc->fb_hand[0])) {
			return (DDI_FAILURE);
	}
	if (ddi_regs_map_setup(devi, 1, (caddr_t *)&softc->m8addr,
			softc->regmap->m8.offset, 
			softc->regmap->m8.size,
			&endian_attr, &softc->fb_hand[1])) {
			return (DDI_FAILURE);
	}
	if (ddi_regs_map_setup(devi, 1, (caddr_t *)&softc->m24addr,
			softc->regmap->m24.offset, 
			softc->regmap->m24.size,
			&endian_attr, &softc->fb_hand[2])) {
			return (DDI_FAILURE);
	}
#endif
	mutex_init(&softc->interlock, "tvfour_interlock",
				MUTEX_DRIVER, NULL);
	mutex_init(&softc->mutex, "tvfour_interlock",
				MUTEX_DRIVER, NULL);
	mutex_init(&softc->dma_lock, "tvfour_dma",
				MUTEX_DRIVER, softc->iblkc);

	cv_init(&softc->dmadone, "tvfour_cv",CV_DRIVER, NULL);

	ddi_remove_intr(devi, 0 ,softc->iblkc);
	softc->ilevel=0;
	if(ddi_add_intr(devi, (u_int) softc->ilevel, &softc->iblkc,
		&softc->idevc, tvfour_intr, (caddr_t) softc)== DDI_FAILURE)
		return(DDI_FAILURE);
	{
	    char minor[16];
            sprintf(minor,"tvfour%d",instance);
	    if(ddi_create_minor_node(devi,minor,S_IFCHR,
		instance,NULL,0) == DDI_FAILURE)
		cmn_err(CE_WARN,"create minor device failed\n");
	}
#if 0
	pci_config_teardown(&softc->tvfour_conf_handle);
#endif
	ddi_report_dev(devi);
	}
	return (DDI_SUCCESS);
}


static int
tvfour_detach(register dev_info_t *devi,
	register ddi_detach_cmd_t cmd)
{
	register int instance = ddi_get_instance(devi);
	register tvfour_softc *softc = TVTWO_DEVI2SOFTC(devi);
	int i;


	switch (cmd) {
	case DDI_DETACH:
		mutex_destroy(&softc->interlock);
		mutex_destroy(&softc->mutex);
		mutex_destroy(&softc->dma_lock);
		cv_destroy(&softc->dmadone);
		ddi_remove_intr(devi, 0, &softc->iblkc);
		ddi_remove_minor_node(devi, (char *) NULL);
		ddi_soft_state_free(state_p, instance);
		return (DDI_SUCCESS);
		break;

	case DDI_SUSPEND:
		if (softc == NULL)
			return (DDI_FAILURE);
		return (DDI_SUCCESS);

	default:
		return (DDI_FAILURE);
	}


}


static int
tvfour_power(dev_info_t *dip, int cmpt, int level)
{
	int		power[2];
	register struct tvfour_softc	*softc;

	(void) ddi_power(dip, cmpt, level);

	return (DDI_SUCCESS);
}


/* ARGSUSED */
static int
tvfour_info(dev_info_t *dip, ddi_info_cmd_t infocmd, void *arg, void **result)
{
	register int	instance;
	register int	error = DDI_SUCCESS;
	register tvfour_softc	*softc;

	instance = getminor((dev_t)arg);
	switch (infocmd) {
	case DDI_INFO_DEVT2DEVINFO:
		softc = getsoftc(instance);
		if (softc == NULL) {
			error = DDI_FAILURE;
		} else {
			*result = (void *)softc->devi;
			error = DDI_SUCCESS;
		}
		break;
	case DDI_INFO_DEVT2INSTANCE:
		*result = (void *)instance;
		error = DDI_SUCCESS;
		break;
	default:
		error = DDI_FAILURE;
	}

	return (error);
}

/*ARGSUSED*/
static
tvfour_open(register dev_t *devp, register int flag,
	register int otyp, register cred_t *cred)
{
	register int instance = getminor(*devp);
	register tvfour_softc	*softc = getsoftc(instance);
	int	error = 0;

	DEBUGF(2, (CE_CONT, "! tvfour_open(%d)\n", instance));

	if (otyp != OTYP_CHR) {
		error = EINVAL;
	} else if (softc == NULL) {
		error = ENXIO;
	}
	return (error);
}


/*ARGSUSED*/
static
tvfour_close(register dev_t dev, register int flag,
	register int otyp, register cred_t *cred)
{
	register int instance = getminor(dev);
	register tvfour_softc	*softc = getsoftc(instance);
	int	error = 0;

	DEBUGF(2, (CE_CONT, "! tvfour_close(%d, %d, %d)\n", instance, flag, otyp));

	if (otyp != OTYP_CHR) {
		error = EINVAL;
	} else if (softc == NULL) {
		error = ENXIO;
	} 

	return (error);
}


#define TVTWO_IN_RANGE(rm, off) \
		(((off) >= (rm).voff) && ((off) < ((rm).voff + (rm).size)))

/* ARGSUSED */
static int tvfour_mmap(dev_t dev, off_t off, int prot)
{
	register tvfour_softc *softc = TVTWO_DEV2SOFTC(dev);
	register XV_regmaps *rm;
	caddr_t addr;
	u_int pfn;

	if (softc == NULL) {
		return (-1);
	}

	rm = softc->regmap;

	/*
	 * Check the offset against all available ranges and
	 * return the appropriate page frame number.
	 *
	 */

#ifdef KLUDGE_PFN
	if (TVTWO_IN_RANGE(rm->xv, off))
		return tvfour_pfn(softc, &rm->xv, off,softc->regs);
	if (TVTWO_IN_RANGE(rm->c3, off))
		return tvfour_pfn(softc, &rm->c3, off,softc->c3regs);
	if (TVTWO_IN_RANGE(rm->m1, off))
		return tvfour_pfn(softc, &rm->m1, off,softc->m1addr);
	if (TVTWO_IN_RANGE(rm->m8, off))
		return tvfour_pfn(softc, &rm->m8, off,softc->m8addr);
	if (TVTWO_IN_RANGE(rm->m24, off))
		return tvfour_pfn(softc, &rm->m24,off,softc->m24addr);
#else
	if (TVTWO_IN_RANGE(rm->xv, off))
		return tvfour_pfn(softc, &rm->xv, off,softc->xvreg_hand);
	if (TVTWO_IN_RANGE(rm->c3, off))
		return tvfour_pfn(softc, &rm->c3, off,softc->c3reg_hand);
	if (TVTWO_IN_RANGE(rm->m1, off))
		return tvfour_pfn(softc, &rm->m1, off,softc->fb_hand[0]);
	if (TVTWO_IN_RANGE(rm->m8, off))
		return tvfour_pfn(softc, &rm->m8, off,softc->fb_hand[1]);
	if (TVTWO_IN_RANGE(rm->m24, off))
		return tvfour_pfn(softc, &rm->m24,off,softc->fb_hand[2]);
#endif

	return (-1);
}

#ifdef KLUDGE_PFN
static int
tvfour_pfn(tvfour_softc *softc, XV_regmap *rm, off_t off,caddr_t base_addr)
{
	caddr_t addr;
	u_int pfn;
	int diff;

	/*
	 * We need to adjust our passed in offset from the base of
	 * the "virtual" register window and add in our offset from
	 * the base of the real register.
	 */

#if 0
	off = (off - rm->voff) + rm->offset;
	if (ddi_map_regs(softc->devi, rm->regnum+1,
		&addr, off, rm->size) != DDI_FAILURE) {

		pfn = hat_getkpfnum(addr);
		off,addr,pfn);

		ddi_unmap_regs(softc->devi, rm->regnum+1,
			&addr, off, rm->size);
	}
#endif
	diff = off - rm->voff;
	cmn_err(CE_WARN," rm->voff=0x%x base=0x%x off=0x%x diff=0x%x\n",rm->offset, base_addr,off,diff);

	pfn = hat_getkpfnum(base_addr+diff);

	return (pfn);
}
#else
static int
tvfour_pfn(tvfour_softc *softc, XV_regmap *rm, off_t off, 
	ddi_acc_handle_t handle)
{
	caddr_t addr;
	u_int pfn;

	/*
	 * We need to adjust our passed in offset from the base of
	 * the "virtual" register window and add in our offset from
	 * the base of the real register.
	 */
	off = (off - rm->voff) + rm->offset;
	if (ddi_regs_map_setup(softc->devi, rm->regnum+1, &addr, off, 4,
			&endian_attr, &handle) != DDI_FAILURE)
	{
		pfn = hat_getkpfnum(addr);
		ddi_regs_map_free(&handle);
		return (pfn);
	}
	return (-1);

}
#endif


/*ARGSUSED*/
static
tvfour_ioctl(dev_t dev, int cmd, int arg, int mode, cred_t *cred, int *rval)
{
	register tvfour_softc *softc = getsoftc(getminor(dev));
	register int cursor_cmap;
	int	 i;

	u_char	*iobuf_cmap_red;
	u_char	*iobuf_cmap_green;
	u_char	*iobuf_cmap_blue;
	u_char	*stack_cmap;
	u_int   bits;

	struct fbcmap	*cmap, cma;
	struct fbcursor	cp;

	u_int	index;
	u_int	count;
	int	enable;
	int error;
	int mask;
	register int cm;

#if 0
	DEBUGF(1, (CE_CONT, "! tvfour_ioctl(%d, 0x%x)\n", getminor(dev), cmd));
#endif
	mutex_enter(&softc->mutex);
	switch (cmd) 
	{
	case BRD_RESET:
	     softc->dmactl->master=0;
	     drv_usecwait(1000);
	     softc->dmactl->master=0xffffffff;
	     break;
	case PV_DISP_ONOFF:
	        if(ddi_copyin((caddr_t)arg,(caddr_t)&enable,sizeof(int),0) == -1)
		{
		     cmn_err(CE_WARN,"!DISP_ONOFF: copyin failed\n");
		     break;
		}
		switch(enable)
		{
		case 1:
			mask= PCIV_RSDW(softc,IntMask.l) | 1;
			PCIV_WREG(softc,IntMask.l,mask);
	     		cm = softc->xvregs->r.IntStatC.l;
			break;
		 case 0:
			mask = PCIV_RSDW(softc, IntMask.l) & ~1;
			PCIV_WREG(softc,IntMask.l,mask);
	     		cm = softc->xvregs->r.IntStatC.l;
			break;
		}
		break;
	case PV_RES_SET:
		{
		     int	ysize;
		     if(ddi_copyin((caddr_t)arg,(caddr_t)&ysize,sizeof(int),0) == -1)
		     {
		          cmn_err(CE_WARN,"!DISP_ONOFF: copyin failed\n");
		          break;
		     }
		     softc->nvram.main.ysize = ysize;
		     softc->nvram.main.v.disp = ysize-1;
		}
		break;
	case MODEBITSA:
	     {
		int value;
	        ddi_copyin((caddr_t)arg,(caddr_t)&value,sizeof(int),0);
#ifdef TRACE_IOCTLS
		cmn_err(CE_WARN,"!ModeBitsA: value=0x%x\n",value);
#endif
	        softc->miscmode->modeBitsA= BYTE_REV(value);

	     }
	     break;
	case MODEBITSB:
	     {
		int value;
	        ddi_copyin((caddr_t)arg,(caddr_t)&value,sizeof(int),0);
#ifdef TRACE_IOCTLS
		cmn_err(CE_WARN,"!ModeBitsB: value=0x%x\n",value);
#endif
	        softc->miscmode->modeBitsB= BYTE_REV(value);
	     }
	     break;
	case PLL_CNTRL:
	     {
		int value;
	        ddi_copyin((caddr_t)arg,(caddr_t)&value,sizeof(int),0);
#ifdef TRACE_IOCTLS
		cmn_err(CE_WARN,"!PLL_CNTRL: value=0x%x\n",value);
#endif
	        softc->miscmode->pll_cntl= BYTE_REV(value);
	     }
	     break;
	case VIS_GETIDENTIFIER:
	     break;
	default:
	     break;
	}/* switch(cmd) */
	mutex_exit(&softc->mutex);
	return (0);
}

/* ARGSUSED */
static int
tvfour_write(dev_t dev, struct uio *uio, cred_t *credp)
{
	return(physio(tvfour_ccube_strategy, NULL,
		dev, B_WRITE, tvfour_minphys, uio));
}

/* ARGSUSED */
static int
tvfour_read(dev_t dev, struct uio *uio, cred_t *credp)
{
	return(physio(tvfour_ccube_strategy, NULL,
		dev, B_READ, tvfour_minphys, uio));
}

/*
 * tvtwo_ccube_strategy is really only for access to the CCube codec
 * engine.  There is no block device access for this hardware
 * so no strategy routine is not listed in the cb_ops structure above.
 */

static int
tvfour_ccube_strategy(struct buf *bp)
{
	register tvfour_softc *softc = TVTWO_DEV2SOFTC(bp->b_edev);
	ddi_dma_cookie_t dma_cookie;    /* for dma information */
	caddr_t dma_addr;
	unsigned int imask;

	if (softc == NULL) {
		bp->b_flags |= B_ERROR;
		bp->b_error = ENXIO;
		iodone(bp);
		return (0);
	}

	imask = PCIV_RSDW(softc, IntMask.l);


	c3_load_dct(softc); /* XXX ??? XXX */


	if (ddi_dma_buf_setup(softc->devi, bp,
		(bp->b_flags & B_WRITE) ? DDI_DMA_WRITE : DDI_DMA_READ,
		DDI_DMA_DONTWAIT, NULL, NULL, &softc->dma) != DDI_DMA_MAPPED) {
			bp->b_flags |= B_ERROR;
			bp->b_error = EIO;
			iodone(bp);
			return (0);
	}

	/*
	 * We need to convert the dma_handle to a dma_cookie so we can get
	 * the DMA address for the TVTWO device to write to.
	 */
	if (ddi_dma_htoc(softc->dma, 0, &dma_cookie) != 0) {
		bp->b_flags |= B_ERROR;
		bp->b_error = EIO;
		ddi_dma_free(softc->dma);
		iodone(bp);
		return (0);
	}

	/*
	 * Save the buf pointer for our interrupt routine to update
	 * when the transfer is complete (or fails).
	 */
	softc->bp = bp;

        softc->dmactl->control0= DMACLEAR;
        softc->dmactl->control1= DMACLEAR;

	if(bp->b_flags & B_READ)
	{
	    c3_purge_fifo(softc);
	}

        softc->dmactl->control1= DMACLEAR;
        softc->dmactl->control1= DMALOADADDR;
	/*
	 * Setup DMA registers address.
	 */
	dma_addr= (caddr_t)dma_cookie.dmac_address;
	softc->addr = 0;
	/*
	 * Setup DMA registers address.
	 */

	pci_load_dma_address(softc,dma_addr,bp->b_bcount);

        softc->dmactl->control1= DMACLEAR;
     
	mutex_enter(&softc->dma_lock);
	/*
	 * Enable interrupts and start DMA.
	 */
	if (c3_start_dma(softc, softc->addr, bp) != DDI_SUCCESS) {
		bp->b_flags |= B_ERROR;
		bp->b_error = EIO;
		ddi_dma_free(softc->dma);
		iodone(bp);
		mutex_exit(&softc->dma_lock);
		return (0);
	}
	cv_wait(&softc->dmadone, &softc->dma_lock);
	/*
	 * Shutdown board dma.
	 */
        softc->dmactl->control0= DMACLEAR;
        softc->dmactl->control1= DMACLEAR;
	if(softc->state!=0)
	{
	    bp->b_flags |= B_ERROR;
	    bp->b_error = EIO;
	}
	/*
	 * Free up resources and wakeup any waiters.
	 */
	ddi_dma_free(softc->dma);
	iodone(bp);
	PCIV_WREG(softc,IntMask.l,imask);
	mutex_exit(&softc->dma_lock);
	return (DDI_SUCCESS);
}

static void
tvfour_minphys(struct buf *bp)
{
	register tvfour_softc *softc = TVTWO_DEV2SOFTC(bp->b_edev);

	if (softc->maxphys && (bp->b_bcount > softc->maxphys))
		bp->b_bcount = softc->maxphys;
}
pci_load_dma_address(tvfour_softc *softc,caddr_t start_addr,int buflen)
{
unsigned long cur_addr,beg_addr,end_addr;
int  len;

     cur_addr= beg_addr= (unsigned long )start_addr;
     end_addr= (unsigned long)(start_addr+ buflen);

     softc->dmactl->start_address= (unsigned long)BYTE_REV(beg_addr);
     len= 4096 - (beg_addr & 4095);
     cur_addr += len;

     while(cur_addr<end_addr)
     {
         softc->dmactl->start_address= (unsigned long)BYTE_REV(cur_addr);
	 cur_addr+= 4096;
     }
     softc->dmactl->start_address= (unsigned long)BYTE_REV(beg_addr);
}
