/*
 * Copyright (c) 1993,1997, Sun Microsystems, Inc.
 * All rights reserved.
 */

#ident	"@(#)pvip.c	1.19	97/04/07 SMI"




#include <sys/debug.h>
#include <sys/types.h>
#include <sys/buf.h>
#include <sys/errno.h>
#include <sys/conf.h>
#include <sys/file.h>
#include <sys/uio.h>
#include <sys/map.h>
#include <sys/cred.h>
#include <sys/open.h>
#include <sys/stat.h>
#include <sys/kmem.h>
#include <sys/cmn_err.h>
#include <sys/kmem.h>
#include <sys/conf.h>
#include <sys/ddi.h>
#include <sys/devops.h>
#include <sys/sunddi.h>
#include <sys/modctl.h>

#include <sys/visual_io.h>
#ifdef _DDICT
#define	NBBY 8
#endif

#include <sys/fbio.h>
#include "tvfour.h"


int	pvip_debug = 1;

#define	DEBUGF(level, args) \
	{ if (pvip_debug == (level)) cmn_err args; }

/* configuration options */



/* Data access requirements. */
#ifdef _LITTLE_ENDIAN
static struct ddi_device_acc_attr endian_attr = {
	DDI_DEVICE_ATTR_V0,
	DDI_STRUCTURE_LE_ACC,
	DDI_STRICTORDER_ACC
};
#else /* _LITTLE_ENDIAN */
static struct ddi_device_acc_attr endian_attr = {
	DDI_DEVICE_ATTR_V0,
	DDI_STRUCTURE_BE_ACC,
	DDI_STRICTORDER_ACC
};
#endif /* _LITTLE_ENDIAN */

static struct ddi_device_acc_attr nosw_attr = {
	DDI_DEVICE_ATTR_V0,
	DDI_NEVERSWAP_ACC,
	DDI_STRICTORDER_ACC
};

#define	CDELAY(c, n)	\
{ \
	register int N = n; \
	while (--N > 0) { \
		if (c) \
			break; \
		drv_usecwait(1); \
	} \
}

#define	getprop(devi, name, def)	\
		ddi_getprop(DDI_DEV_T_ANY, (devi), \
		DDI_PROP_DONTPASS, (name), (def))

/* config info */

static int	tvfour_open(dev_t *, int, int, cred_t *);
static int	tvfour_close(dev_t, int, int, cred_t *);
static int	tvfour_ioctl(dev_t, int, int, int, cred_t *, int *);
static int	tvfour_mmap(dev_t, off_t, int);

static struct cb_ops tvfour_cb_ops = {
	tvfour_open,		/* open */
	tvfour_close,		/* close */
	nodev,			/* strategy */
	nodev,			/* print */
	nodev,			/* dump */
	nodev,			/* read */
	nodev,			/* write */
	tvfour_ioctl,		/* ioctl */
	nodev,			/* devmap */
	tvfour_mmap,		/* mmap */
	ddi_segmap,		/* segmap */
	nochpoll,		/* poll */
	ddi_prop_op,		/* cb_prop_op */
	0,			/* streamtab */
	D_NEW | D_MP		/* Driver compatibility flag */
};

static int	tvfour_info(dev_info_t *, ddi_info_cmd_t, void *, void **);
static int	tvfour_attach(dev_info_t *, ddi_attach_cmd_t);
static int	tvfour_detach(dev_info_t *, ddi_detach_cmd_t);
static int	tvfour_power(dev_info_t *, int, int);

static struct dev_ops tvfour_ops = {
	DEVO_REV,		/* devo_rev, */
	0,			/* refcnt */
	tvfour_info,		/* info */
	nulldev,		/* identify */
	nulldev,		/* probe */
	tvfour_attach,		/* attach */
	tvfour_detach,		/* detach */
	nodev,			/* reset */
	&tvfour_cb_ops,		/* driver operations */
	(struct bus_ops *) 0,	/* bus operations */
	tvfour_power,		/* power operations */
};

#define	PVIP_CMAP_ENTRIES	256
#define	PVIP_CURSOR_ENTRIES	2

/*
 * Per-context info:
 *	many registers in the tec and fbc do
 *	not need to be saved/restored.
 */


#define	PVIP_VRTIOCTL	1		/* FBIOVERTICAL in effect */
#define	PVIP_VRTCTR	2		/* OWGX vertical retrace counter */


static u_int	tvfour_pagesize;


/* default structure for FBIOGATTR ioctl */
static struct fbgattr	tvfour_attr;

/*
 * handy macros
 */
static void *state_p;	/* opaque handle where all the state hangs */

#define	getsoftc(instance) \
	((tvfour_softc *)ddi_get_soft_state(state_p, (instance)))

#define	btob(n)			ctob(btoc(n))

#define	PVIP_WAIT		500000	/* .5 seconds */




/*
 * forward references
 */
static u_int	tvfour_intr(tvfour_softc *);
static int	tvfour_getpid(void);

/*
 *
 */

static XV_regmaps XV_promrev1 = {
	{ XVREGNUM, XVBASE, XVBASE, XVSIZE, XVVOFF },
	{ XVREGNUM, C3BASE, C3BASE, C3SIZE, C3VOFF },
	{ XVREGNUM, M1BASE, M1BASE, M1SIZE, M1VOFF },
	{ XVREGNUM, M8BASE, M8BASE, M8SIZE, M8VOFF },
	{ XVREGNUM, M24BASE, M24BASE, M24SIZE, M24VOFF },
	{ XVREGNUM, PROMBASE, PROMBASE, PROMSIZE, PROMVOFF }
};


/* Loadable Driver stuff */

extern struct mod_ops	mod_driverops;

static struct modldrv	tvfour_modldrv = {
	&mod_driverops,		/* Type of module.This one is a driver */
	"sal-xtra pci driver v97/07/21",	/* Name of the module. */
	&tvfour_ops,			/* driver ops */
};

static struct modlinkage	tvfour_modlinkage = {
	MODREV_1,
	(void *) &tvfour_modldrv,
	NULL,
};


int
_init(void)
{
	register int	e;

	DEBUGF(1, (CE_CONT, "tvfour: compiled %s, %s\n", __TIME__, __DATE__));

	if ((e = ddi_soft_state_init(&state_p,
		sizeof (tvfour_softc), 1)) != 0) {
		DEBUGF(1, (CE_CONT, "done\n"));
		return (e);
	}

	e = mod_install(&tvfour_modlinkage);

	if (e) {
		ddi_soft_state_fini(&state_p);
		DEBUGF(1, (CE_CONT, "done\n"));
	}

	DEBUGF(1, (CE_CONT, "tvfour: _init done rtn=%d\n", e));

	return (e);
}

int _fini(void)
{
	register int	e;

	if ((e = mod_remove(&tvfour_modlinkage)) != 0)
		return (e);

	ddi_soft_state_fini(&state_p);

	return (0);
}

int _info(struct modinfo *modinfop)
{
	return (mod_info(&tvfour_modlinkage, modinfop));
}


#define PVSAL_PCI_CONF_ID  0
#define PVSAL_PCI_CONF_CMD 4
#define PVSAL_PCI_CONF_STATUS 6
#define PVSAL_PCI_CONF_REV 8
#define PVSAL_PCI_CONF_BASE 0x10

static int
tvfour_attach(register dev_info_t *devi,
	register ddi_attach_cmd_t cmd)
{
	register tvfour_softc	*softc;
	register int instance;

	char		name[16];
	uint32_t	timestamp[2];
	int		power[2];
	int		proplen, i;
	int		vendor_id,device_id,status,revision;
	int 		*base_addr;


	switch (cmd) 
	{
	case DDI_ATTACH:


	/*
	tvfour_pagesize = ddi_ptob(devi, 1);
	 * Work out which instance we are, and remember
	 */
	instance = ddi_get_instance(devi);

	/* Allocate softc struct */
	if (ddi_soft_state_zalloc(state_p, instance))
	{
		printf("ddi_soft_state_zalloc failed\n");
		return (DDI_FAILURE);
	}

/*
	softc = TVTWO_DEVI2SOFTC(devi);
 */
	softc = getsoftc(instance);

	/* link it in */
	softc->devi = devi;

	DEBUGF(1, (CE_CONT, "tvfour_attach devi=0x%x instance=%d\n", devi, instance));

	ddi_set_driver_private(devi, (caddr_t) softc);

	/* tvthree on the PCI */

	/* map in the configuration registers */
	if (pci_config_setup(devi, &softc->tvfour_conf_handle)) {
		DEBUGF(2, (CE_CONT,
			"tvfour_attach: ddi_conf_regs (pvip) FAILED\n"));
		return (DDI_FAILURE);
	}
	vendor_id = pci_config_getw(softc->tvfour_conf_handle, PVSAL_PCI_CONF_ID);
	device_id = pci_config_getw(softc->tvfour_conf_handle, PVSAL_PCI_CONF_ID+2);
	status=pci_config_getw(softc->tvfour_conf_handle, PVSAL_PCI_CONF_STATUS);
	base_addr = (int *)pci_config_getl(softc->tvfour_conf_handle,PVSAL_PCI_CONF_BASE);
	DEBUGF(1,(CE_CONT,"tvfour_attach vendor_id=0x%x device_id=0x%x status=0x%x\n",
	     vendor_id,device_id,status));
	DEBUGF(1, (CE_CONT, "tvfour_attach base_addr=0x%x \n", base_addr));

	softc->regmap= &XV_promrev1;

	softc->regs= kmem_zalloc(sizeof(XV_regs),KM_SLEEP);
	if(softc->regs == NULL) {
		ddi_soft_state_free(state_p,instance);
		return(DDI_FAILURE);
	}
	/*
	 * Map our main XVideo control registers.
	 */
	if (TVTWO_MAP_REGS(softc, softc->regmap->xv,
		softc->regs->xv,endian_attr) != DDI_SUCCESS) 
	{
		cmn_err(CE_WARN, "tvfour%d: can't map registers\n", instance);
		ddi_soft_state_free(state_p, instance);
		return (DDI_FAILURE);
	}
	/*
	 * Now map our C3 registers.
	 */
	if (TVTWO_MAP_REGS(softc, softc->regmap->c3,
			softc->regs->c3,endian_attr) != DDI_SUCCESS) 
	{
		cmn_err(CE_WARN, "tvfour%d: can't map C3 registers\n", instance);
		TVTWO_UNMAP_REGS(softc);
		ddi_soft_state_free(state_p, instance);
		return (DDI_FAILURE);
	}
	cmn_err(CE_NOTE,"tvfour_attach: regmap->xv=0x%x regmap->c3=0x%x\n",
		softc->regmap->xv,softc->regmap->c3);
	pci_config_teardown(&softc->tvfour_conf_handle);
	ddi_report_dev(devi);
	}
	return (DDI_SUCCESS);
}


static int
tvfour_detach(register dev_info_t *devi,
	register ddi_detach_cmd_t cmd)
{
	register int instance = ddi_get_instance(devi);
	register tvfour_softc *softc = TVTWO_DEVI2SOFTC(devi);
	int i;

	DEBUGF(1, (CE_CONT, "tvfour_detach softc=%x, devi=0x%x\n", softc, devi));

	switch (cmd) {
	case DDI_DETACH:
		TVTWO_UNMAP_REGS(softc);
		TVTWO_UNMAP_REGS(softc);
		if(softc->regs)
		    kmem_free(softc->regs, sizeof(XV_regs));
		ddi_soft_state_free(state_p, instance);
		return (DDI_SUCCESS);
		break;

	case DDI_SUSPEND:
		if (softc == NULL)
			return (DDI_FAILURE);

		mutex_enter(&softc->softc_mutex);



		mutex_exit(&softc->softc_mutex);

		return (DDI_SUCCESS);

	default:
		return (DDI_FAILURE);
	}


}


static int
tvfour_power(dev_info_t *dip, int cmpt, int level)
{
	int		power[2];
	register struct tvfour_softc	*softc;

	(void) ddi_power(dip, cmpt, level);

	return (DDI_SUCCESS);
}


/* ARGSUSED */
static int
tvfour_info(dev_info_t *dip, ddi_info_cmd_t infocmd, void *arg, void **result)
{
	register int	instance;
	register int	error = DDI_SUCCESS;
	register tvfour_softc	*softc;

	instance = getminor((dev_t)arg);
	switch (infocmd) {
	case DDI_INFO_DEVT2DEVINFO:
		softc = getsoftc(instance);
		if (softc == NULL) {
			error = DDI_FAILURE;
		} else {
			*result = (void *)softc->devi;
			error = DDI_SUCCESS;
		}
		break;
	case DDI_INFO_DEVT2INSTANCE:
		*result = (void *)instance;
		error = DDI_SUCCESS;
		break;
	default:
		error = DDI_FAILURE;
	}

	return (error);
}

/*ARGSUSED*/
static
tvfour_open(register dev_t *devp, register int flag,
	register int otyp, register cred_t *cred)
{
	register int instance = getminor(*devp);
	register tvfour_softc	*softc = getsoftc(instance);
	int	error = 0;

	DEBUGF(2, (CE_CONT, "tvfour_open(%d)\n", instance));

	/*
	* is this gorp necessary?
	*/
	if (otyp != OTYP_CHR) {
		error = EINVAL;
	} else if (softc == NULL) {
		error = ENXIO;
	}
	return (error);
}


/*ARGSUSED*/
static
tvfour_close(register dev_t dev, register int flag,
	register int otyp, register cred_t *cred)
{
	register int instance = getminor(dev);
	register tvfour_softc	*softc = getsoftc(instance);
	int	error = 0;

	DEBUGF(2, (CE_CONT, "tvfour_close(%d, %d, %d)\n", instance, flag, otyp));

	if (otyp != OTYP_CHR) {
		error = EINVAL;
	} else if (softc == NULL) {
		error = ENXIO;
	} 

	return (error);
}


#define TVTWO_IN_RANGE(rm, off) \
		(((off) >= (rm).voff) && ((off) < ((rm).voff + (rm).size)))

/* ARGSUSED */
static int
tvfour_mmap(dev_t dev, off_t off, int prot)
{
	register tvfour_softc *softc = TVTWO_DEV2SOFTC(dev);
	register XV_regmaps *rm;
	caddr_t addr;
	u_int pfn;

	if (softc == NULL) {
		return (-1);
	}

	rm = softc->regmap;

	/*
	 * Check the offset against all available ranges and
	 * return the appropriate page frame number.
	 *
	 */

	switch (TVTWO_DEV2MINOR(dev)) {

		case TVTWO_TV2_MINOR:
		{
			if (TVTWO_IN_RANGE(rm->xv, off))
				return tvfour_pfn(softc, &rm->xv, off);
			if (TVTWO_IN_RANGE(rm->c3, off))
				return tvfour_pfn(softc, &rm->c3, off);
			if (TVTWO_IN_RANGE(rm->m1, off))
				return tvfour_pfn(softc, &rm->m1, off);
			if (TVTWO_IN_RANGE(rm->m8, off))
				return tvfour_pfn(softc, &rm->m8, off);
			if (TVTWO_IN_RANGE(rm->m24, off))
				return tvfour_pfn(softc, &rm->m24,off);
			break;
		}

		case TVTWO_C3_MINOR:
		{
			if (TVTWO_IN_RANGE(rm->xv, off))
				return tvfour_pfn(softc, &rm->xv, off);
			if (TVTWO_IN_RANGE(rm->c3, off))
				return tvfour_pfn(softc, &rm->c3, off);
			break;
		}

		case TVTWO_CG3_MINOR:
		{
			if (TVTWO_IN_RANGE(rm->m8, off))
				return tvfour_pfn(softc, &rm->m8, off);
			break;
		}

	}
	return (-1);
}

static int
tvfour_pfn(tvfour_softc *softc, XV_regmap *rm, off_t off)
{
	caddr_t addr;
	u_int pfn;

	/*
	 * We need to adjust our passed in offset from the base of
	 * the "virtual" register window and add in our offset from
	 * the base of the real register.
	 */
	off = (off - rm->voff) + rm->offset;

	if (ddi_map_regs(softc->devi, rm->regnum,
		&addr, off, sizeof(int)) != DDI_FAILURE) {

		pfn = hat_getkpfnum(addr);

		ddi_unmap_regs(softc->devi, rm->regnum,
			&addr, off, sizeof(int));
	}

	return (pfn);
}

/*ARGSUSED*/
static
tvfour_ioctl(dev_t dev, int cmd, int data, int mode, cred_t *cred, int *rval)
{
	register tvfour_softc *softc = getsoftc(getminor(dev));
	register int cursor_cmap;
	int	 i;

	u_char	*iobuf_cmap_red;
	u_char	*iobuf_cmap_green;
	u_char	*iobuf_cmap_blue;
	u_char	*stack_cmap;

	struct fbcmap	*cmap, cma;
	struct fbcursor	cp;

	u_int	index;
	u_int	count;
	u_char	*map;
	u_int	entries;

	DEBUGF(3, (CE_CONT, "tvfour_ioctl(%d, 0x%x)\n", getminor(dev), cmd));


	switch (cmd) {
	case VIS_GETIDENTIFIER:
		break;
	default:
		break;
	}				/* switch(cmd) */

	return (0);
}


static u_int
tvfour_intr(tvfour_softc *softc)
{


	return (DDI_INTR_CLAIMED);
}


/*
 * tvfour_getpid()
 *
 * Simple wrapper around process ID call to drv_getparm(9f).
 */
static int
tvfour_getpid(void)
{
	ulong_t	rval;

	if (drv_getparm(PPID, &rval) == -1)
		return (0);
	else
		return ((int)rval);
}

