/*
 * Copyright (c) 1993 Parallax Graphics Inc.
 */

#pragma ident "@(#)tvtwo.c	1.10 Copyright (c) 1993 Parallax Graphics Inc"

/*
 * XVideo device driver
 */

#include <sys/types.h>
#include <sys/open.h>		/* OTYP */
#include <sys/kmem.h>
#include <sys/stat.h>		/* S_IFCHR */
#include <sys/conf.h>
#include <sys/ddi.h>
#include <sys/sunddi.h>

#include "tvtwo.h"
#ifdef RTV_SUPPORT
#define MAX_BANDWIDTH 28*1024
#define BUCKET_SIZE 64*1024

/*
 * Externalize variables for the rtvmod streams module.
 */
#ifdef RAMDISK_MASTER
char _depends_on[] = "usr/kernel/drv/ramdisk";
extern SHM_FIELD_QUE *global_shm_que;
#ifdef TV3
char _depends_on[] = "drv/tvthree";
#else /* TV3 */
char _depends_on[] = "drv/tvtwo";
#endif /* TV3 */
#else
#if 0
SHM_FIELD_QUE *global_send_que[3];
SHM_FIELD_QUE *global_recv_que[3];
#endif
SHM_FIELD_QUE *global_shm[3];
#endif

typedef struct {
	int sid;
	}SID_ATTRS;

typedef struct {
	int sid;
	int record_size;
	}REQUEST_RATE_INFO;

typedef struct {
	int wid;
	int start_x;
	int start_y;
	int width;
	int height;
	int video_tag;
	int single_field;
	}START_INFO;

typedef struct {
	int qfactor;
	int interlace;
	int max_rate;
	int fps;
	}SETUP_INFO;

#endif


void *state_p;	/* opaque handle where all the state hangs */

/*
 * Register kludge ... the old proms didn't map enough of the
 * board to be useful.  Under 2.2 We overrode the default registers
 * in tvtwo.conf so we could map everything that we need.
 *
 *	registers=-1,0x40000,0x7c0000, -1,0x10000,0x28000;  
 */
static XV_regmaps XV_promrev0 = {
	{ XVREGNUM, XVBASE, XVREGOFF, XVREGSIZE, XVVOFF },
	{ C3REGNUM, C3BASE, C3REGOFF, C3REGSIZE, C3VOFF },
	{ M1REGNUM, M1BASE, M1REGOFF, M1REGSIZE, M1VOFF },
	{ M8REGNUM, M8BASE, M8REGOFF, M8REGSIZE, M8VOFF },
	{ M24REGNUM, M24BASE, M24REGOFF, M24REGSIZE, M24VOFF },
	{ PROMREGNUM, PROMBASE, PROMREGOFF, PROMREGSIZE, PROMVOFF }
};

/*
 * The new XVideo prom maps the entire 8MB range into R0.
 *
 * XV_regmap is used mainly by tvtwo_mmap to pass back the
 * appropriate pfn.  Since we are currenly indiscriminate about
 * what addresses we pass back we could simplify things by just
 * returning mappings from anywhere, but we keep this here just
 * in case we would like to restrict access in the future.
 */

static XV_regmaps XV_promrev1 = {
	{ XVREGNUM, XVBASE, XVBASE, XVSIZE, XVVOFF },
	{ XVREGNUM, C3BASE, C3BASE, C3SIZE, C3VOFF },
	{ XVREGNUM, M1BASE, M1BASE, M1SIZE, M1VOFF },
	{ XVREGNUM, M8BASE, M8BASE, M8SIZE, M8VOFF },
	{ XVREGNUM, M24BASE, M24BASE, M24SIZE, M24VOFF },
	{ XVREGNUM, PROMBASE, PROMBASE, PROMSIZE, PROMVOFF }
};

static int
tvtwo_open(dev_t * dev, int flag, int otyp, cred_t * credp);

static int
tvtwo_close(dev_t dev, int flag, int otyp, struct cred * credp);

static int
tvtwo_mmap(dev_t dev, off_t off, int prot);

static int
tvtwo_write(dev_t dev, struct uio *uio, cred_t *credp);

static int
tvtwo_read(dev_t dev, struct uio *uio, cred_t *credp);

static int
tvtwo_rdwr(int flags, dev_t dev, struct uio *uio);

static int
tvtwo_ccube_strategy(struct buf *bp);

static int
tvtwo_nvram_strategy(struct buf *bp);

static int
tvtwo_ioctl(dev_t, int, int, int, cred_t *, int *);

static struct cb_ops tvtwo_cb = {
	tvtwo_open,	/* open		*/
	tvtwo_close,	/* close	*/
	nodev,		/* strategy	*/
	nodev,		/* print	*/
	nodev,		/* dump		*/
	tvtwo_read,	/* read		*/
	tvtwo_write,	/* write	*/
	tvtwo_ioctl,	/* ioctl	*/
	nodev,		/* devmap	*/
	tvtwo_mmap,	/* mmap		*/
	ddi_segmap,	/* segmap	*/
	nochpoll,	/* poll		*/
	ddi_prop_op,	/* props	*/
	0,		/* streamtab	*/
	(D_NEW | D_MP)	/* compat flag	*/
};
static int
tvtwo_identify(dev_info_t * devi);

static int
tvtwo_probe(dev_info_t *devi);

static int
tvtwo_attach(dev_info_t * devi, ddi_attach_cmd_t cmd);

static int
tvtwo_detach(dev_info_t * devi, ddi_detach_cmd_t cmd);

static int
tvtwo_info(dev_info_t *, ddi_info_cmd_t, void *, void **);

struct dev_ops tvtwo_ops = {
	DEVO_REV,		/* version		*/
	0,			/* refcnt		*/
	tvtwo_info,		/* info			*/
	tvtwo_identify,		/* identify		*/
	tvtwo_probe,		/* probe		*/
	tvtwo_attach,		/* attach		*/
	tvtwo_detach,		/* detach		*/
	nodev,			/* reset		*/
	&tvtwo_cb,		/* driver operations	*/
	(struct bus_ops *) 0	/* no bus operations	*/
};


static u_int
tvtwo_intr(caddr_t i_handler_arg);

static void
tvtwo_minphys(struct buf *bp);

static int
tvtwo_pfn(tvtwo_softc *softc, XV_regmap *rm, off_t off);

#include <sys/modctl.h>

extern struct mod_ops mod_driverops;


static struct modldrv modldrv = {
	&mod_driverops,		/* Type of module.  This one is a driver */
	"CCube 550 driver 3.0",/* Driver title */
	&tvtwo_ops		/* driver ops */
};

static struct modlinkage modlinkage = {
	MODREV_1, (void *) &modldrv, NULL
};

int
_init(void)
{
	register int error;

	/*
	 * Initialize our soft state pointer.  Used as a place to hang our
	 * set of per-instance structures.
	 */
	if ((error = ddi_soft_state_init(&state_p, sizeof (tvtwo_softc), 1))
		!= 0) {
		return (error);
	}
	if ((error = mod_install(&modlinkage)) != 0) {
		ddi_soft_state_fini(&state_p);
	}
#ifdef TV3
	cmn_err(CE_NOTE,"Parallax TV3 Rtv Driver Loaded"); 
#else
	cmn_err(CE_NOTE,"Parallax TV2 Rtv Driver Loaded"); 
#endif

	return (error);
}


/*
 * Build the unload transaction, if possible.
 */

int
_fini(void)
{
	register int error;

	if ((error = mod_remove(&modlinkage)) != 0) {
		return (error);
	}

	/*
	 * Undo the rest of _init.
	 */
	ddi_soft_state_fini(&state_p);

	return (0);
}

int
_info(struct modinfo * modinfop)
{
	return (mod_info(&modlinkage, modinfop));
}

static int
tvtwo_identify(dev_info_t *devi)
{
#ifdef TV3
	DEBUGF((CE_CONT,"tvtwo_identify = %s\n",ddi_get_name(devi)));
	if (strcmp(ddi_get_name(devi), TVTHREE_PROMID) == 0) {
		return (DDI_IDENTIFIED);
#else /* ! TV3 */
	if (strcmp(ddi_get_name(devi), TVTWO_PROMID) == 0) {
		return (DDI_IDENTIFIED);
#endif /* TV3 */
	} else {
		return (DDI_NOT_IDENTIFIED);
	}
}

/* ARGSUSED */
static int
tvtwo_probe(dev_info_t *devi)
{
	return (DDI_PROBE_SUCCESS);
}

#define WAIT_FOR_MEM 1

static int
tvtwo_attach(dev_info_t *devi, ddi_attach_cmd_t cmd)
{
ddi_dma_cookie_t dma_cookie;    /* for dma information */
register QUEUE_INFO *record_qinfo,*playback_qinfo;
register int i,k;


	switch (cmd) 
	{
	    case DDI_ATTACH:
	    {

		register int instance;
		register tvtwo_softc *softc;

		/*
		 * Work out which instance we are, and remember
		 */
		instance = ddi_get_instance(devi);

		/*
		 * Allocate a soft state structure for this 
		 * instance and set up some handy backward links.
		 */
		if (ddi_soft_state_zalloc(state_p, instance) != 0) {
				return (DDI_PROBE_FAILURE);
		}
		softc = TVTWO_DEVI2SOFTC(devi);
		softc->devi = devi;
		softc->bp = NULL;
		softc->maxphys = 0;
		softc->ffield = -1;
		softc->sfield = -1;
		softc->visleep = 0;
		softc->instance=instance;
		ddi_set_driver_private(devi, (caddr_t) softc);

		/*
		 * Setup a structure that tells us how to
		 * map our registers.  The old proms are
		 * overridden by tvtwo.conf and XV_promrev0
		 * maps the registers appropriately.
		 */
		{
			int nregs;
			ddi_dev_nregs(devi, &nregs);
#ifdef TV3
			softc->regmap = &XV_promrev1;
#else /* !TV3 */
			if (nregs == 1)
				softc->regmap = &XV_promrev1;
			else
				softc->regmap = &XV_promrev0;
#endif /* TV3 */
		}
#ifdef RTV_SUPPORT
		softc->shm_que = 
		    kmem_zalloc(sizeof(SHM_FIELD_QUE),KM_SLEEP);
#if 0
	        global_recv_que[softc->instance] =softc->shm_que;
	        global_send_que[softc->instance] =softc->shm_que;
#endif
	        global_shm[softc->instance] =softc->shm_que;
		/*
	 	 * Allocate 60 contiguous buffers of 32K 
		 *    (approximate upper limit ccube field size).
	 	 */
		softc->request_size = MAX_SHM_SIZE;
		if(ddi_mem_alloc(softc->devi,NULL,(u_int)softc->request_size,
			(u_int)WAIT_FOR_MEM,(caddr_t *)&softc->input_shared_data,
			(u_int *)&softc->actual_input_size) != DDI_SUCCESS) 
		{
		     	cmn_err(CE_WARN,"tvtwo:%d cannot allocate ccube shared memory!\n",
				ddi_get_instance(softc->devi));
				return(DDI_FAILURE);
		}
		cmn_err(CE_NOTE,"! fldbuf address=0x%x actual_size=%d \n",
			softc->input_shared_data,softc->actual_input_size);
		/*
	 	* Map memory to dma.
	 	*/
		if(ddi_dma_addr_setup(softc->devi,NULL,
		    softc->input_shared_data,
		    softc->actual_input_size,
		    DDI_DMA_RDWR,DDI_DMA_SLEEP,(caddr_t)0,
		    NULL,&softc->ccube_read_dma)!=DDI_DMA_MAPPED)
		    {
		        ddi_dma_free(softc->input_shared_data);
	    		cmn_err(CE_WARN,"ccube %d: cannot map input buffer",
					ddi_get_instance(softc->devi));
				return(DDI_FAILURE);
		    }
		softc->shm_que->dma_in_addr=softc->input_shared_data;
		softc->shm_que->dma_out_addr=softc->input_shared_data;
		softc->shm_que->max_shm_size=softc->actual_input_size;

	        playback_qinfo= 
		    (QUEUE_INFO *)&softc->shm_que->playback_streams[0];
		sema_init(&playback_qinfo->sema_lock,1,"play queue lock",
			SEMA_DRIVER,(void *)NULL);
	        playback_qinfo= 
		    (QUEUE_INFO *)&softc->shm_que->playback_streams[1];
		sema_init(&playback_qinfo->sema_lock,1,"play queue lock",
			SEMA_DRIVER,(void *)NULL);
	        record_qinfo= 
		    (QUEUE_INFO *)&softc->shm_que->playback_streams[0];
		sema_init(&record_qinfo->sema_lock,1,"record queue lock",
			SEMA_DRIVER,(void *)NULL);
#endif
		/*
		 * Allocate our register structure.
		 * We map multiple areas so we need more than
		 * just one pointer.
		 */
		softc->regs = kmem_zalloc(sizeof(XV_regs), KM_SLEEP);
		if (softc->regs == NULL) {
			ddi_soft_state_free(state_p, instance);
			return (DDI_FAILURE);
		}
		/*
		 * Map our main XVideo control registers.
		 */

		if (TVTWO_MAP_REGS(softc, softc->regmap->xv,
			softc->regs->xv) != DDI_SUCCESS) 
		{
			cmn_err(CE_WARN,
			"tvtwo%d: can't map registers\n",
				instance);
			ddi_soft_state_free(state_p, instance);
			return (DDI_FAILURE);
		}

		/*
		 * Now map our C3 registers.
		 */
		if (TVTWO_MAP_REGS(softc, softc->regmap->c3,
			softc->regs->c3) != DDI_SUCCESS) 
			{
			cmn_err(CE_WARN,
				"tvtwo%d: can't map C3 registers\n",
				instance);
			TVTWO_UNMAP_REGS(softc, softc->regmap->xv,
				softc->regs->xv);
				ddi_soft_state_free(state_p, instance);
				return (DDI_FAILURE);
			}
		/*
		 * Fetch the "freqcode" attribute from Forth
		 */

		{
			char buf[10];
			char freqcode;
			int ii=10;
			if(ddi_getlongprop_buf(DDI_DEV_T_ANY,
				devi,
				DDI_PROP_DONTPASS,
				"freqcode",
				buf,&ii) == DDI_PROP_NOT_FOUND)
			{
/*
				    DEBUGF((CE_CONT,"prop not found\n"));
*/
				    freqcode = '1';
			}
			else
			{
/*
				    DEBUGF((CE_CONT,"freqcode = %c\n",
					buf[0]));
				    buf[9] = '\0';
				    DEBUGF((CE_CONT,"freqcodes = %s\n",buf));
*/
				    freqcode = buf[0];
			}
			softc->freqcode=freqcode;
		}

		/*
		 * Now initialize the rest of the board
		 */
		DEBUGF((CE_CONT,"tvtwo_attach before tvtwo_init\n"));
		if (tvtwo_init(softc) != DDI_SUCCESS) {
			cmn_err(CE_WARN,
				"tvtwo%d: can't initialize\n",
				instance);
			TVTWO_UNMAP_REGS(softc, softc->regmap->xv,
				softc->regs->xv);
			TVTWO_UNMAP_REGS(softc, softc->regmap->c3,
				softc->regs->c3);
			ddi_soft_state_free(state_p, instance);
		}

		/*
		 * Create the device nodes in /devices
		 *
		 * We name the framebuffer as:
		 *
		 *	tvtwo@<addr>:tvtwo<instance>
		 *
		 * which is the Sun framebuffer convention.
		 * All other devices are mainly for control
		 * and administration.
		 */
		DEBUGF((CE_CONT,"tvtwo_attach before device node creation\n"));
		{
		    char minor[16];

#ifdef TV3
		    sprintf(minor, "tvthree%d", instance);
		    ddi_create_minor_node(devi, minor, S_IFCHR,
			    (int) TVTWO_INST2MINOR_TV2(instance),
			    NULL, 0);
#else
		    sprintf(minor, "tvtwo%d", instance);
		    ddi_create_minor_node(devi, minor, S_IFCHR,
			    (int) TVTWO_INST2MINOR_TV2(instance),
			    DDI_NT_DISPLAY, 0);
		    ddi_create_minor_node(devi, "nvram", S_IFCHR,
			    (int) TVTWO_INST2MINOR_NVRAM(instance),
			    DDI_PSEUDO, 0);
		    ddi_create_minor_node(devi, "opt", S_IFCHR,
			    (int) TVTWO_INST2MINOR_OPT(instance),
			    DDI_PSEUDO, 0);
		    ddi_create_minor_node(devi, "tuner", S_IFCHR,
			    (int) TVTWO_INST2MINOR_TUNER(instance),
			    DDI_PSEUDO, 0);
		    ddi_create_minor_node(devi, "ccube", S_IFCHR,
			    (int) TVTWO_INST2MINOR_C3(instance),
			    DDI_PSEUDO, 0);
		    sprintf(minor, "cgthree%d", instance);
		    ddi_create_minor_node(devi, minor, S_IFCHR,
			    (int) TVTWO_INST2MINOR_CG3(instance),
			    DDI_NT_DISPLAY, 0);
#endif
		}

		/*
		 * Register our interrupt handler.
		 */
		DEBUGF((CE_CONT,"tvtwo_attach before ddi_add_intr\n"));
		ddi_add_intr(devi, (u_int) 0, &softc->iblkc,
			&softc->idevc, tvtwo_intr, (caddr_t) softc);

		/*
		 * Create and initialize mutex and condition variables.
		 */
		DEBUGF((CE_CONT,"tvtwo_attach before mutex_init's\n"));
		mutex_init(&softc->regs_mutex, "tvtwo register mutex",
				MUTEX_DRIVER, softc->iblkc);
		mutex_init(&softc->softc_mutex, "tvtwo softc mutex",
				MUTEX_DRIVER, softc->iblkc);
		mutex_init(&softc->dma_lock, "tvtwo dma mutex",
				MUTEX_DRIVER, softc->iblkc);
		cv_init(&softc->dmadone, "tvtwo dma",CV_DRIVER, NULL);
#ifdef RTV_SUPPORT
		mutex_init(&softc->rtv_dma_lock, "rtv dma mutex",
				MUTEX_DRIVER, softc->iblkc);
		mutex_init(&softc->rtv_sleep_lock, "rtv dma mutex",
				MUTEX_DRIVER, softc->iblkc);
		cv_init(&softc->rtv_dma_done, "rtv dma",CV_DRIVER, NULL);
		cv_init(&softc->rtv_wake_lock,"rtv wakeup",CV_DRIVER,NULL);

                sema_init(&softc->shm_que->recv_lock,1,"main lock",
			    SEMA_DRIVER,(void *)NULL);
                sema_init(&softc->shm_que->send_lock,1,"main lock",
			    SEMA_DRIVER,(void *)NULL);
#endif

		/*
		 * Report our device instance.
		 * This will only show when booting -v ...
		 */
		DEBUGF((CE_CONT,"tvtwo_attach before ddi_report_device\n"));
		ddi_report_dev(devi);

		DEBUGF((CE_CONT,"tvtwo_attach exit\n"));
		return (DDI_SUCCESS);

	    }

	    default:
		/*
		 * Other values of 'cmd' are reserved and may be
		 * extended in future releases.  So we check to see
		 * that we're only responding for the case we've
		 * implemented here.
		 */
		DEBUGF((CE_CONT,"tvtwo_attach exit bad cmd\n"));
		return (DDI_FAILURE);
	}
}

/*
 * The detach routine is the logical inverse of the
 * attach routine.  If it completes successfully it
 * is assumed by the framework that the device is
 * entirely quiescent.
 */

static int
tvtwo_detach(dev_info_t * devi, ddi_detach_cmd_t cmd)
{
register QUEUE_INFO *record_qinfo,*playback_qinfo;
register tvtwo_softc *softc = TVTWO_DEVI2SOFTC(devi);
register int i,k;

	if (softc == NULL) {
		return (DDI_FAILURE);
	}
	switch (cmd) 
	{
	    case DDI_DETACH:
	    {
		/*
		* Tear down in reverse order from attach
		*/

#ifdef RTV_SUPPORT
	        record_qinfo= 
		    (QUEUE_INFO *)&softc->shm_que->playback_streams[0];
                 sema_destroy(&softc->shm_que->recv_lock);
                 sema_destroy(&softc->shm_que->send_lock);
	        playback_qinfo= 
		    (QUEUE_INFO *)&softc->shm_que->playback_streams[0];
                 sema_destroy(&playback_qinfo->sema_lock);
	        playback_qinfo= 
		    (QUEUE_INFO *)&softc->shm_que->playback_streams[1];
                 sema_destroy(&playback_qinfo->sema_lock);
	        record_qinfo= 
		    (QUEUE_INFO *)&softc->shm_que->playback_streams[0];
                 sema_destroy(&record_qinfo->sema_lock);
#endif
		/*
		 * disable interrupts
		 */
		tvtwo_uninit(softc);
		/*
		 * Unregister any pending timeouts, otherwise
		 * we crash when they hit.
		 */
		if (softc->timeout)
			untimeout(softc->tid);
		
		ddi_remove_intr(devi, 0, &softc->iblkc);

		/*
		 * Remove all minor nodes
		 */
		ddi_remove_minor_node(devi, (char *) NULL);

		mutex_destroy(&softc->regs_mutex);
		mutex_destroy(&softc->softc_mutex);
		mutex_destroy(&softc->dma_lock);
		cv_destroy(&softc->dmadone);
#ifdef RTV_SUPPORT
		mutex_destroy(&softc->rtv_dma_lock);
		mutex_destroy(&softc->rtv_sleep_lock);
		cv_destroy(&softc->rtv_dma_done);
		cv_destroy(&softc->rtv_wake_lock);
#endif

		TVTWO_UNMAP_REGS(softc, softc->regmap->xv,
			softc->regs->xv);
		TVTWO_UNMAP_REGS(softc, softc->regmap->c3,
			softc->regs->c3);

#ifdef RTV_SUPPORT
		if(softc->ccube_read_dma)
		   	ddi_dma_free(softc->ccube_read_dma);
	     	if(softc->input_shared_data)
			ddi_mem_free((caddr_t)softc->input_shared_data);
		if(softc->shm_que) 
			kmem_free(softc->shm_que,sizeof(SHM_FIELD_QUE));
#endif

		if(softc->regs)
			kmem_free(softc->regs, sizeof(XV_regs));
		ddi_soft_state_free(state_p, ddi_get_instance(devi));
		return (DDI_SUCCESS);
	    }
	    default:
		return (DDI_FAILURE);
	}
}

/* ARGSUSED */
static int
tvtwo_info(dev_info_t * dip, ddi_info_cmd_t infocmd, void *arg, void **result)
{
	register dev_t dev = (dev_t) arg;
	register int error;
	register tvtwo_softc *softc = TVTWO_DEV2SOFTC(dev);

	*result = (void *) NULL;

	switch (infocmd) {

		case DDI_INFO_DEVT2DEVINFO:

			if (softc == NULL) {
				error = (DDI_FAILURE);
			} else {
				*result = (void *) softc->devi;
				error = DDI_SUCCESS;
			}
			break;

		case DDI_INFO_DEVT2INSTANCE:

			*result = (void *) TVTWO_DEV2INST((dev_t) arg);
			error = DDI_SUCCESS;
			break;

		default:

			error = DDI_FAILURE;
			break;
	}

	return (error);
}

/* ARGSUSED */
static int
tvtwo_open(dev_t * dev, int flag, int otyp, cred_t * credp)
{
register tvtwo_softc *softc = TVTWO_DEV2SOFTC(*dev);


	if (softc == NULL) {
		return (ENXIO);
	}

	if (otyp != OTYP_CHR) {
		return (EINVAL);
	}
	softc->rtv_dma_state=0;
	return (0);
}

/* ARGSUSED */
static int
tvtwo_close(dev_t dev, int flag, int otyp, struct cred * credp)
{
	register tvtwo_softc *softc = TVTWO_DEV2SOFTC(dev);

	if (softc == NULL) {
		return (ENXIO);
	}
#ifdef RTV_SUPPORT
	disable_interrupts(softc);
#endif


	if (otyp != OTYP_CHR) {
		return (EINVAL);
	}
	return (0);
}

#define TVTWO_IN_RANGE(rm, off) \
		(((off) >= (rm).voff) && ((off) < ((rm).voff + (rm).size)))

/* ARGSUSED */
static int
tvtwo_mmap(dev_t dev, off_t off, int prot)
{
	register tvtwo_softc *softc = TVTWO_DEV2SOFTC(dev);
	register XV_regmaps *rm;
	caddr_t addr;
	u_int pfn;

	if (softc == NULL) {
		return (-1);
	}
	if(softc->rtv_dma_state!=0)
	{
		addr= softc->input_shared_data;
		pfn = hat_getkpfnum(addr);
		return(pfn);
	}


	rm = softc->regmap;

	/*
	 * Check the offset against all available ranges and
	 * return the appropriate page frame number.
	 *
	 */

	switch (TVTWO_DEV2MINOR(dev)) {

		case TVTWO_TV2_MINOR:
		{
			if (TVTWO_IN_RANGE(rm->xv, off))
				return tvtwo_pfn(softc, &rm->xv, off);
			if (TVTWO_IN_RANGE(rm->c3, off))
				return tvtwo_pfn(softc, &rm->c3, off);
			if (TVTWO_IN_RANGE(rm->m1, off))
				return tvtwo_pfn(softc, &rm->m1, off);
			if (TVTWO_IN_RANGE(rm->m8, off))
				return tvtwo_pfn(softc, &rm->m8, off);
			if (TVTWO_IN_RANGE(rm->m24, off))
				return tvtwo_pfn(softc, &rm->m24,off);
			break;
		}

		case TVTWO_C3_MINOR:
		{
			if (TVTWO_IN_RANGE(rm->xv, off))
				return tvtwo_pfn(softc, &rm->xv, off);
			if (TVTWO_IN_RANGE(rm->c3, off))
				return tvtwo_pfn(softc, &rm->c3, off);
			break;
		}

		case TVTWO_CG3_MINOR:
		{
			if (TVTWO_IN_RANGE(rm->m8, off))
				return tvtwo_pfn(softc, &rm->m8, off);
			break;
		}

	}
	return (-1);
}

static int
tvtwo_pfn(tvtwo_softc *softc, XV_regmap *rm, off_t off)
{
	caddr_t addr;
	u_int pfn;

	/*
	 * We need to adjust our passed in offset from the base of
	 * the "virtual" register window and add in our offset from
	 * the base of the real register.
	 */
	off = (off - rm->voff) + rm->offset;

	if (ddi_map_regs(softc->devi, rm->regnum,
		&addr, off, sizeof(int)) != DDI_FAILURE) {

		pfn = hat_getkpfnum(addr);

		ddi_unmap_regs(softc->devi, rm->regnum,
			&addr, off, sizeof(int));
	}

	return (pfn);
}

/* ARGSUSED */
static int
tvtwo_write(dev_t dev, struct uio *uio, cred_t *credp)
{
	return tvtwo_rdwr(B_WRITE, dev, uio);
}

/* ARGSUSED */
static int
tvtwo_read(dev_t dev, struct uio *uio, cred_t *credp)
{
	return tvtwo_rdwr(B_READ, dev, uio);
}

static int
tvtwo_rdwr(int flags, dev_t dev, struct uio *uio)
{
	register tvtwo_softc *softc = TVTWO_DEV2SOFTC(dev);
	int error = EINVAL;

	switch (TVTWO_DEV2MINOR(dev)) {

		case TVTWO_NVRAM_MINOR:
		case TVTWO_OPT_MINOR:
		case TVTWO_TUNER_MINOR:
		{
			error = physio(tvtwo_nvram_strategy, NULL,
				dev, flags, tvtwo_minphys, uio);
			/*
			 * If the nvram is updated we must reinitialize in order
			 * to make sure the new values take effect.
			 */
			if ((flags & B_WRITE) && (error == DDI_SUCCESS)) {
				tvtwo_reinit(softc);
			}
			break;
		}

		/*
		 * XXX
		 * tvtwo currently allows access to the ccube
		 * dma engine.
		 * XXX
		 */
		case TVTWO_C3_MINOR:
		case TVTWO_TV2_MINOR:
		{
			error = physio(tvtwo_ccube_strategy, NULL,
				dev, flags, tvtwo_minphys, uio);
			break;
		}
	}

	return (error);
}

/*
 * We need an nvram strategy routine so that we
 * can use physio to lock down the pages for us.
 */
static int
tvtwo_nvram_strategy(struct buf *bp)
{
	register tvtwo_softc *softc = TVTWO_DEV2SOFTC(bp->b_edev);
	int error = 0;

	/*
	 * XXX offset is always zero XXX
	 */
	bp_mapin(bp);

	if (bp->b_flags & B_READ) {
		if (tvtwo_nv_read(softc->regs,
			bp->b_edev, paddr(bp), 0, bp->b_bcount) != 0)
			error = EIO;
	} else {
		if (tvtwo_nv_write(softc->regs,
			bp->b_edev, paddr(bp), 0, bp->b_bcount) != 0)
			error = EIO;
	}

	bp_mapout(bp);
	iodone(bp);

	return (error);
}

/*
 * tvtwo_ccube_strategy is really only for access to the CCube codec
 * engine.  There is no block device access for this hardware
 * so no strategy routine is not listed in the cb_ops structure above.
 */

static int
tvtwo_ccube_strategy(struct buf *bp)
{
	register tvtwo_softc *softc = TVTWO_DEV2SOFTC(bp->b_edev);
	ddi_dma_cookie_t dma_cookie;    /* for dma information */
	unsigned int imask, startfield;
	static int frm=0;

	if (softc == NULL) {
		bp->b_flags |= B_ERROR;
		bp->b_error = ENXIO;
		iodone(bp);
		return (0);
	}

	mutex_enter(&softc->dma_lock);
	/* save int mask, and starting field  state */
	 
	 imask = XV_RSDW(softc->regs, IntMask.l);
	 startfield = (XV_RREG(softc->regs, IntStat.l) & 0x100) >> 8;
			  

	c3_load_dct(softc); /* XXX ??? XXX */

	if (ddi_dma_buf_setup(softc->devi, bp,
		(bp->b_flags & B_WRITE) ? DDI_DMA_WRITE : DDI_DMA_READ,
		DDI_DMA_DONTWAIT, NULL, NULL, &softc->dma) != DDI_DMA_MAPPED) {
			bp->b_flags |= B_ERROR;
			bp->b_error = EIO;
			iodone(bp);
			return (0);
	}

	/*
	 * We need to convert the dma_handle to a dma_cookie so we can get
	 * the DMA address for the TVTWO device to write to.
	 */
	if (ddi_dma_htoc(softc->dma, 0, &dma_cookie) != 0) {
		bp->b_flags |= B_ERROR;
		bp->b_error = EIO;
		ddi_dma_free(softc->dma);
		iodone(bp);
		return (0);
	}

	/*
	 * Save the buf pointer for our interrupt routine to update
	 * when the transfer is complete (or fails).
	 */
	softc->bp = bp;

	/*
	 * Setup DMA registers address.
	 */
	softc->addr = (caddr_t)dma_cookie.dmac_address;

	if (bp->b_flags & B_READ)
	{
		int thisfield, myfield, t;

		/* do we need to wait for a field? */

		if(softc->sfield != -1)
		{
			if(softc->ffield != -1)	/* ffield takes presidence */
				myfield = softc->ffield << 8;
			else
				myfield = softc->sfield << 8;

			do		/* allways wait for at least 1 int */
			{
				softc->visleep = 1;
				t = XV_RSDW(softc->regs, IntMask.l);
				if((t & 0x8) == 0)
				{
					t |= 0x8; /* set int on */
					XV_WREG(softc->regs, IntMask.l, t);
					t = XV_RREG(softc->regs, IntStatC.l); /* clear any ints */
				}
					/* wait for interrupt */
				cv_wait(&softc->viwake, &softc->dma_lock);
				softc->visleep = 0;
			    thisfield = XV_RREG(softc->regs, IntStat.l) & 0x100;
			} while (thisfield == myfield);
	    	softc->sfield = -1;
		}
    }
	/*
	 * Enable interrupts and start DMA.
	 */
	if (c3_start_dma(softc, softc->addr, bp) != DDI_SUCCESS) {
		bp->b_flags |= B_ERROR;
		bp->b_error = EIO;
		ddi_dma_free(softc->dma);
		iodone(bp);
		softc->bp = NULL;
		mutex_exit(&softc->dma_lock);
		return (0);
	}
 
	/*
	 * Wait for completion or error.
	 */
	cv_wait(&softc->dmadone, &softc->dma_lock);

	/*
	 * Free up resources and wakeup any waiters.
	 */
	softc->bp = NULL;
	ddi_dma_free(softc->dma);
	iodone(bp);
        imask = XV_RSDW(softc->regs, IntMask.l) | (imask & 9);
        XV_WREG(softc->regs, IntMask.l, imask);
        imask = XV_RREG(softc->regs, IntStatC.l);
        if(softc->ffield != -1)
        {
               imask = (imask & 0x100) >> 8;
               if((imask != startfield) && (imask != softc->ffield))
                       XV_WREG4(softc->regs, VCtl.l, 2);
        }
	mutex_exit(&softc->dma_lock);

	return (DDI_SUCCESS);
}

static void
tvtwo_minphys(struct buf *bp)
{
	register tvtwo_softc *softc = TVTWO_DEV2SOFTC(bp->b_edev);

	if (softc->maxphys && (bp->b_bcount > softc->maxphys))
		bp->b_bcount = softc->maxphys;
}
#ifdef RTV_SUPPORT

static void
alarm_clock(tvtwo_softc *softc)
{
cmn_err(CE_WARN,"! alarm_clock: rtv_sleeping=%d\n",softc->rtv_sleeping);
    if(softc->rtv_sleeping == 1)
    {
        mutex_enter(&softc->rtv_sleep_lock);
        cv_broadcast(&softc->rtv_wake_lock);
        mutex_exit(&softc->rtv_sleep_lock);
    }
    softc->recv_action[0]=0;
    softc->rtv_sleeping=0;
}
typedef struct {
	int no;
	int *data;
	}FLD;
typedef struct {
	int field_no;
	int length;
	int qfactor;
	}FLD_HDR;

typedef struct {
	int delay_time;
	int qlev;
	}POLL_INFO;

static int
tvtwo_rtv_init(tvtwo_softc *softc)
{
register int i,k,offset;
register QUEUE_INFO *record_qinfo,*playback_qinfo;
SHM_FIELD_QUE *shm_que = (SHM_FIELD_QUE *)softc->shm_que;

	   softc->rtv_dma_state=RTV_HOLD;
	   softc->ccube_comp_state= softc->ccube_comp_state= DST_IDLE;
	   softc->stream_mask=0;
	   softc->stream_id=0;
	   softc->frame_count=0;
	   softc->ccube_operation=1;
	   softc->interlace=0;
	   softc->audio_sync=0;
	   softc->scan_size= 1280;
	   softc->compress_qindex=0;
	   softc->decomp_qindex=0;
	   softc->field_qindex=0;
	   softc->rtv_sleeping=0;
	   softc->record_size=MAX_FIELD_SIZE;
	   softc->field_offset[0]=softc->field_offset[1]=0;
	   softc->play_action=softc->record_action=0;
	   softc->que_level_in[0]=softc->que_level_out[0]=0;
	   shm_que->que_level_in[0]=0;
	   shm_que->record_action=PAUSE;
	   shm_que->play_action=PAUSE;
	   shm_que->dma_in_offset=0;
	 for(k=0;k<4;k++)
	 {
	       playback_qinfo= (QUEUE_INFO *)&shm_que->playback_streams[k];
	       playback_qinfo->head= playback_qinfo->que;
	       playback_qinfo->tail= &playback_qinfo->que[MAX_FIELD_QUE_SIZE-1];
	       playback_qinfo->input=playback_qinfo->output=playback_qinfo->head;
	       playback_qinfo->level=0;

	       record_qinfo= (QUEUE_INFO *)&shm_que->record_streams[k];
	       record_qinfo->head= record_qinfo->que;
	       record_qinfo->tail= &record_qinfo->que[MAX_FIELD_QUE_SIZE-1];
	       record_qinfo->input=record_qinfo->output=record_qinfo->head;
	       record_qinfo->level=0;
	       shm_que->dma_out_offset[k]= (softc->actual_output_size/4)*k;
	   }
	   rtv_c3_init(softc);
	   ccube_init_state(softc);
}
#endif
#ifdef RTV_SUPPORT

static int
setup_record(tvtwo_softc *softc,SETUP_INFO *record_setup)
{
register int min_q,q_index,qfactor;
register int n;
int *rbuf;

#if 0
	global_send_que[softc->instance] =softc->shm_que; 
#endif
	softc->max_rate= 16*1024;
	softc->frame_rate=30;
	min_q=softc->min_queue= record_setup->qfactor;
	softc->max_rate= record_setup->max_rate;
	if(softc->max_rate>MAX_BANDWIDTH)
		softc->max_rate=MAX_BANDWIDTH;
	softc->frame_rate=record_setup->fps;
	softc->interlace=record_setup->interlace;
	cmn_err(CE_WARN,"!SETUP_RECORD: qf=%d fps=%d rate=%d interlace=%d\n",
			min_q,record_setup->fps,softc->max_rate,
			softc->interlace);
	softc->rate_info.prev_qindex[0]= ccube_q_to_index(min_q);
	softc->rate_info.prev_qindex[1]= ccube_q_to_index(min_q);
	softc->rate_info.prev_rate[0]=
		softc->rate_info.prev_rate[1]=(3*ccube_q_to_rate(min_q))/2;
	q_index=ccube_set_new_q(softc->rate_info.prev_qindex[0],
		softc->rate_info.prev_rate[0],
		softc->max_rate,softc->min_queue);
	softc->rate_info.prev_qindex[0]= q_index;
	softc->rate_info.prev_qindex[1]= q_index;
	/*
	* Convert new q_index to new qfactor.
	*/
	ccube_select_comp_tables(softc,q_index);
	qfactor=softc->field_qfactor= ccube_index_to_q(q_index);
	softc->rate_info.qfactor[0]=qfactor;
	softc->rate_info.qfactor[1]=qfactor;
	softc->field_qindex=q_index;
	softc->decomp_qindex=0;
	softc->compress_qindex=0;
}
static int
set_record_window(tvtwo_softc *softc,START_INFO *rec_info)
{
int wid;
	softc->record_window=wid=rec_info->wid;
	softc->record_start_pixel[0]= rec_info->start_y ;
	softc->record_start_pixel[1]= rec_info->start_y ;
}
static int
set_play_window(tvtwo_softc *softc,START_INFO *play_info)
{
int wid;
	 softc->play_window=wid= play_info->wid;
	 softc->play_wintbl[wid].video_tag=0;
	 softc->play_start_pixel[0]= play_info->start_y;
	 softc->play_start_pixel[1]= play_info->start_y;
}
#endif

tvtwo_ioctl(dev_t dev, int cmd, int arg, int mode, cred_t * cred_p, int *rval_p)
{
register tvtwo_softc *softc = TVTWO_DEV2SOFTC(dev);

#ifdef RTV_SUPPORT
SHM_FIELD_QUE *shm_que = (SHM_FIELD_QUE *)softc->shm_que;
static START_INFO rec_info,play_info;
static SETUP_INFO record_setup,play_setup;
QUEUE_INFO *playback_qinfo;
QUEUE_INFO *record_stream;
#endif
#ifdef RTVNET_SUPPORT
static NET_INFO netinfo;
#endif
register int rval;
int sid;

#ifdef TRACE_RTV_IOCTL
	cmn_err(CE_WARN,"! rtv_ioctl: cmd=0x%x\n",cmd);
#endif
	switch(cmd)
	{
	case GET_INSTANCE:
	     ddi_copyout((caddr_t)&softc->instance,(caddr_t)arg,4,0);
	     rval=0;
	     break;
#ifdef RTV_SUPPORT
	case SETUP_PLAY:
	     ddi_copyin((caddr_t)arg,(caddr_t)&play_setup,sizeof(SETUP_INFO),0);
	     softc->interlace=play_setup.interlace;
	     softc->frame_rate=play_setup.fps;
#if 0
	     global_recv_que[softc->instance] =softc->shm_que;
#endif
	     play_init(softc);
	     reset_queue(softc,0);
	     rval=0;
	     break;
	case START_RECORD:
	     start_record(softc);
	     rval=0;
	     break;
	case PAUSE_RECORD:
	     pause_record(softc);
	     rval=0;
	     break;
	case RESUME_RECORD:
	     resume_record(softc);
	     rval=0;
	     break;
	case STOP_RECORD:
	     stop_record(softc);
	     rval=0;
	     break;
	case START_PLAY:
	     start_play(softc);
	     rval=0;
	     break;
	case PAUSE_PLAY:
	     pause_play(softc,0);
	     rval=0;
	     break;
	case RESUME_PLAY:
	     resume_play(softc);
	     rval=0;
	     break;
	case STOP_PLAY:
	     stop_play(softc,0);
	     /*
	     reset_queue(softc,0);
	     */
	     rval=0;
	     break;
	case SET_RECORD_WINDOW:
	     ddi_copyin((caddr_t)arg,(caddr_t)&rec_info,sizeof(START_INFO),0);
	     set_record_window(softc,&rec_info);
	     rval=0;
	     break;
	case SET_PLAY_WINDOW:
	     ddi_copyin((caddr_t)arg,(caddr_t)&play_info,sizeof(START_INFO),0);
	     set_play_window(softc,&play_info);
	     break;
	case SETUP_RECORD:
	     ddi_copyin((caddr_t)arg,(caddr_t)&record_setup,sizeof(SETUP_INFO),0);
	     setup_record(softc,&record_setup);
	     record_init(softc);
	     rval=0;
	     break;
	 case ENABLE_RTVMODE:
	      tvtwo_rtv_init(softc);
	      rval=0;
	      break;
	 case DISABLE_RTVMODE:
	      if(softc->rtv_dma_state== RECORD && 
		  softc->record_action==RECORD) stop_record(softc);
	      else if(softc->rtv_dma_state==PLAY && 
		      softc->play_action==PLAY) stop_play(softc,0);
	      disable_interrupts(softc);
	     /*
	      (void)ddi_dma_sync(softc->ccube_read_dma,0,
		          softc->record_size,DDI_DMA_SYNC_FORKERNEL);
	  */
	      softc->rtv_dma_state=0;
	      rval=0;
	      break;
	 case RTV_GET_RECORD_LEVEL:
	      sid=0;
	      record_stream= (QUEUE_INFO *)&shm_que->record_streams[sid];
	      ddi_copyout((caddr_t)&record_stream->level,(caddr_t)arg,4,0);
	      rval=0;
	      break;
	 case RTV_GET_FIELD:
	      tvtwo_read_field(softc,(caddr_t)arg);
	      rval=0;
	      break;
	 case RTV_GET_PLAY_LEVEL:
	      sid=0;
	      playback_qinfo= (QUEUE_INFO *)&shm_que->playback_streams[sid];
	      ddi_copyout((caddr_t)&playback_qinfo->level,(caddr_t)arg,4,0);
	      rval=0;
	      break;
	 case RTV_PUT_FIELD:
	      tvtwo_write_field(softc,(caddr_t)arg);
	      rval=0;
	      break;
	 case CCUBE_PRESENT: 
	      if(softc->nvram.main.c3_rev<1)
	      return(-1);
	      rval=0;
	      break;
#ifdef RTVNET_SUPPORT
        case START_TRANSMIT:
	     ddi_copyin((caddr_t)arg,(caddr_t)&netinfo,sizeof(NET_INFO),0);
	     if(netinfo.media==RTV_SCREEN)
	     {
		 softc->send_action[0]=TRANSMIT;
		 cmn_err(CE_NOTE,"! send_action[0] = TRANSMIT\n");
	     }
	     break;
	case START_RECEIVE:
	     ddi_copyin((caddr_t)arg,(caddr_t)&netinfo,sizeof(NET_INFO),0);
	     if(netinfo.media==RTV_SCREEN)
		    softc->recv_action[0]=FEEDBACK;
	     else if(netinfo.media==RTV_DISK)
			softc->recv_action[0]=ACKNOWLEDGE;
	     rval=0;
	     break;
	case STOP_TRANSMIT:
	     ddi_copyin((caddr_t)arg,(caddr_t)&netinfo,sizeof(NET_INFO),0);
	     softc->send_action[0]=0;
	     rval=0;
	     break;
	case SYNC_AUDIO:
	     softc->audio_sync=1;
	     rval=0;
	     break;
#endif
#endif
	 default:
	      rval=tvtwo_ioctl_fb(dev, cmd, arg, mode, cred_p, rval_p);
	      break;
	}
	 return(rval);
}

#ifdef RTV_SUPPORT
record_init(tvtwo_softc *softc)
{
SHM_FIELD_QUE *shm_que = (SHM_FIELD_QUE *)softc->shm_que;
QUEUE_INFO *record_stream;
int sid;

	 softc->field_offset[0]=0;
	 softc->rtv_dma_busy=0;
	 softc->field_polarity=0;
	 softc->ccube_operation=1;
	 softc->record_action=RECORD;
	 softc->ccube_comp_state= DST_IDLE;
	 softc->send_action[0]=0;
	 softc->frame_count=0;
	 softc->msg_count[0]=softc->msg_count[1]=0;
	 shm_que->interlace= softc->interlace;
	 sid=softc->stream_id=0;
	 record_stream= (QUEUE_INFO *)&shm_que->record_streams[sid];
	 record_stream->input=record_stream->output=record_stream->head;
	 record_stream->level=0;
    	 softc->rtv_dma_state=RECORD;
}
play_init(tvtwo_softc *softc)
{
SHM_FIELD_QUE *shm_que = (SHM_FIELD_QUE *)softc->shm_que;
QUEUE_INFO *playback_qinfo;
int sid;
	 softc->field_offset[1]=0;
	 softc->field_polarity=0;
	 softc->start_decomp=0;
	 softc->ccube_comp_state= DST_IDLE;
	 softc->rtv_dma_busy=0;
	 softc->recv_action[0]=0;
	 softc->rtv_sleeping=0;
	 sid=softc->stream_id=0;
	 softc->msg_count[sid] =0;
	 softc->msg_count[1]=0;
	 playback_qinfo= (QUEUE_INFO *)&shm_que->playback_streams[sid];
	 playback_qinfo->input=playback_qinfo->head;
	 playback_qinfo->output=playback_qinfo->tail;
	 playback_qinfo->level=0;
	 shm_que->interlace= softc->interlace;
	 ccube_init_state(softc);
    	 softc->rtv_dma_state=PLAY;
}
start_record(tvtwo_softc *softc)
{
	cmn_err(CE_WARN,"! start_record\n");
	softc->record_action=RECORD;
	softc->shm_que->record_action=RECORD;
	softc->rtv_sleeping=1;
	process_event(softc,START_COMPRESSION);
}
start_play(tvtwo_softc *softc)
{
	/*
	ccube_wchan_mode(softc,PLAY);
	*/
	softc->play_action=PLAY;
	softc->shm_que->play_action=PLAY;
	softc->shm_que->stream_active[0]=1;
	softc->rtv_sleeping=1;
	cmn_err(CE_WARN,"! start_play: rtv_sleeping=%d\n",softc->rtv_sleeping);
	process_event(softc,START_DECOMP);
}
stop_record(tvtwo_softc *softc )
{
int loop;
	softc->record_action=STOP;
	softc->send_action[0]=0;
	softc->shm_que->record_action=STOP;
	softc->tid=timeout(alarm_clock,(caddr_t)softc,200);
        mutex_enter(&softc->rtv_sleep_lock);
	if(softc->rtv_sleeping)
	    cv_wait(&softc->rtv_wake_lock,&softc->rtv_sleep_lock);
        mutex_exit(&softc->rtv_sleep_lock);
	untimeout(softc->tid);
	softc->frame_count=0;
	cmn_err(CE_WARN,"! stop_record\n");
}
stop_play(tvtwo_softc *softc,int wid)
{
	softc->play_action=STOP;
	softc->tid=timeout(alarm_clock,(caddr_t)softc,200);
        mutex_enter(&softc->rtv_sleep_lock);
	if(softc->rtv_sleeping)
	{
	    cmn_err(CE_WARN,"! stop_play: waiting for wake lock broadcast\n");
	    cv_wait(&softc->rtv_wake_lock,&softc->rtv_sleep_lock);
	}
        mutex_exit(&softc->rtv_sleep_lock);
	untimeout(softc->tid);
	softc->recv_action[0]=0;
	softc->shm_que->play_action=STOP;
	cmn_err(CE_WARN,"! stop_play\n");
}
pause_record(tvtwo_softc *softc)
{
	softc->record_action=PAUSE;
	softc->send_action[0]=0;
	softc->shm_que->record_action=PAUSE;
}
resume_record(tvtwo_softc *softc)
{
	softc->record_action=RECORD;
	softc->shm_que->record_action=RECORD;
	softc->send_action[0]=TRANSMIT;
}
pause_play(tvtwo_softc *softc,int wid)
{
	softc->play_action=PAUSE;
	softc->recv_action[0]=0;
	softc->shm_que->play_action=PAUSE;
}
resume_play(tvtwo_softc *softc)
{
	softc->play_action=PLAY;
	softc->shm_que->play_action=PLAY;
	softc->recv_action[0]=FEEDBACK;
	softc->shm_que->stream_active[0]=1;
}
get_queue_level(tvtwo_softc *softc,int wid)
{
SHM_FIELD_QUE *shm_que = (SHM_FIELD_QUE *)softc->shm_que;
QUEUE_INFO *playback_qinfo;
	if(wid < 0 || wid > 3)return;
	playback_qinfo= (QUEUE_INFO *)&shm_que->playback_streams[wid];
	if(playback_qinfo->level<0)
	    return(0);
	else
	    return(playback_qinfo->level);
}
reset_queue(tvtwo_softc *softc,int wid)
{
SHM_FIELD_QUE *shm_que = (SHM_FIELD_QUE *)softc->shm_que;
QUEUE_INFO *playback_qinfo;
	if(wid < 0 || wid > 3)return;
	playback_qinfo= (QUEUE_INFO *)&shm_que->playback_streams[wid];
	playback_qinfo->input=playback_qinfo->output=playback_qinfo->head;
	playback_qinfo->level=0;
}
#endif /* End of Rtv Support */
static u_int
tvtwo_intr(caddr_t i_handler_arg)
{
	register tvtwo_softc *softc = (tvtwo_softc *) i_handler_arg;
	register int serviced = DDI_INTR_UNCLAIMED;
	register u_int ints, bints;
	register int trace=0;
	extern void (*bint_services[])();
	extern void (*int_services[])();
	extern void (*all_services[])();

	if (softc == NULL) {
		return (DDI_INTR_UNCLAIMED);
	}

	mutex_enter(&softc->dma_lock);
	if(softc->rtv_dma_state==0)
	{

	/*
	 * Turn off DMA so we can talk to the board.
	 */
	     tvtwo_pause_dma(softc);

	/*
	 * We must & in the interrupt mask bits because
	 * the masked (disabled) bits may be set even though
	 * they did not generate an interrupt.
	 */
	     softc->istat = XV_RREG(softc->regs, IntStatC.l);
	     softc->bstat = XV_RREG(softc->regs, BIntStatC.l);
	
	/*
	 * Service the BIC interrupts.
	 */
	     bints = XV_RSDW(softc->regs, BIntMask.l) & softc->bstat;
	     if (bints)
		     tvtwo_service(softc, bint_services, bints);

	/*
	 * Service the CTC interrupts.
	 */
	     ints = XV_RSDW(softc->regs, IntMask.l) & softc->istat;
     
	     if (ints)
		     tvtwo_service(softc, int_services, ints);
	
	     {
		volatile int i = XV_RREG(softc->regs, IntStatC.l);/*XXX*/
#ifdef lint
		i = i;
#endif
	     }

	/*
	 * DMA used to be automatically continued by
	 * svc_dma and svc_c3.  We no longer let them.
	 * This is the only place that dma should be restarted,
	 * and we only do so if there is something to process
	 * and no errors have occured.
	 */
	     if ((softc->state != STATE0) && (softc->state != STATE_ERR))
		     tvtwo_continue_dma(softc);

	}
#ifdef RTV_SUPPORT
	else
	{
	    /* The hardware will hold any int's while this register is set to
	     * 0x8000.  We can assume that no int's will occure while we are
	     * in this routine.
	     */
      
	    XV_WREG(softc->regs,IntMask.l, 0x00008000);	/* turn off int's */
	    XV_WREG(softc->regs,BIntMask.l, 0x00000000);/* turn off int's */
	   
	    /* read the Interrupt Status registers & Clear  */

	    softc->bstat= XV_RREG(softc->regs, BIntStatC.l);
	    softc->istat= XV_RREG(softc->regs, IntStatC.l);

	    ints = 0;
	    ints |= (softc->istat & softc->imask & 0xff) << 2;
	    ints |= (softc->bstat & softc->bmask & 0x03) ;

	    if(ints)
	    {
		softc->interrupt_bits=ints;
		tvtwo_service(softc, all_services, ints);
	    }

	    XV_WREG(softc->regs,IntMask.l, softc->imask);/* turn on int's */
	    XV_WREG(softc->regs,BIntMask.l, softc->bmask);/* turn on int's */

	}
#endif
	serviced = DDI_INTR_CLAIMED;

	mutex_exit(&softc->dma_lock);

	return (serviced);
}

